#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Editors;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   Simple color palette class for choosing colors.  The palette can be rendered to any container.<br />
    ///   Here's an example of typical usage:
    ///   <pre><code>
    ///          var cp = new Ext.ColorPalette({value:'993300'});  // initial selected color
    ///          cp.render('my-div');
    ///          cp.on('select', function(palette, selColor){
    ///          // do something with selColor
    ///          });
    ///        </code></pre>
    /// </summary>
    public class ColorPalette : BaseComponent
    {


        /// <summary>
        ///   An existing XTemplate instance to be used in place of the default template for rendering the component.
        /// </summary>
        [JsonProperty( "tpl" )]
        public string Tpl { get; set; }

        /// <summary>
        ///   The CSS class to apply to the containing element (defaults to 'x-color-palette')
        /// </summary>
        [JsonProperty( "itemCls" )]
        public override string ItemCls { get; set; }

        /// <summary>
        ///   The initial color to highlight (should be a valid 6-digit color hex code without the # symbol).  Note that
        ///   the hex codes are case-sensitive.
        /// </summary>
        [JsonProperty( "value" )]
        public string Value { get; set; }

        /// <summary>
        ///   The DOM event that will cause a color to be selected. This can be any valid event name (dblclick, contextmenu).
        ///   Defaults to <tt>'click'</tt>.
        /// </summary>
        [JsonProperty( "clickEvent" )]
        public string ClickEvent { get; set; }

        /// <summary>
        ///   If set to true then reselecting a color that is already selected fires the {@link #select} event
        /// </summary>
        [JsonProperty( "allowReselect" )]
        public bool AllowReselect { get; set; }

        /// <summary>
        ///   <p>An array of 6-digit color hex code strings (without the # symbol).  This array can contain any number
        ///     of colors, and each hex code should be unique.  The width of the palette is controlled via CSS by adjusting
        ///     the width property of the 'x-color-palette' class (or assigning a custom class), so you can balance the number
        ///     of colors with the width setting until the box is symmetrical.</p>
        ///   <p>You can override individual colors if needed:</p>
        ///   <pre><code>
        ///          var cp = new Ext.ColorPalette();
        ///          cp.colors[0] = 'FF0000';  // change the first box to red
        ///        </code></pre>
        ///   Or you can provide a custom array of your own for complete control:
        ///   <pre><code>
        ///          var cp = new Ext.ColorPalette();
        ///          cp.colors = ['000000', '993300', '333300'];
        ///        </code></pre>
        /// </summary>
        [JsonProperty( "colors" )]
        public IList Colors { get; set; }

        /// <summary>
        ///   Optional. A function that will handle the select event of this palette.
        ///   The handler is passed the following parameters:<div class = "mdetail-params"><ul>
        ///                                                                                  <li><code>palette</code> : ColorPalette<div class = "sub-desc">The {@link #palette Ext.ColorPalette}.</div></li>
        ///                                                                                  <li><code>color</code> : String<div class = "sub-desc">The 6-digit color hex code (without the # symbol).</div></li>
        ///                                                                                </ul></div>
        /// </summary>
        [JsonProperty( "handler" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        public string Handler { get; set; }

        /// <summary>
        ///   The scope (<tt><b>this</b></tt> reference) in which the <code>{@link #handler}</code>
        ///   function will be called.  Defaults to this ColorPalette instance.
        /// </summary>
        [JsonProperty( "scope" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string Scope { get; set; }
    }
}
