#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Editors;
using NetJsWire.Lite.Widgets.UI.Data;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   A mechanism for displaying data using custom layout templates and formatting. DataView uses an {@link Ext.XTemplate}
    ///   as its internal templating mechanism, and is bound to an {@link Ext.data.Store}
    ///   so that as the data in the store changes the view is automatically updated to reflect the changes.  The view also
    ///   provides built-in behavior for many common events that can occur for its contained items including click, doubleclick,
    ///   mouseover, mouseout, etc. as well as a built-in selection model. <b>In order to use these features, an {@link #itemSelector}
    ///                                                                      config must be provided for the DataView to determine what nodes it will be working with.</b>
    ///   <p>The example below binds a DataView to a {@link Ext.data.Store} and renders it into an {@link Ext.Panel}.</p>
    ///   <pre><code>
    ///          var store = new Ext.data.JsonStore({
    ///          url: 'get-images.php',
    ///          root: 'images',
    ///          fields: [
    ///          'name', 'url',
    ///          {name:'size', type: 'float'},
    ///          {name:'lastmod', type:'date', dateFormat:'timestamp'}
    ///          ]
    ///          });
    ///          store.load();
    ///          var tpl = new Ext.XTemplate(
    ///          '&lt;tpl for="."&gt;',
    ///          '&lt;div class="thumb-wrap" id="{name}"&gt;',
    ///          '&lt;div class="thumb"&gt;&lt;img src="{url}" title="{name}"&gt;&lt;/div&gt;',
    ///          '&lt;span class="x-editable"&gt;{shortName}&lt;/span&gt;&lt;/div&gt;',
    ///          '&lt;/tpl&gt;',
    ///          '&lt;div class="x-clear"&gt;&lt;/div&gt;'
    ///          );
    ///          var panel = new Ext.Panel({
    ///          id:'images-view',
    ///          frame:true,
    ///          width:535,
    ///          autoHeight:true,
    ///          collapsible:true,
    ///          layout:'fit',
    ///          title:'Simple DataView',
    ///          items: new Ext.DataView({
    ///          store: store,
    ///          tpl: tpl,
    ///          autoHeight:true,
    ///          multiSelect: true,
    ///          overClass:'x-view-over',
    ///          itemSelector:'div.thumb-wrap',
    ///          emptyText: 'No images to display'
    ///          })
    ///          });
    ///          panel.render(document.body);
    ///        </code></pre>
    /// </summary>
    public class DataView : BoxComponent
    {
        /// <summary>
        ///   Create a new DataView
        ///   @xtype dataview
        /// </summary>
        /// <returns></returns>
        public DataView() : this( null ) { }

        public DataView( IContainer container )
            : base( container )
        {
            base.Xtype = "dataview";
        }

        /// <summary>
        ///   The HTML fragment or an array of fragments that will make up the template used by this DataView.  This should
        ///   be specified in the same format expected by the constructor of {@link Ext.XTemplate}.
        /// </summary>
        [JsonProperty( "tpl" )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [DefaultValue( ( string ) null )]
        public string Tpl { get; set; }

        /// <summary>
        ///   The {@link Ext.data.Store} to bind this DataView to.
        /// </summary>
        [JsonProperty( "store" )]
        [DefaultValue( null )]
        public Store Store { get; set; }

        /// <summary>
        ///   <b>This is a required setting</b>. A simple CSS selector (e.g. <tt>div.some-class</tt> or
        ///   <tt>span:first-child</tt>) that will be used to determine what nodes this DataView will be
        ///   working with.
        /// </summary>
        [JsonProperty( "itemSelector" )]
        [DefaultValue( ( string ) null )]
        public string ItemSelector { get; set; }

        /// <summary>
        ///   True to allow selection of more than one item at a time, false to allow selection of only a single item
        ///   at a time or no selection at all, depending on the value of {@link #singleSelect} (defaults to false).
        /// </summary>
        [JsonProperty( "multiSelect" )]
        [DefaultValue( false )]
        public bool MultiSelect { get; set; }

        /// <summary>
        ///   True to allow selection of exactly one item at a time, false to allow no selection at all (defaults to false).
        ///   Note that if {@link #multiSelect} = true, this value will be ignored.
        /// </summary>
        [JsonProperty( "singleSelect" )]
        [DefaultValue( false )]
        public bool SingleSelect { get; set; }

        /// <summary>
        ///   True to enable multiselection by clicking on multiple items without requiring the user to hold Shift or Ctrl,
        ///   false to force the user to hold Ctrl or Shift to select more than on item (defaults to false).
        /// </summary>
        [JsonProperty( "simpleSelect" )]
        [DefaultValue( false )]
        public bool SimpleSelect { get; set; }

        /// <summary>
        ///   A CSS class to apply to each item in the view on mouseover (defaults to undefined).
        /// </summary>
        [JsonProperty( "overClass" )]
        [DefaultValue( ( string ) null )]
        public string OverClass { get; set; }

        /// <summary>
        ///   A string to display during data load operations (defaults to undefined).  If specified, this text will be
        ///   displayed in a loading div and the view's contents will be cleared while loading, otherwise the view's
        ///   contents will continue to display normally until the new data is loaded and the contents are replaced.
        /// </summary>
        [JsonProperty( "loadingText" )]
        [DefaultValue( ( string ) null )]
        public string LoadingText { get; set; }

        /// <summary>
        ///   A CSS class to apply to each selected item in the view (defaults to 'x-view-selected').
        /// </summary>
        [JsonProperty( "selectedClass" )]
        [DefaultValue( ( string ) null )]
        public string SelectedClass { get; set; }

        /// <summary>
        ///   The text to display in the view when there is no data to display (defaults to '').
        /// </summary>
        [JsonProperty( "emptyText" )]
        [DefaultValue( ( string ) null )]
        public string EmptyText { get; set; }

        /// <summary>
        ///   True to defer emptyText being applied until the store's first load
        /// </summary>
        [JsonProperty( "deferEmptyText" )]
        [DefaultValue( false )]
        public bool DeferEmptyText { get; set; }

        /// <summary>
        ///   True to enable mouseenter and mouseleave events
        /// </summary>
        [JsonProperty( "trackOver" )]
        [DefaultValue( false )]
        public bool TrackOver { get; set; }

        /// <summary>
        ///   Set this to true to ignore datachanged events on the bound store. This is useful ifyou wish to provide custom transition animations via a plugin (defaults to false)
        /// </summary>
        [JsonProperty( "blockRefresh" )]
        [DefaultValue( false )]
        public bool BlockRefresh { get; set; }
    }
}
