#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Editors;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   <p>A popup date picker. This class is used by the {@link Ext.form.DateField DateField} class
    ///     to allow browsing and selection of valid dates.</p>
    ///   <p>All the string values documented below may be overridden by including an Ext locale file in
    ///     your page.</p>
    /// </summary>
    public class DatePicker : BaseComponent
    {


        /// <summary>
        ///   The text to display on the button that selects the current date (defaults to <code>'Today'</code>)
        /// </summary>
        [JsonProperty( "todayText" )]
        [DefaultValue( ( string ) null )]
        public string TodayText { get; set; }

        /// <summary>
        ///   The text to display on the ok button (defaults to <code>'&#160;OK&#160;'</code> to give the user extra clicking room)
        /// </summary>
        [JsonProperty( "okText" )]
        [DefaultValue( ( string ) null )]
        public string OkText { get; set; }

        /// <summary>
        ///   The text to display on the cancel button (defaults to <code>'Cancel'</code>)
        /// </summary>
        [JsonProperty( "cancelText" )]
        [DefaultValue( ( string ) null )]
        public string CancelText { get; set; }

        /// <summary>
        ///   Optional. A function that will handle the select event of this picker.
        ///   The handler is passed the following parameters:<div class = "mdetail-params"><ul>
        ///                                                                                  <li><code>picker</code> : DatePicker<div class = "sub-desc">This DatePicker.</div></li>
        ///                                                                                  <li><code>date</code> : Date<div class = "sub-desc">The selected date.</div></li>
        ///                                                                                </ul></div>
        /// </summary>
        [JsonProperty( "handler" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [DefaultValue( ( string ) null )]
        public string Handler { get; set; }

        /// <summary>
        ///   The scope (<code><b>this</b></code> reference) in which the <code>{@link #handler}</code>
        ///   function will be called.  Defaults to this DatePicker instance.
        /// </summary>
        [JsonProperty( "scope" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [DefaultValue( ( string ) null )]
        public string Scope { get; set; }

        /// <summary>
        ///   A string used to format the message for displaying in a tooltip over the button that
        ///   selects the current date. Defaults to <code>'{0} (Spacebar)'</code> where
        ///   the <code>{0}</code> token is replaced by today's date.
        /// </summary>
        [JsonProperty( "todayTip" )]
        [DefaultValue( ( string ) null )]
        public string TodayTip { get; set; }

        /// <summary>
        ///   The error text to display if the minDate validation fails (defaults to <code>'This date is before the minimum date'</code>)
        /// </summary>
        [JsonProperty( "minText" )]
        [DefaultValue( ( string ) null )]
        public string MinText { get; set; }

        /// <summary>
        ///   The error text to display if the maxDate validation fails (defaults to <code>'This date is after the maximum date'</code>)
        /// </summary>
        [JsonProperty( "maxText" )]
        [DefaultValue( ( string ) null )]
        public string MaxText { get; set; }

        /// <summary>
        ///   The default date format string which can be overriden for localization support.  The format must be
        ///   valid according to {@link Date#parseDate} (defaults to <code>'m/d/y'</code>).
        /// </summary>
        [JsonProperty( "format" )]
        [DefaultValue( ( string ) null )]
        public string Format { get; set; }

        /// <summary>
        ///   The tooltip to display when the date falls on a disabled day (defaults to <code>'Disabled'</code>)
        /// </summary>
        [JsonProperty( "disabledDaysText" )]
        [DefaultValue( ( string ) null )]
        public string DisabledDaysText { get; set; }

        /// <summary>
        ///   The tooltip text to display when the date falls on a disabled date (defaults to <code>'Disabled'</code>)
        /// </summary>
        [JsonProperty( "disabledDatesText" )]
        [DefaultValue( ( string ) null )]
        public string DisabledDatesText { get; set; }

        /// <summary>
        ///   An array of textual month names which can be overriden for localization support (defaults to Date.monthNames)
        /// </summary>
        [JsonProperty( "monthNames" )]
        [DefaultValue( ( string ) null )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string MonthNames { get; set; }

        /// <summary>
        ///   An array of textual day names which can be overriden for localization support (defaults to Date.dayNames)
        /// </summary>
        [JsonProperty( "dayNames" )]
        [DefaultValue( ( string ) null )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string DayNames { get; set; }

        /// <summary>
        ///   The next month navigation button tooltip (defaults to <code>'Next Month (Control+Right)'</code>)
        /// </summary>
        [JsonProperty( "nextText" )]
        [DefaultValue( ( string ) null )]
        public string NextText { get; set; }

        /// <summary>
        ///   The previous month navigation button tooltip (defaults to <code>'Previous Month (Control+Left)'</code>)
        /// </summary>
        [JsonProperty( "prevText" )]
        [DefaultValue( ( string ) null )]
        public string PrevText { get; set; }

        /// <summary>
        ///   The header month selector tooltip (defaults to <code>'Choose a month (Control+Up/Down to move years)'</code>)
        /// </summary>
        [JsonProperty( "monthYearText" )]
        [DefaultValue( ( string ) null )]
        public string MonthYearText { get; set; }

        /// <summary>
        ///   Day index at which the week should begin, 0-based (defaults to 0, which is Sunday)
        /// </summary>
        [JsonProperty( "startDay" )]
        [DefaultValue( ( string ) null )]
        public double StartDay { get; set; }

        /// <summary>
        ///   False to hide the footer area containing the Today button and disable the keyboard handler for spacebar
        ///   that selects the current date (defaults to <code>true</code>).
        /// </summary>
        [JsonProperty( "showToday" )]
        [DefaultValue( ( string ) null )]
        public bool ShowToday { get; set; }

        /// <summary>
        ///   Minimum allowable date (JavaScript date object, defaults to null)
        /// </summary>
        [JsonProperty( "minDate" )]
        [DefaultValue( null )]
        public DateTime? MinDate { get; set; }

        /// <summary>
        ///   Maximum allowable date (JavaScript date object, defaults to null)
        /// </summary>
        [JsonProperty( "maxDate" )]
        [DefaultValue( null )]
        public DateTime? MaxDate { get; set; }

        /// <summary>
        ///   An array of days to disable, 0-based. For example, [0, 6] disables Sunday and Saturday (defaults to null).
        /// </summary>
        [JsonProperty( "disabledDays" )]
        public IList DisabledDays { get; set; }

        /// <summary>
        ///   JavaScript regular expression used to disable a pattern of dates (defaults to null).  The {@link #disabledDates}
        ///   config will generate this regex internally, but if you specify disabledDatesRE it will take precedence over the
        ///   disabledDates value.
        /// </summary>
        [JsonProperty( "disabledDatesRE" )]
        [DefaultValue( ( string ) null )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string DisabledDatesRe { get; set; }

        /// <summary>
        ///   An array of 'dates' to disable, as strings. These strings will be used to build a dynamic regular
        ///   expression so they are very powerful. Some examples:
        ///   <ul>
        ///     <li>['03/08/2003', '09/16/2003'] would disable those exact dates</li>
        ///     <li>['03/08', '09/16'] would disable those days for every year</li>
        ///     <li>['^03/08'] would only match the beginning (useful if you are using short years)</li>
        ///     <li>['03/../2006'] would disable every day in March 2006</li>
        ///     <li>['^03'] would disable every day in every March</li>
        ///   </ul>
        ///   Note that the format of the dates included in the array should exactly match the {@link #format} config.
        ///   In order to support regular expressions, if you are using a date format that has '.' in it, you will have to
        ///   escape the dot when restricting dates. For example: ['03\\.08\\.03'].
        /// </summary>
        [JsonProperty( "disabledDates" )]
        [DefaultValue( ( string ) null )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string DisabledDates { get; set; }
    }
}
