#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.Windows.Forms;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   Defines the interface and base operation of items that that can be
    ///   dragged or can be drop targets.  It was designed to be extended, overriding
    ///   the event handlers for startDrag, onDrag, onDragOver and onDragOut.
    ///   Up to three html elements can be associated with a DragDrop instance:
    ///   <ul>
    ///     <li>linked element: the element that is passed into the constructor.
    ///       This is the element which defines the boundaries for interaction with
    ///       other DragDrop objects.</li>
    ///     <li>handle element(s): The drag operation only occurs if the element that
    ///       was clicked matches a handle element.  By default this is the linked
    ///       element, but there are times that you will want only a portion of the
    ///       linked element to initiate the drag operation, and the setHandleElId()
    ///       method provides a way to define this.</li>
    ///     <li>drag element: this represents the element that would be moved along
    ///       with the cursor during a drag operation.  By default, this is the linked
    ///       element itself as in {@link Ext.dd.DD}.  setDragElId() lets you define
    ///       a separate element that would be moved, as in {@link Ext.dd.DDProxy}.
    ///     </li>
    ///   </ul>
    ///   This class should not be instantiated until the onload event to ensure that
    ///   the associated elements are available.
    ///   The following would define a DragDrop obj that would interact with any
    ///   other DragDrop obj in the "group1" group:
    ///   <pre>
    ///     dd = new Ext.dd.DragDrop("div1", "group1");
    ///   </pre>
    ///   Since none of the event handlers have been implemented, nothing would
    ///   actually happen if you were to run the code above.  Normally you would
    ///   override this class or one of the default implementations, but you can
    ///   also override the methods you want on an instance of the class...
    ///   <pre>
    ///     dd.onDragDrop = function(e, id) {
    ///     &nbsp;&nbsp;alert("dd was dropped on " + id);
    ///     }
    ///   </pre>
    /// </summary>
    public class DragDrop
    {
        /// <summary>
        ///   Set to false to enable a DragDrop object to fire drag events while dragging
        ///   over its own Element. Defaults to true - DragDrop objects do not by default
        ///   fire drag events to themselves.
        /// </summary>
        [JsonProperty( "ignoreSelf" )]
        public bool IgnoreSelf { get; set; }

        /// <summary>
        ///   The id of the element associated with this object.  This is what we
        ///   refer to as the "linked element" because the size and position of
        ///   this element is used to determine when the drag and drop objects have
        ///   interacted.
        /// </summary>
        [JsonProperty( "id" )]
        public string Id { get; set; }

        /// <summary>
        ///   An object who's property names identify HTML tags to be considered invalid as drag handles.
        ///   A non-null property value identifies the tag as invalid. Defaults to the
        ///   following value which prevents drag operations from being initiated by &lt;a> elements:<pre><code>
        ///                                                                                                 {
        ///                                                                                                 A: "A"
        ///                                                                                                 }</code></pre>
        /// </summary>
        [JsonProperty( "invalidHandleTypes" )]
        public object InvalidHandleTypes { get; set; }

        /// <summary>
        ///   An object who's property names identify the IDs of elements to be considered invalid as drag handles.
        ///   A non-null property value identifies the ID as invalid. For example, to prevent
        ///   dragging from being initiated on element ID "foo", use:<pre><code>
        ///                                                                 {
        ///                                                                 foo: true
        ///                                                                 }</code></pre>
        /// </summary>
        [JsonProperty( "invalidHandleIds" )]
        public object InvalidHandleIds { get; set; }

        /// <summary>
        ///   An Array of CSS class names for elements to be considered in valid as drag handles.
        /// </summary>
        [JsonProperty( "invalidHandleClasses" )]
        public IList InvalidHandleClasses { get; set; }

        /// <summary>
        ///   The group defines a logical collection of DragDrop objects that are
        ///   related.  Instances only get events when interacting with other
        ///   DragDrop object in the same group.  This lets us define multiple
        ///   groups using a single DragDrop subclass if we want.
        /// </summary>
        [JsonProperty( "groups" )]
        public object Groups { get; set; }

        /// <summary>
        ///   When set to true, other DD objects in cooperating DDGroups do not receive
        ///   notification events when this DD object is dragged over them. Defaults to false.
        /// </summary>
        [JsonProperty( "moveOnly" )]
        public bool MoveOnly { get; set; }

        /// <summary>
        ///   By default, all instances can be a drop target.  This can be disabled by
        ///   setting isTarget to false.
        /// </summary>
        [JsonProperty( "isTarget" )]
        public bool IsTarget { get; set; }

        /// <summary>
        ///   The padding configured for this drag and drop object for calculating
        ///   the drop zone intersection with this object.
        /// </summary>
        [JsonProperty( "padding" )]
        public Padding Padding { get; set; }

        /// <summary>
        ///   Maintain offsets when we resetconstraints.  Set to true when you want
        ///   the position of the element relative to its parent to stay the same
        ///   when the page changes
        /// </summary>
        [JsonProperty( "maintainOffset" )]
        public bool MaintainOffset { get; set; }

        /// <summary>
        ///   Array of pixel locations the element will snap to if we specified a
        ///   horizontal graduation/interval.  This array is generated automatically
        ///   when you define a tick interval.
        /// </summary>
        [JsonProperty( "xTicks" )]
        public int[] XTicks { get; set; }

        /// <summary>
        ///   Array of pixel locations the element will snap to if we specified a
        ///   vertical graduation/interval.  This array is generated automatically
        ///   when you define a tick interval.
        /// </summary>
        [JsonProperty( "yTicks" )]
        public int[] YTicks { get; set; }

        /// <summary>
        ///   By default the drag and drop instance will only respond to the primary
        ///   button click (left button for a right-handed mouse).  Set to true to
        ///   allow drag and drop to start with any mouse click that is propogated
        ///   by the browser
        /// </summary>
        [JsonProperty( "primaryButtonOnly" )]
        public bool PrimaryButtonOnly { get; set; }

        /// <summary>
        ///   The available property is false until the linked dom element is accessible.
        /// </summary>
        [JsonProperty( "available" )]
        public bool Available { get; set; }

        /// <summary>
        ///   By default, drags can only be initiated if the mousedown occurs in the
        ///   region the linked element is.  This is done in part to work around a
        ///   bug in some browsers that mis-report the mousedown if the previous
        ///   mouseup happened outside of the window.  This property is set to true
        ///   if outer handles are defined.
        ///   @default false
        /// </summary>
        [JsonProperty( "hasOuterHandles" )]
        public bool HasOuterHandles { get; set; }

        /// <summary>
        ///   Provides default constraint padding to "constrainTo" elements (defaults to {left: 0, right:0, top:0, bottom:0}).
        /// </summary>
        [JsonProperty( "defaultPadding" )]
        public Padding DefaultPadding { get; set; }
    }
}
