#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Dd
{
    /// <summary>
    ///   /**
    ///   <p>This class provides a container DD instance that allows dragging of multiple child source nodes.</p>
    ///   <p>This class does not move the drag target nodes, but a proxy element which may contain
    ///     any DOM structure you wish. The DOM element to show in the proxy is provided by either a
    ///     provided implementation of {@link #getDragData}, or by registered draggables registered with {@link Ext.dd.Registry}</p>
    ///   <p>If you wish to provide draggability for an arbitrary number of DOM nodes, each of which represent some
    ///     application object (For example nodes in a {@link Ext.DataView DataView}) then use of this class
    ///     is the most efficient way to "activate" those nodes.</p>
    ///   <p>By default, this class requires that draggable child nodes are registered with {@link Ext.dd.Registry}.
    ///     However a simpler way to allow a DragZone to manage any number of draggable elements is to configure
    ///     the DragZone with  an implementation of the {@link #getDragData} method which interrogates the passed
    ///     mouse event to see if it has taken place within an element, or class of elements. This is easily done
    ///     by using the event's {@link Ext.EventObject#getTarget getTarget} method to identify a node based on a
    ///     {@link Ext.DomQuery} selector. For example, to make the nodes of a DataView draggable, use the following
    ///     technique. Knowledge of the use of the DataView is required:</p><pre><code>
    ///                                                                            myDataView.on('render', function(v) {
    ///                                                                            myDataView.dragZone = new Ext.dd.DragZone(v.getEl(), {
    ///                                                                            //      On receipt of a mousedown event, see if it is within a DataView node.
    ///                                                                            //      Return a drag data object if so.
    ///                                                                            getDragData: function(e) {
    ///                                                                            //          Use the DataView's own itemSelector (a mandatory property) to
    ///                                                                            //          test if the mousedown is within one of the DataView's nodes.
    ///                                                                            var sourceEl = e.getTarget(v.itemSelector, 10);
    ///                                                                            //          If the mousedown is within a DataView node, clone the node to produce
    ///                                                                            //          a ddel element for use by the drag proxy. Also add application data
    ///                                                                            //          to the returned data object.
    ///                                                                            if (sourceEl) {
    ///                                                                            d = sourceEl.cloneNode(true);
    ///                                                                            d.id = Ext.id();
    ///                                                                            return {
    ///                                                                            ddel: d,
    ///                                                                            sourceEl: sourceEl,
    ///                                                                            repairXY: Ext.fly(sourceEl).getXY(),
    ///                                                                            sourceStore: v.store,
    ///                                                                            draggedRecord: v.{@link Ext.DataView#getRecord getRecord}(sourceEl)
    ///                                                                            }
    ///                                                                            }
    ///                                                                            },
    ///                                                                            //      Provide coordinates for the proxy to slide back to on failed drag.
    ///                                                                            //      This is the original XY coordinates of the draggable element captured
    ///                                                                            //      in the getDragData method.
    ///                                                                            getRepairXY: function() {
    ///                                                                            return this.dragData.repairXY;
    ///                                                                            }
    ///                                                                            });
    ///                                                                            });</code></pre>
    ///   See the {@link Ext.dd.DropZone DropZone} documentation for details about building a DropZone which
    ///   cooperates with this DragZone.
    /// </summary>
    public class DragZone : DragSource
    {
        /// <summary>
        ///   True to register this container with the Scrollmanagerfor auto scrolling during drag operations.
        /// </summary>
        [JsonProperty( "containerScroll" )]
        public bool ContainerScroll { get; set; }

        /// <summary>
        ///   The color to use when visually highlighting the drag source in the afterRepairmethod after a failed drop (defaults to "c3daf9" - light blue)
        /// </summary>
        [JsonProperty( "hlColor" )]
        public string HlColor { get; set; }
    }
}
