#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Form
{
    /// <summary>
    ///   /**
    ///   <p>A grouping container for {@link Ext.form.Checkbox} controls.</p>
    ///   <p>Sample usage:</p>
    ///   <pre><code>
    ///          var myCheckboxGroup = new Ext.form.CheckboxGroup({
    ///          id:'myGroup',
    ///          xtype: 'checkboxgroup',
    ///          fieldLabel: 'Single Column',
    ///          itemCls: 'x-check-group-alt',
    ///          // Put all controls in a single column with width 100%
    ///          columns: 1,
    ///          items: [
    ///          {boxLabel: 'Item 1', name: 'cb-col-1'},
    ///          {boxLabel: 'Item 2', name: 'cb-col-2', checked: true},
    ///          {boxLabel: 'Item 3', name: 'cb-col-3'}
    ///          ]
    ///          });
    ///        </code></pre>
    /// </summary>
    public class CheckboxGroup : FieldLW
    {

        /// <summary>
        ///   An Array of {@link Ext.form.Checkbox Checkbox}es or Checkbox config objectsto arrange in the group.
        /// </summary>
        [JsonProperty( "items" )]
        public IList Items { get; set; }

        /// <summary>
        ///   Specifies the number of columns to use when displaying groupedcheckbox/radio controls using automatic layout.  This config can take several types of values:
        ///   <ul><li><b>'auto'</b> : <p class = "sub-desc">The controls will be rendered one per column on one row and the width
        ///                             of each column will be evenly distributed based on the width of the overall field container. This is the default.</p></li>
        ///     <li><b>Number</b> : <p class = "sub-desc">If you specific a number (e.g., 3) that number of columns will be
        ///                           created and the contained controls will be automatically distributed based on the value of {@link #vertical}.</p></li>
        ///     <li><b>Array</b> : Object<p class = "sub-desc">You can also specify an array of column widths, mixing integer
        ///                                (fixed width) and float (percentage width) values as needed (e.g., [100, .25, .75]). Any integer values will
        ///                                be rendered first, then any float values will be calculated as a percentage of the remaining space. Float
        ///                                values do not have to add up to 1 (100%) although if you want the controls to take up the entire field
        ///                                container you should do so.</p></li></ul>
        /// </summary>
        [JsonProperty( "columns" )]
        public object Columns { get; set; }

        /// <summary>
        ///   True to distribute contained controls across columns, completely filling each columntop to bottom before starting on the next column.  The number of controls in each column will be automatically
        ///   calculated to keep columns as even as possible.  The default value is false, so that controls will be added
        ///   to columns one at a time, completely filling each row left to right before starting on the next row.
        /// </summary>
        [JsonProperty( "vertical" )]
        public bool Vertical { get; set; }

        /// <summary>
        ///   False to validate that at least one item in the group is checked (defaults to true).If no items are selected at validation time, {@link @blankText} will be used as the error text.
        /// </summary>
        [JsonProperty( "allowBlank" )]
        public bool AllowBlank { get; set; }

        /// <summary>
        ///   Error text to display if the {@link #allowBlank} validation fails (defaults to "You mustselect at least one item in this group")
        /// </summary>
        [JsonProperty( "blankText" )]
        public string BlankText { get; set; }
    }
}
