﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Layout.Config
{
    public class FormLayoutConfig : AnchorLayoutConfig
    {

        public FormLayoutConfig()
        {
            TrackLabels = true;
        }

        /// <summary>
        ///   A compiled Ext.Template for rendering the fully wrapped, labeled and styled form Field. Defaults to:
        ///   <code>
        ///     new Ext.Template(
        ///     '<div class = "x-form-item {itemCls}" tabIndex = "-1">',
        ///        '<label for = "{id}" style = "{labelStyle}" class = "x-form-item-label">{label}{labelSeparator}</label>',
        ///        '<div class = "x-form-element" id = "x-form-el-{id}" style = "{elementStyle}">',
        ///           '</div><div class = "{clearCls}"></div>',
        ///        '</div>'
        ///     );
        ///   </code>
        ///   This may be specified to produce a different DOM structure when rendering form Fields.
        ///   <list type = "bullet">
        ///     <listheader>
        ///       A description of the properties within the template follows:
        ///     </listheader>
        ///     <item>
        ///       itemCls : String
        ///       The CSS class applied to the outermost div wrapper that contains this field label and field element (the default class is 'x-form-item' and itemCls will be added to that). If supplied, itemCls at the field level will override the default itemCls supplied at the container level.
        ///     </item>
        ///     <item>
        ///       id : String
        ///       The id of the Field
        ///     </item>
        ///     <item>
        ///       labelStyle : String
        ///       A CSS style specification string to add to the field label for this field (defaults to '' or the layout's value for labelStyle).
        ///     </item>
        ///     <item>
        ///       label : String
        ///       The text to display as the label for this field (defaults to '')
        ///     </item>
        ///     <item>
        ///       labelSeparator : String
        ///       The separator to display after the text of the label for this field (defaults to a colon ':' or the layout's value for labelSeparator). To hide the separator use empty string ''.
        ///     </item>
        ///     <item>
        ///       elementStyle : String
        ///       The styles text for the input element's wrapper.
        ///     </item>
        ///     <item>
        ///       clearCls : String
        ///       The CSS class to apply to the special clearing div rendered directly after each form field wrapper (defaults to 'x-form-clear-left')
        ///     </item>
        ///   </list>
        /// </summary>
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( MultilineStringEditor ), typeof( UITypeEditor ) )]
        [JsonProperty( "fieldTpl", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [Description( "A compiled Ext.Template for rendering the fully wrapped, labeled and styled form Field." )]
        [DefaultValue( ( string ) null )]
        public string FieldTpl { get; set; }

        [JsonProperty( "labelSeparator", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [Description( "Configuration of this property at the container level takes precedence." )]
        [DefaultValue( ( string ) null )]
        public string LabelSeparator { get; set; }

        [JsonProperty( "trackLabels", DefaultValueHandling = DefaultValueHandling.Ignore )]
        [Description( "True to show/hide the field label when the field is hidden." )]
        [DefaultValue( true )]
        public bool TrackLabels { get; set; }

        public override bool ShouldSerialize()
        {
            return base.ShouldSerialize() || !string.IsNullOrWhiteSpace( FieldTpl ) || !string.IsNullOrEmpty( LabelSeparator ) || !TrackLabels;
        }
    }
}
