#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing.Design;
using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Editors;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   /**
    ///   The base class for all items that render into menus.  BaseItem provides default rendering, activated state
    ///   management and base configuration options shared by all menu components.
    /// </summary>
    [ToolboxItem( false )]
    [DesignTimeVisible( false )]
    public class BaseItem : BaseComponent
    {
        private int _clickHideDelay = 1;
        private bool _hideOnClick = true;

        protected BaseItem()
            : this( null )
        {
        }

        protected BaseItem( IContainer container )
        {
            if ( container != null )
                container.Add( this );
        }


        /// <summary>
        ///   A function that will handle the click event of this menu item (optional).
        ///   The handler is passed the following parameters:
        ///   <div class = "mdetail-params">
        ///     <ul>
        ///       <li><code>b</code> : Item<div class = "sub-desc">This menu Item.</div></li>
        ///       <li><code>e</code> : EventObject<div class = "sub-desc">The click event.</div></li>
        ///     </ul>
        ///   </div>
        /// </summary>
        [JsonProperty( "handler" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [DefaultValue( ( string ) null )]
        public string Handler { get; set; }

        /// <summary>
        ///   The scope (<tt><b>this</b></tt> reference) in which the handler function will be called.
        /// </summary>
        [JsonProperty( "scope" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        [DefaultValue( ( string ) null )]
        public string Scope { get; set; }

        /// <summary>
        ///   True if this item can be visually activated (defaults to false)
        /// </summary>
        [JsonProperty( "canActivate" )]
        [DefaultValue( false )]
        public bool CanActivate { get; set; }

        /// <summary>
        ///   The CSS class to use when the item becomes activated (defaults to "x-menu-item-active")
        /// </summary>
        [JsonProperty( "activeClass" )]
        [DefaultValue( ( string ) null )]
        public string ActiveClass { get; set; }

        /// <summary>
        ///   True to hide the containing menu after this item is clicked (defaults to true)
        /// </summary>
        [JsonProperty( "hideOnClick" )]
        [DefaultValue( true )]
        public virtual bool HideOnClick
        {
            get { return _hideOnClick; }
            set { _hideOnClick = value; }
        }

        /// <summary>
        ///   Length of time in milliseconds to wait before hiding after a click (defaults to 1)
        /// </summary>
        [JsonProperty( "clickHideDelay" )]
        [DefaultValue( 1 )]
        public int ClickHideDelay
        {
            get { return _clickHideDelay; }
            set { _clickHideDelay = value; }
        }

        /// <summary>
        ///   The text to display for this item (defaults to '')
        /// </summary>
        [JsonProperty( "text", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        [Browsable( true )]
        [DefaultValue( ( string ) null )]
        public override string Text { get; set; }

        [DefaultValue( "menubaseitem" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Include, NullValueHandling = NullValueHandling.Ignore )]
        public virtual string Xtype
        {
            get
            {
                return "menubaseitem";
            }
            set { }
        }
    }
}
