#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.Linq;
using NetJsWire.Lite.Editors;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Menu
{
    /// <summary>
    ///   <p>A menu object.  This is the container to which you may add menu items.  Menu can also serve as a base class
    ///     when you want a specialized menu based off of another component (like {@link Ext.menu.DateMenu} for example).</p>
    ///   <p>Menus may contain either {@link Ext.menu.Item menu items}, or general {@link Ext.Component Component}s.</p>
    ///   <p>To make a contained general {@link Ext.Component Component} line up with other {@link Ext.menu.Item menu items}
    ///     specify <tt>iconCls: 'no-icon'</tt>.  This reserves a space for an icon, and indents the Component in line
    ///     with the other menu items.  See {@link Ext.form.ComboBox}.{@link Ext.form.ComboBox#getListParent getListParent}
    ///     for an example.</p>
    ///   <p>By default, Menus are absolutely positioned, floating Components. By configuring a Menu with
    ///     <b><tt>{@link #floating}:false</tt></b>, a Menu may be used as child of a Container.</p>
    ///   @xtype menu
    /// </summary>
    [ToolboxItem( true )]
    [Designer( typeof( ComponentDesigner ) )]
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class Menu : Container
    {
        private bool _enableScrolling = true;
        private bool _showSeparator = true;

        /// <summary>
        ///   A config object that will be applied to all items added to this container either via the {@link #items}
        ///   config or via the {@link #add} method.  The defaults config can contain any number of
        ///   name/value property pairs to be added to each item, and should be valid for the types of items
        ///   being added to the menu.
        /// </summary>
        [JsonProperty( "defaults" )]
        [DefaultValue( null )]
        [Editor( typeof( ScriptEditor ), typeof( UITypeEditor ) )]
        [TypeConverter( typeof( StringConverter ) )]
        public object Defaults { get; set; }

        /// <summary>
        ///   The minimum width of the menu in pixels (defaults to 120)
        /// </summary>
        [JsonProperty( "minWidth" )]
        [DefaultValue( 0 )]
        public int MinWidth { get; set; }

        /// <summary>
        ///   True or 'sides' for the default effect, 'frame' for 4-way shadow, and 'drop'for bottom-right shadow (defaults to 'sides')
        /// </summary>
        [JsonProperty( "shadow" )]
        [DefaultValue( ( string ) null )]
        public string Shadow { get; set; }

        /// <summary>
        ///   The {@link Ext.Element#alignTo} anchor position value to use for submenus ofthis menu (defaults to 'tl-tr?')
        /// </summary>
        [JsonProperty( "subMenuAlign" )]
        [DefaultValue( ( string ) null )]
        public string SubMenuAlign { get; set; }

        /// <summary>
        ///   The default {@link Ext.Element#alignTo} anchor position value for this menurelative to its element of origin (defaults to 'tl-bl?')
        /// </summary>
        [JsonProperty( "defaultAlign" )]
        [DefaultValue( ( string ) null )]
        public string DefaultAlign { get; set; }

        /// <summary>
        ///   True to allow multiple menus to be displayed at the same time (defaults to false)
        /// </summary>
        [JsonProperty( "allowOtherMenus" )]
        [DefaultValue( false )]
        public bool AllowOtherMenus { get; set; }

        /// <summary>
        ///   True to ignore clicks on any item in this menu that is a parent item (displaysa submenu) so that the submenu is not dismissed when clicking the parent item (defaults to false).
        /// </summary>
        [JsonProperty( "ignoreParentClicks" )]
        [DefaultValue( false )]
        public bool IgnoreParentClicks { get; set; }

        /// <summary>
        ///   True to allow the menu container to have scroller controls if the menu is too long (defaults to true).
        /// </summary>
        [JsonProperty( "enableScrolling" )]
        [DefaultValue( true )]
        public bool EnableScrolling
        {
            get { return _enableScrolling; }
            set { _enableScrolling = value; }
        }

        /// <summary>
        ///   The maximum height of the menu. Only applies when enableScrolling is set to True (defaults to null).
        /// </summary>
        [JsonProperty( "maxHeight" )]
        [DefaultValue( 0 )]
        public int MaxHeight { get; set; }

        /// <summary>
        ///   The amount to scroll the menu. Only applies when enableScrolling is set to True (defaults to 24).
        /// </summary>
        [JsonProperty( "scrollIncrement" )]
        [DefaultValue( 0 )]
        public int ScrollIncrement { get; set; }

        /// <summary>
        ///   True to show the icon separator. (defaults to true).
        /// </summary>
        [JsonProperty( "showSeparator" )]
        [DefaultValue( true )]
        public bool ShowSeparator
        {
            get { return _showSeparator; }
            set { _showSeparator = value; }
        }

        /// <summary>
        ///   An array specifying the [x, y] offset in pixels by which tochange the default Menu popup position after aligning according to the {@link #defaultAlign}
        ///   configuration. Defaults to <tt>[0, 0]</tt>.
        /// </summary>
        [JsonProperty( "defaultOffsets" )]
        [DefaultValue( ( string ) null )]
        public string DefaultOffsets { get; set; }

        /// <summary>
        ///   True to remove the incised line down the left side of the menu. Defaults to <tt>false</tt>.
        /// </summary>
        [JsonProperty( "plain" )]
        [DefaultValue( false )]
        public bool Plain { get; set; }

        /// <summary>
        ///   <p>By default, a Menu configured as <b><code>floating:true</code></b>
        ///     will be rendered as an {@link Ext.Layer} (an absolutely positioned,
        ///     floating Component with zindex=15000).
        ///     If configured as <b><code>floating:false</code></b>, the Menu may be
        ///     used as child item of another Container instead of a free-floating
        ///     {@link Ext.Layer Layer}.</p>
        /// </summary>
        [JsonProperty( "floating" )]
        [DefaultValue( false )]
        public bool Floating { get; set; }

        /// <summary>
        ///   zIndex to use when the menu is floating.
        /// </summary>
        [JsonProperty( "zIndex" )]
        [DefaultValue( 0 )]
        public int ZIndex { get; set; }

        //[DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        [Editor( typeof( MenuItemsEditor ), typeof( UITypeEditor ) ), Browsable( true ), JsonProperty( "items", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        public new BaseComponent[] Items { get; set; }

        public override IEnumerable RenderItems
        {
            get
            {
                return Items != null && Items.Length > 0 ? Items.ToList() : null;
            }
        }

        [DefaultValue( "menu" )]
        [JsonProperty( "xtype", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore )]
        public override string Xtype
        {
            get
            {
                return "menu";
            }
            set
            {
            }
        }

        public new bool ShouldSerializeItems()
        {
            return Items != null && Items.Length > 0;
        }

        protected void ResetItems()
        {
            Items = null;
        }
    }
}
