#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Toolbar
{
    /// <summary>
    ///   /**
    ///   A simple class that renders text directly into a toolbar
    ///   (with css class:<tt>'xtb-text'</tt>). Example usage:
    ///   <pre><code>
    ///          new Ext.Panel({
    ///          tbar : [
    ///          {xtype: 'tbtext', text: 'Item 1'} // or simply 'Item 1'
    ///          ]
    ///          });
    ///        </code></pre>
    /// </summary>
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public class TextItem : Item
    {
        private string _text;

        /// <summary>
        ///   Creates a new TextItem
        ///   @xtype tbtext
        /// </summary>
        /// <returns></returns>
        public TextItem()
        {
            base.Xtype = "tbtext";
        }

        [JsonProperty( "text", NullValueHandling = NullValueHandling.Ignore, DefaultValueHandling = DefaultValueHandling.Ignore )]
        [Description( "The text to be used as innerHTML (html tags are accepted)" )]
        [Browsable( true )]
        [RefreshProperties( RefreshProperties.All )]
        public override string Text
        {
            get { return _text; }
            set
            {
                _text = value;
                if ( !string.IsNullOrEmpty( _text ) )
                {
                    using ( var g = CreateGraphics() )
                    {
                        using ( var font = new Font( "Arial", 11, FontStyle.Regular, GraphicsUnit.Pixel ) )
                        {
                            var width = g.MeasureString( _text, font );
                            Size = new Size( ( int ) width.Width, 22 );
                        }
                    }

                }
                else
                {
                    Size = new Size( 2, 22 );
                }
            }
        }

        public override Color BackColor
        {
            get
            {
                return Color.FromArgb( 205, 220, 238 );
            }
            set
            {
            }
        }

        [JsonIgnore]
        public override int Height
        {
            get
            {
                return base.Height;
            }
            set
            {
                base.Height = value;
            }
        }

        [JsonIgnore]
        public override int? X
        {
            get
            {
                return base.X;
            }
        }

        [JsonIgnore]
        public override int? Y
        {
            get
            {
                return base.Y;
            }
        }

        protected override void OnNonClientAreaCalcSize( ref Rectangle bounds, bool update )
        {
            bounds = bounds;
        }

        protected override void OnPaint( PaintEventArgs e )
        {
            base.OnPaint( e );

            if ( !string.IsNullOrWhiteSpace( Text ) )
            {
                using ( var font = new Font( "Arial", 11, FontStyle.Regular, GraphicsUnit.Pixel ) )
                {
                    var strSize = e.Graphics.MeasureString( Text, font );
                    TextRenderer.DrawText( e.Graphics, Text, font,
                                           new Rectangle( 0, 0 /* ( int ) ( ( this.Size.Height / 2 ) - ( strSize.Height / 2 ) )*/, ClientSize.Width, ClientSize.Height ),
                                           Color.Black, Color.Transparent,
                                           TextFormatFlags.VerticalCenter );
                }
            }
        }
    }
}