#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using NetJsWire.Lite.Designers;
using NetJsWire.Lite.Properties;
using NetJsWire.Lite.Widgets.UI.Layout;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI
{
    /// <summary>
    ///   /**
    ///   <p>Basic Toolbar class. Although the <tt>{@link Ext.Container#defaultType defaultType}</tt> for Toolbar
    ///     is <tt>{@link Ext.Button button}</tt>, Toolbar elements (child items for the Toolbar container) may
    ///     be virtually any type of Component. Toolbar elements can be created explicitly via their constructors,
    ///     or implicitly via their xtypes, and can be <tt>{@link #add}</tt>ed dynamically.</p>
    ///   <p>Some items have shortcut strings for creation:</p>
    ///   <pre>
    ///     <u>Shortcut</u>  <u>xtype</u>          <u>Class</u>                  <u>Description</u>
    ///     '->'      'tbfill'       {@link Ext.Toolbar.Fill}       begin using the right-justified button container
    ///     '-'       'tbseparator'  {@link Ext.Toolbar.Separator}  add a vertical separator bar between toolbar items
    ///     ' '       'tbspacer'     {@link Ext.Toolbar.Spacer}     add horiztonal space between elements
    ///   </pre>
    ///   Example usage of various elements:
    ///   <pre><code>
    ///          var tb = new Ext.Toolbar({
    ///          renderTo: document.body,
    ///          width: 600,
    ///          height: 100,
    ///          items: [
    ///          {
    ///          // xtype: 'button', // default for Toolbars, same as 'tbbutton'
    ///          text: 'Button'
    ///          },
    ///          {
    ///          xtype: 'splitbutton', // same as 'tbsplitbutton'
    ///          text: 'Split Button'
    ///          },
    ///          // begin using the right-justified button container
    ///          '->', // same as {xtype: 'tbfill'}, // Ext.Toolbar.Fill
    ///          {
    ///          xtype: 'textfield',
    ///          name: 'field1',
    ///          emptyText: 'enter search term'
    ///          },
    ///          // add a vertical separator bar between toolbar items
    ///          '-', // same as {xtype: 'tbseparator'} to create Ext.Toolbar.Separator
    ///          'text 1', // same as {xtype: 'tbtext', text: 'text1'} to create Ext.Toolbar.TextItem
    ///          {xtype: 'tbspacer'},// same as ' ' to create Ext.Toolbar.Spacer
    ///          'text 2',
    ///          {xtype: 'tbspacer', width: 50}, // add a 50px space
    ///          'text 3'
    ///          ]
    ///          });
    ///        </code></pre>
    ///   Example adding a ComboBox within a menu of a button:
    ///   <pre><code>
    ///          // ComboBox creation
    ///          var combo = new Ext.form.ComboBox({
    ///          store: new Ext.data.ArrayStore({
    ///          autoDestroy: true,
    ///          fields: ['initials', 'fullname'],
    ///          data : [
    ///          ['FF', 'Fred Flintstone'],
    ///          ['BR', 'Barney Rubble']
    ///          ]
    ///          }),
    ///          displayField: 'fullname',
    ///          typeAhead: true,
    ///          mode: 'local',
    ///          forceSelection: true,
    ///          triggerAction: 'all',
    ///          emptyText: 'Select a name...',
    ///          selectOnFocus: true,
    ///          width: 135,
    ///          getListParent: function() {
    ///          return this.el.up('.x-menu');
    ///          },
    ///          iconCls: 'no-icon' //use iconCls if placing within menu to shift to right side of menu
    ///          });
    ///          // put ComboBox in a Menu
    ///          var menu = new Ext.menu.Menu({
    ///          id: 'mainMenu',
    ///          items: [
    ///          combo // A Field in a Menu
    ///          ]
    ///          });
    ///          // add a Button with the menu
    ///          tb.add({
    ///          text:'Button w/ Menu',
    ///          menu: menu  // assign menu by instance
    ///          });
    ///          tb.doLayout();
    ///        </code></pre>
    /// </summary>
    [Designer( typeof( ContainerDesigner ) )]
    [ToolboxItem( true )]
    public class ToolbarControl : Container
    {
        private ButtonAlign _buttonAlign = ButtonAlign.Left;

        public ToolbarControl()
            : this( null )
        {
        }

        public ToolbarControl( IContainer container )
            : base( container )
        {
            base.Xtype = "toolbar";
            base.Layout = LayoutType.Toolbar;
            Size = new Size( Size.Width, 26 );
            SizeChanged += ToolbarControlSizeChanged;

            MouseUp += ToolbarControl_MouseUp;
        }

        //protected override void OnNonClientAreaCalcSize( ref Rectangle bounds, bool update )
        //{
        //    var clientPadding = new Padding( 1 );
        //    bounds = new Rectangle( bounds.Left + clientPadding.Left, bounds.Top + clientPadding.Top,
        //        bounds.Width - clientPadding.Horizontal, bounds.Height - clientPadding.Vertical );
        //}

        //protected override void OnPaintNcArea( PaintEventArgs e )
        //{
        //    e.Graphics.DrawRectangle( new Pen( Color.FromArgb( 153, 187, 232 ) ), e.ClipRectangle );
        //}

        /// <summary>
        ///   This class assigns a default layout (<code>layout:'<b>toolbar</b>'</code>).
        ///   Developers <i>may</i> override this configuration option if another layout
        ///   is required (the constructor must be passed a configuration object in this
        ///   case instead of an array).
        ///   See {@link Ext.Container#layout} for additional information.
        /// </summary>
        [JsonProperty( "layout" )]
        [DefaultValue( LayoutType.Toolbar )]
        public override LayoutType Layout
        {
            get { return base.Layout; }
            set { base.Layout = value; }
        }

        /// <summary>
        ///   Defaults to false. Configure <tt>true</tt> to make the toolbar provide a button
        ///   which activates a dropdown Menu to show items which overflow the Toolbar's width.
        /// </summary>
        [JsonProperty( "enableOverflow" )]
        [DefaultValue( false )]
        public bool EnableOverflow { get; set; }

        /// <summary>
        ///   <p>The default position at which to align child items. Defaults to <code>"left"</code></p>
        ///   <p>May be specified as <code>"center"</code> to cause items added before a Fill (A <code>"->"</code>) item
        ///     to be centered in the Toolbar. Items added after a Fill are still right-aligned.</p>
        ///   <p>Specify as <code>"right"</code> to right align all child items.</p>
        /// </summary>
        [JsonProperty( "buttonAlign" )]
        [DefaultValue( ButtonAlign.Left )]
        public ButtonAlign ButtonAlign
        {
            get { return _buttonAlign; }
            set { _buttonAlign = value; }
        }

        [JsonProperty( "width" )]
        public override int Width
        {
            get
            {
                return base.Width;
            }
            set
            {
                base.Width = value;
            }
        }

        [JsonProperty( "height" )]
        public override int Height
        {
            get
            {
                return base.Height;
            }
            set
            {
                base.Height = value;
            }
        }

        void ToolbarControl_MouseUp( object sender, MouseEventArgs e )
        {
            if(e.Button == MouseButtons.Left)
            {
                foreach ( Control control in Controls )
                {
                    if(control.Bounds.Contains( e.Location ))
                        control.Select();
                }
            }
        }

        void ToolbarControlSizeChanged( object sender, EventArgs e )
        {
            if ( Size.Height != 26 )
                Size = new Size( Size.Width, 26 );
        }

        protected override void Dispose( bool disposing )
        {
            SizeChanged -= ToolbarControlSizeChanged;
            base.Dispose( disposing );
        }

        protected override void OnPaint( PaintEventArgs e )
        {
            e.Graphics.DrawImage( Resources.bg, e.ClipRectangle, 0, 0, 1, Resources.bg.Height, GraphicsUnit.Pixel );
            e.Graphics.DrawRectangle( new Pen( Panel.FrameBorderColor ), e.ClipRectangle );
            if ( Items != null )
            {
                foreach ( var item in Items.Cast<BaseComponent>() )
                {
                    var r = new Rectangle( e.ClipRectangle.X + item.Location.X, e.ClipRectangle.Y + item.Location.Y,
                                           item.Size.Width, item.Size.Height );
                    item.OnPaintInternal( new PaintEventArgs( e.Graphics, r ) );
                }
            }
        }

        public bool ShouldSerializeWidth()
        {
            return !( Parent is Panel ) || ( ( Panel ) Parent ).Bbar != this;
        }

        public bool ShouldSerializeHeight()
        {
            return false;
        }
    }
}
