#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using NetJsWire.Lite.Converters;
using NetJsWire.Lite.Widgets.UI.Util;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   /**
    ///   A TreeLoader provides for lazy loading of an {@link Ext.tree.TreeNode}'s child
    ///   nodes from a specified URL. The response must be a JavaScript Array definition
    ///   whose elements are node definition objects. e.g.:
    ///   <pre><code>
    ///          [{
    ///          id: 1,
    ///          text: 'A leaf Node',
    ///          leaf: true
    ///          },{
    ///          id: 2,
    ///          text: 'A folder Node',
    ///          children: [{
    ///          id: 3,
    ///          text: 'A child Node',
    ///          leaf: true
    ///          }]
    ///          }]
    ///        </code></pre>
    ///   <br><br>
    ///         A server request is sent, and child nodes are loaded only when a node is expanded.
    ///         The loading node's id is passed to the server under the parameter name "node" to
    ///         enable the server to produce the correct child nodes.
    ///         <br><br>
    ///               To pass extra parameters, an event handler may be attached to the "beforeload"
    ///               event, and the parameters specified in the TreeLoader's baseParams property:
    ///               <pre><code>
    ///                      myTreeLoader.on("beforeload", function(treeLoader, node) {
    ///                      this.baseParams.category = node.attributes.category;
    ///                      }, this);
    ///                    </code></pre>
    ///               This would pass an HTTP parameter called "category" to the server containing
    ///               the value of the Node's "category" attribute.
    /// </summary>
    public class TreeLoader : BaseObservable
    {
        /// <summary>
        ///   The URL from which to request a Json string whichspecifies an array of node definition objects representing the child nodes
        ///   to be loaded.
        /// </summary>
        [JsonProperty( "dataUrl" )]
        public string DataUrl { get; set; }

        /// <summary>
        ///   The HTTP request method for loading data (defaults to the value of {@link Ext.Ajax#method}).
        /// </summary>
        [JsonProperty( "requestMethod" )]
        public string RequestMethod { get; set; }

        /// <summary>
        ///   Equivalent to {@link #dataUrl}.
        /// </summary>
        [JsonProperty( "url" )]
        public string Url { get; set; }

        /// <summary>
        ///   If set to true, the loader recursively loads "children" attributes when doing the first load on nodes.
        /// </summary>
        [JsonProperty( "preloadChildren" )]
        public bool PreloadChildren { get; set; }

        /// <summary>
        ///   (optional) An object containing properties whichspecify HTTP parameters to be passed to each request for child nodes.
        /// </summary>
        [JsonProperty( "baseParams" )]
        public object BaseParams { get; set; }

        /// <summary>
        ///   (optional) An object containing attributes to be added to all nodescreated by this loader. If the attributes sent by the server have an attribute in this object,
        ///   they take priority.
        /// </summary>
        [JsonProperty( "baseAttrs" )]
        public object BaseAttrs { get; set; }

        /// <summary>
        ///   (optional) An object containing properties whichspecify custom {@link Ext.tree.TreeNodeUI} implementations. If the optional
        ///   <i>uiProvider</i> attribute of a returned child node is a string rather
        ///   than a reference to a TreeNodeUI implementation, then that string value
        ///   is used as a property name in the uiProviders object.
        /// </summary>
        [JsonProperty( "uiProviders" )]
        public object UiProviders { get; set; }

        /// <summary>
        ///   (optional) Default to true. Remove previously existingchild nodes before loading.
        /// </summary>
        [JsonProperty( "clearOnLoad" )]
        public bool ClearOnLoad { get; set; }

        /// <summary>
        ///   Defaults to <tt>undefined</tt>. Only used when using directFn.Specifies the params in the order in which they must be passed to the server-side Direct method
        ///   as either (1) an Array of String values, or (2) a String of params delimited by either whitespace,
        ///   comma, or pipe. For example,
        ///   any of the following would be acceptable:<pre><code>
        ///                                                   nodeParameter: 'node',
        ///                                                   paramOrder: ['param1','param2','param3']
        ///                                                   paramOrder: 'node param1 param2 param3'
        ///                                                   paramOrder: 'param1,node,param2,param3'
        ///                                                   paramOrder: 'param1|param2|param|node'
        ///                                                 </code></pre>
        /// </summary>
        [JsonProperty( "paramOrder" )]
        public object ParamOrder { get; set; }

        /// <summary>
        ///   Only used when using directFn.Send parameters as a collection of named arguments (defaults to <tt>false</tt>). Providing a
        ///   <tt>{@link #paramOrder}</tt> nullifies this configuration.
        /// </summary>
        [JsonProperty( "paramsAsHash" )]
        public bool ParamsAsHash { get; set; }

        /// <summary>
        ///   The name of the parameter sent to the server which containsthe identifier of the node. Defaults to <tt>'node'</tt>.
        /// </summary>
        [JsonProperty( "nodeParameter" )]
        public string NodeParameter { get; set; }

        /// <summary>
        ///   Function to call when executing a request.
        /// </summary>
        [JsonProperty( "directFn" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string DirectFn { get; set; }
    }
}
