#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms.Design;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   /**
    ///   <p>The TreePanel provides tree-structured UI representation of tree-structured data.</p>
    ///   <p>{@link Ext.tree.TreeNode TreeNode}s added to the TreePanel may each contain metadata
    ///     used by your application in their {@link Ext.tree.TreeNode#attributes attributes} property.</p>
    ///   <p><b>A TreePanel must have a {@link #root} node before it is rendered.</b> This may either be
    ///     specified using the {@link #root} config option, or using the {@link #setRootNode} method.
    ///     <p>An example of tree rendered to an existing div:</p><pre><code>
    ///                                                                  var tree = new Ext.tree.TreePanel({
    ///                                                                  renderTo: 'tree-div',
    ///                                                                  useArrows: true,
    ///                                                                  autoScroll: true,
    ///                                                                  animate: true,
    ///                                                                  enableDD: true,
    ///                                                                  containerScroll: true,
    ///                                                                  border: false,
    ///                                                                  // auto create TreeLoader
    ///                                                                  dataUrl: 'get-nodes.php',
    ///                                                                  root: {
    ///                                                                  nodeType: 'async',
    ///                                                                  text: 'Ext JS',
    ///                                                                  draggable: false,
    ///                                                                  id: 'source'
    ///                                                                  }
    ///                                                                  });
    ///                                                                  tree.getRootNode().expand();
    ///                                                                </code></pre>
    ///     <p>The example above would work with a data packet similar to this:</p><pre><code>
    ///                                                                                   [{
    ///                                                                                   "text": "adapter",
    ///                                                                                   "id": "source\/adapter",
    ///                                                                                   "cls": "folder"
    ///                                                                                   }, {
    ///                                                                                   "text": "dd",
    ///                                                                                   "id": "source\/dd",
    ///                                                                                   "cls": "folder"
    ///                                                                                   }, {
    ///                                                                                   "text": "debug.js",
    ///                                                                                   "id": "source\/debug.js",
    ///                                                                                   "leaf": true,
    ///                                                                                   "cls": "file"
    ///                                                                                   }]
    ///                                                                                 </code></pre>
    ///     <p>An example of tree within a Viewport:</p><pre><code>
    ///                                                        new Ext.Viewport({
    ///                                                        layout: 'border',
    ///                                                        items: [{
    ///                                                        region: 'west',
    ///                                                        collapsible: true,
    ///                                                        title: 'Navigation',
    ///                                                        xtype: 'treepanel',
    ///                                                        width: 200,
    ///                                                        autoScroll: true,
    ///                                                        split: true,
    ///                                                        loader: new Ext.tree.TreeLoader(),
    ///                                                        root: new Ext.tree.AsyncTreeNode({
    ///                                                        expanded: true,
    ///                                                        children: [{
    ///                                                        text: 'Menu Option 1',
    ///                                                        leaf: true
    ///                                                        }, {
    ///                                                        text: 'Menu Option 2',
    ///                                                        leaf: true
    ///                                                        }, {
    ///                                                        text: 'Menu Option 3',
    ///                                                        leaf: true
    ///                                                        }]
    ///                                                        }),
    ///                                                        rootVisible: false,
    ///                                                        listeners: {
    ///                                                        click: function(n) {
    ///                                                        Ext.Msg.alert('Navigation Tree Click', 'You clicked: "' + n.attributes.text + '"');
    ///                                                        }
    ///                                                        }
    ///                                                        }, {
    ///                                                        region: 'center',
    ///                                                        xtype: 'tabpanel',
    ///                                                        // remaining code not shown ...
    ///                                                        }]
    ///                                                        });
    ///                                                      </code></pre>
    ///     @cfg {Ext.tree.TreeNode} root The root node for the tree.
    ///     @cfg {Boolean} rootVisible <tt>false</tt> to hide the root node (defaults to <tt>true</tt>)
    ///     @cfg {Boolean} lines <tt>false</tt> to disable tree lines (defaults to <tt>true</tt>)
    ///     @cfg {Boolean} enableDD <tt>true</tt> to enable drag and drop
    ///     @cfg {Boolean} enableDrag <tt>true</tt> to enable just drag
    ///     @cfg {Boolean} enableDrop <tt>true</tt> to enable just drop
    ///     @cfg {Object} dragConfig Custom config to pass to the {@link Ext.tree.TreeDragZone} instance
    ///     @cfg {Object} dropConfig Custom config to pass to the {@link Ext.tree.TreeDropZone} instance
    ///     @cfg {String} ddGroup The DD group this TreePanel belongs to
    ///     @cfg {Boolean} ddAppendOnly <tt>true</tt> if the tree should only allow append drops (use for trees which are sorted)
    ///     @cfg {Boolean} ddScroll <tt>true</tt> to enable body scrolling
    ///     @cfg {Boolean} containerScroll <tt>true</tt> to register this container with ScrollManager
    ///     @cfg {Boolean} hlDrop <tt>false</tt> to disable node highlight on drop (defaults to the value of {@link Ext#enableFx})
    ///     @cfg {String} hlColor The color of the node highlight (defaults to <tt>'C3DAF9'</tt>)
    ///     @cfg {Boolean} animate <tt>true</tt> to enable animated expand/collapse (defaults to the value of {@link Ext#enableFx})
    ///     @cfg {Boolean} singleExpand <tt>true</tt> if only 1 node per branch may be expanded
    ///     @cfg {Object} selModel A tree selection model to use with this TreePanel (defaults to an {@link Ext.tree.DefaultSelectionModel})
    ///     @cfg {Boolean} trackMouseOver <tt>false</tt> to disable mouse over highlighting
    ///     @cfg {Ext.tree.TreeLoader} loader A {@link Ext.tree.TreeLoader} for use with this TreePanel
    ///     @cfg {String} pathSeparator The token used to separate sub-paths in path strings (defaults to <tt>'/'</tt>)
    ///     @cfg {Boolean} useArrows <tt>true</tt> to use Vista-style arrows in the tree (defaults to <tt>false</tt>)
    ///     @cfg {String} requestMethod The HTTP request method for loading data (defaults to the value of {@link Ext.Ajax#method}).
    /// </summary>
    [Designer( typeof( ControlDesigner ) )]
    public class TreePanel : Panel
    {
        public TreePanel()
            : this( null )
        {
        }

        public TreePanel( IContainer container )
            : base( container )
        {
            base.Xtype = "treepanel";
            Size = new Size( 100, 200 );
            Root = new TreeNode();
        }

        /// <summary>
        ///   <p>An array of events that, when fired, should be bubbled to any parent container.
        ///     See {@link Ext.util.Observable#enableBubble}.
        ///     Defaults to <tt>[]</tt>.</p>
        /// </summary>
        [JsonProperty( "bubbleEvents" )]
        public string BubbleEvents { get; set; }

        /// <summary>
        ///   The root node of this tree.
        /// </summary>
        [JsonProperty( "root" )]
        public TreeNode Root { get; set; }

        /// <summary>
        ///   The dropZone used by this tree if drop is enabled (see {@link #enableDD} or {@link #enableDrop})
        /// </summary>
        [JsonProperty( "dropZone" )]
        public TreeDropZone DropZone { get; set; }

        /// <summary>
        ///   The dragZone used by this tree if drag is enabled (see {@link #enableDD} or {@link #enableDrag})
        /// </summary>
        [JsonProperty( "dragZone" )]
        public TreeDragZone DragZone { get; set; }
    }
}
