#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using NetJsWire.Lite.Converters;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Tree
{
    /// <summary>
    ///   Provides sorting of nodes in a {@link Ext.tree.TreePanel}.  The TreeSorter automatically monitors events on the
    ///   associated TreePanel that might affect the tree's sort order (beforechildrenrendered, append, insert and textchange).
    ///   Example usage:<br />
    ///   <pre><code>
    ///          new Ext.tree.TreeSorter(myTree, {
    ///          folderSort: true,
    ///          dir: "desc",
    ///          sortType: function(node) {
    ///          // sort by a custom, typed attribute:
    ///          return parseInt(node.id, 10);
    ///          }
    ///          });
    ///        </code></pre>
    /// </summary>
    public class TreeSorter
    {
        /// <summary>
        ///   True to sort leaf nodes under non-leaf nodes (defaults to false)
        /// </summary>
        [JsonProperty( "folderSort" )]
        public bool FolderSort { get; set; }

        /// <summary>
        ///   The named attribute on the node to sort by (defaults to "text").  Note that this property is only used if no {@link #sortType} function is specified, otherwise it is ignored.
        /// </summary>
        [JsonProperty( "property" )]
        public string Property { get; set; }

        /// <summary>
        ///   The direction to sort ("asc" or "desc," case-insensitive, defaults to "asc")
        /// </summary>
        [JsonProperty( "dir" )]
        public string Dir { get; set; }

        /// <summary>
        ///   The attribute used to determine leaf nodes when {@link #folderSort} = true (defaults to "leaf")
        /// </summary>
        [JsonProperty( "leafAttr" )]
        public string LeafAttr { get; set; }

        /// <summary>
        ///   true for case-sensitive sort (defaults to false)
        /// </summary>
        [JsonProperty( "caseSensitive" )]
        public bool CaseSensitive { get; set; }

        /// <summary>
        ///   A custom "casting" function used to convert node values before sorting.  The function will be called with a single parameter (the {@link Ext.tree.TreeNode} being evaluated) and is expected to return
        ///   the node's sort value cast to the specific data type required for sorting.  This could be used, for example, when
        ///   a node's text (or other attribute) should be sorted as a date or numeric value.  See the class description for
        ///   example usage.  Note that if a sortType is specified, any {@link #property} config will be ignored.
        /// </summary>
        [JsonProperty( "sortType" )]
        [JsonConverter( typeof( NotQuotedJsonStringConverter ) )]
        public string SortType { get; set; }
    }
}
