#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Xml.Serialization;
using NetJsWire.Lite.Attributes;
using NetJsWire.Lite.Interfaces;
using Newtonsoft.Json;

namespace NetJsWire.Lite.Widgets.UI.Util
{
    /// <summary>
    ///   /**
    ///   Abstract base class that provides a common interface for publishing events. Subclasses are expected to
    ///   to have a property "events" with all the events defined.<br>
    ///                                                             For example:
    ///                                                             <pre><code>
    ///                                                                    Employee = function(name){
    ///                                                                    this.name = name;
    ///                                                                    this.addEvents({
    ///                                                                    "fired" : true,
    ///                                                                    "quit" : true
    ///                                                                    });
    ///                                                                    }
    ///                                                                    Ext.extend(Employee, Ext.util.Observable);
    ///                                                                  </code></pre>
    ///                                                             */
    ///                                                             Ext.util.Observable = function(){
    /// </summary>
    [Serializable]
    public abstract class BaseObservable : IObservable, IComponent
    {
        private ListenersCollection _listeners;

        public BaseObservable()
        {
            _listeners = new ListenersCollection();
        }

        #region IComponent Members

        public event EventHandler Disposed;

        #endregion

        #region IObservable Members

        /// <summary>
        ///   A config object containing one or more event handlers to be added to this object during initialization.<br />
        ///   While some ExtJs Component classes export selected DOM events (e.g. "click", "mouseover" etc), this is usually only done when extra value can be added. 
        ///   For example the DataView's click event passing the node clicked on. 
        ///   To access DOM events directly from a Component's HTMLElement, listeners must be added to the Element after the Component has been rendered.
        /// </summary>
        [Description( "A config object containing one or more event handlers to be added to this object during initialization." )]
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        [JsonProperty( "listeners", DefaultValueHandling = DefaultValueHandling.Ignore, NullValueHandling = NullValueHandling.Ignore, Required = Required.AllowNull )]
        [Browsable( true )]
        [ListenerCollection( true )]
        public ListenersCollection Listeners
        {
            get { return _listeners; }
            set { _listeners = value; }
        }

        #endregion

        public bool ShouldSerializeListeners()
        {
            return Listeners.Count > 0;
        }

        protected void ResetListeners()
        {
            Listeners.Clear();
        }

        #region Implementation of IDisposable

        /// <summary>
        ///   Realiza tareas definidas por la aplicacin asociadas a la liberacin o al restablecimiento de recursos no administrados.
        /// </summary>
        /// <filterpriority>2</filterpriority>
        public void Dispose()
        {
            Listeners = null;
            ContainerControl = null;
            var dispEvent = Disposed;
            if ( dispEvent != null )
                dispEvent( this, EventArgs.Empty );
        }

        #endregion

        #region Implementation of IComponent

        [NonSerialized]
        private ISite _site;

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        [XmlIgnore]
        public Container ContainerControl { get; set; }

        /// <summary>
        ///   Obtiene o establece la interfaz <see cref = "T:System.ComponentModel.ISite" /> asociada a la interfaz <see cref = "T:System.ComponentModel.IComponent" />.
        /// </summary>
        /// <returns>
        ///   Objeto <see cref = "T:System.ComponentModel.ISite" /> asociado al componente; o null, si el componente no tiene un sitio asociado.
        /// </returns>
        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        [XmlIgnore]
        public ISite Site
        {
            get { return _site; }
            set
            {
                _site = value;
                if ( value == null )
                    return;

                var host = ( IDesignerHost ) value.GetService( typeof( IDesignerHost ) );

                if ( host == null )
                    return;
                var baseComp = host.RootComponent;

                if ( baseComp is Container )
                    ContainerControl = ( Container ) baseComp;
            }
        }

        #endregion
    }
}
