#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections;
using System.Diagnostics;
using System.Reflection;
using System.Runtime.Remoting.Messaging;
using System.Security.Principal;
using System.Threading;
using System.Web;

namespace NetJsWire.JsonRpcHandler.RealAsync
{

    #region IWorkRequest interface

    #endregion

    #region ThreadPool class

    public sealed class ThreadPool : WaitHandle
    {
        #region ThreadPool constructors

        public ThreadPool( int initialThreadCount, int maxThreadCount, string poolName )
            : this( initialThreadCount, maxThreadCount, poolName,
                    DefaultNewThreadTriggerTime,
                    DefaultDynamicThreadDecayTime,
                    DefaultThreadPriority,
                    DefaultRequestQueueLimit )
        {
        }

        public ThreadPool( int initialThreadCount, int maxThreadCount, string poolName,
                           int newThreadTrigger, int dynamicThreadDecayTime,
                           ThreadPriority threadPriority, int requestQueueLimit )
        {
            Debug.WriteLine( string.Format( "New thread pool {0} created:", poolName ) );
            Debug.WriteLine( string.Format( "  initial thread count:      {0}", initialThreadCount ) );
            Debug.WriteLine( string.Format( "  max thread count:          {0}", maxThreadCount ) );
            Debug.WriteLine( string.Format( "  new thread trigger:        {0} ms", newThreadTrigger ) );
            Debug.WriteLine( string.Format( "  dynamic thread decay time: {0} ms", dynamicThreadDecayTime ) );
            Debug.WriteLine( string.Format( "  request queue limit:       {0} entries", requestQueueLimit ) );

            SafeWaitHandle = _stopCompleteEvent.SafeWaitHandle;

            if ( maxThreadCount < initialThreadCount )
            {
                throw new ArgumentException( "Maximum thread count must be >= initial thread count.", "maxThreadCount" );
            }

            if ( dynamicThreadDecayTime <= 0 )
            {
                throw new ArgumentException( "Dynamic thread decay time cannot be <= 0.", "dynamicThreadDecayTime" );
            }

            if ( newThreadTrigger <= 0 )
            {
                throw new ArgumentException( "New thread trigger time cannot be <= 0.", "newThreadTrigger" );
            }

            _initialThreadCount = initialThreadCount;
            _maxThreadCount = maxThreadCount;
            _requestQueueLimit = ( requestQueueLimit < 0 ? DefaultRequestQueueLimit : requestQueueLimit );
            _decayTime = dynamicThreadDecayTime;
            _newThreadTrigger = new TimeSpan( TimeSpan.TicksPerMillisecond * newThreadTrigger );
            _threadPriority = threadPriority;
            _requestQueue = new Queue( requestQueueLimit < 0 ? 4096 : requestQueueLimit );

            if ( poolName == null )
            {
                throw new ArgumentNullException( "poolName", "Thread pool name cannot be null" );
            }

            _threadPoolName = poolName;
        }

        #endregion

        #region ThreadPool properties

        // The Priority & DynamicThreadDecay properties are not thread safe
        // and can only be set before Start is called.
        //
        public ThreadPriority Priority
        {
            get { return ( _threadPriority ); }

            set
            {
                if ( _hasBeenStarted )
                {
                    throw new InvalidOperationException( "Cannot adjust thread priority after pool has been started." );
                }

                _threadPriority = value;
            }
        }

        public int DynamicThreadDecay
        {
            get { return ( _decayTime ); }

            set
            {
                if ( _hasBeenStarted )
                {
                    throw new InvalidOperationException(
                        "Cannot adjust dynamic thread decay time after pool has been started." );
                }

                if ( value <= 0 )
                {
                    throw new ArgumentException( "Dynamic thread decay time cannot be <= 0.", "value" );
                }

                _decayTime = value;
            }
        }

        public int NewThreadTrigger
        {
            get { return ( ( int ) _newThreadTrigger.TotalMilliseconds ); }

            set
            {
                if ( value <= 0 )
                {
                    throw new ArgumentException( "New thread trigger time cannot be <= 0.", "value" );
                }

                lock ( this )
                {
                    _newThreadTrigger = new TimeSpan( TimeSpan.TicksPerMillisecond * value );
                }
            }
        }

        public int RequestQueueLimit
        {
            get { return ( _requestQueueLimit ); }
            set { _requestQueueLimit = ( value < 0 ? DefaultRequestQueueLimit : value ); }
        }

        public int AvailableThreads
        {
            get { return ( _maxThreadCount - _currentThreadCount ); }
        }

        public int MaxThreads
        {
            get { return ( _maxThreadCount ); }

            set
            {
                if ( value < _initialThreadCount )
                {
                    throw new ArgumentException( "Maximum thread count must be >= initial thread count.", "MaxThreads" );
                }

                _maxThreadCount = value;
            }
        }

        public bool IsStarted
        {
            get { return ( _hasBeenStarted ); }
        }

        public bool PropogateThreadPrincipal
        {
            get { return ( _propogateThreadPrincipal ); }
            set { _propogateThreadPrincipal = value; }
        }

        public bool PropogateCallContext
        {
            get { return ( _propogateCallContext ); }
            set { _propogateCallContext = value; }
        }

        public bool PropogateHttpContext
        {
            get { return ( _propogateHttpContext ); }
            set { _propogateHttpContext = value; }
        }

        public bool PropogateCASMarkers
        {
            get { return ( PropogateCasMarkers ); }
            // When CompressedStack get/set is opened up,
            // add the following setter back in.
            //
            // set { propogateCASMarkers = value; }
        }

        public bool IsBackground
        {
            get { return ( _useBackgroundThreads ); }

            set
            {
                if ( _hasBeenStarted )
                {
                    throw new InvalidOperationException( "Cannot adjust background status after pool has been started." );
                }

                _useBackgroundThreads = value;
            }
        }

        #endregion

        #region ThreadPool events

        public event ThreadPoolDelegate Started;
        public event ThreadPoolDelegate Stopped;

        #endregion

        public void Start()
        {
            lock ( this )
            {
                if ( _hasBeenStarted )
                {
                    throw new InvalidOperationException( "Pool has already been started." );
                }

                _hasBeenStarted = true;

                // Check to see if there were already items posted to the queue
                // before Start was called.  If so, reset their timestamps to
                // the current time.
                //
                if ( _requestQueue.Count > 0 )
                {
                    ResetWorkRequestTimes();
                }

                for ( var n = 0; n < _initialThreadCount; n++ )
                {
                    var thread = new ThreadWrapper( this, true, _threadPriority,
                                           string.Format( "{0} (static)", _threadPoolName ) );
                    thread.Start();
                }

                if ( Started != null )
                {
                    Started(); // TODO: reconsider firing this event while holding the lock...
                }
            }
        }

        // The actual implementation of PostRequest.
        //
        private bool PostRequest( WorkRequest request )
        {
            lock ( this )
            {
                // A requestQueueLimit of -1 means the queue is "unbounded"
                // (subject to available resources).  IOW, no artificial limit
                // has been placed on the maximum # of requests that can be
                // placed into the queue.
                //
                if ( ( _requestQueueLimit == -1 ) || ( _requestQueue.Count < _requestQueueLimit ) )
                {
                    try
                    {
                        _requestQueue.Enqueue( request );
                        Monitor.Pulse( this );
                        return ( true );
                    }
                    catch
                    {
                    }
                }
            }

            return ( false );
        }

        private void ResetWorkRequestTimes()
        {
            lock ( this )
            {
                var newTime = DateTime.Now; // DateTime.Now.Add(pool.newThreadTrigger);

                foreach ( WorkRequest wr in _requestQueue )
                {
                    wr.WorkingTime = newTime;
                }
            }
        }

        #region Private ThreadPool constants

        // Default parameters.
        //
        private const int DefaultDynamicThreadDecayTime = 5 /* minutes */* 60 /* sec/min */* 1000 /* ms/sec */;
        private const int DefaultNewThreadTriggerTime = 500; // milliseconds
        private const ThreadPriority DefaultThreadPriority = ThreadPriority.Normal;
        private const int DefaultRequestQueueLimit = -1; // unbounded

        #endregion

        #region Private ThreadPool member variables

        private const bool PropogateCasMarkers = false;
        private readonly int _initialThreadCount;
        private readonly Queue _requestQueue;
        private readonly ManualResetEvent _stopCompleteEvent = new ManualResetEvent( false );
        private readonly string _threadPoolName;
        // Initial # of threads to create (called "static threads" in this class).

        // Cap for thread count.  Threads added above initialThreadCount are called "dynamic" threads.

        private int _currentThreadCount; // Current # of threads in the pool (static + dynamic).

        private int _decayTime;
        private bool _hasBeenStarted;
        private int _maxThreadCount;
        // If a dynamic thread is idle for this period of time w/o processing work requests, it will exit.

        private TimeSpan _newThreadTrigger;
        // If a work request sits in the queue this long before being processed, a new thread will be added to queue up to the max.

        private bool _propogateCallContext;
        private bool _propogateHttpContext;
        private bool _propogateThreadPrincipal;
        private int _requestQueueLimit; // Throttle for maximum # of work requests that can be added.
        private bool _stopInProgress;
        private ThreadPriority _threadPriority;
        private bool _useBackgroundThreads = true;

        #endregion

        #region ThreadPool.ThreadInfo

        private class ThreadInfo
        {
            public static ThreadInfo Capture( bool propogateThreadPrincipal, bool propogateCallContext,
                                              bool propogateHttpContext, bool propogateCasMarkers )
            {
                return new ThreadInfo( propogateThreadPrincipal, propogateCallContext,
                                       propogateHttpContext, propogateCasMarkers );
            }

            public static ThreadInfo Impersonate( ThreadInfo ti )
            {
                if ( ti == null ) throw new ArgumentNullException( "ti" );

                var prevInfo = Capture( true, true, true, true );
                Restore( ti );
                return ( prevInfo );
            }

            public static void Restore( ThreadInfo ti )
            {
                if ( ti == null ) throw new ArgumentNullException( "ti" );

                // Restore call context.
                //
                if ( MiSetLogicalCallContext != null )
                {
                    MiSetLogicalCallContext.Invoke( Thread.CurrentThread, new object[] { ti._callContext } );
                }

                // Restore HttpContext with the moral equivalent of
                // HttpContext.Current = ti.httpContext;
                //
                CallContext.SetData( HttpContextSlotName, ti._httpContext );

                // Restore thread identity.  It's important that this be done after
                // restoring call context above, since restoring call context also
                // overwrites the current thread principal setting.  If propogateCallContext
                // and propogateThreadPrincipal are both true, then the following is redundant.
                // However, since propogating call context requires the use of reflection
                // to capture/restore call context, I want that behavior to be independantly
                // switchable so that it can be disabled; while still allowing thread principal
                // to be propogated.  This also covers us in the event that call context
                // propogation changes so that it no longer propogates thread principal.
                //
                Thread.CurrentPrincipal = ti._principal;

                if ( ti._compressedStack != null )
                {
                    // TODO: Uncomment the following when Thread.SetCompressedStack is no longer guarded
                    //       by a StrongNameIdentityPermission.
                    //
                    // Thread.CurrentThread.SetCompressedStack(ti.compressedStack);
                }
            }

            private ThreadInfo( bool propogateThreadPrincipal, bool propogateCallContext,
                                bool propogateHttpContext, bool propogateCasMarkers )
            {
                if ( propogateThreadPrincipal )
                {
                    _principal = Thread.CurrentPrincipal;
                }

                if ( propogateHttpContext )
                {
                    _httpContext = HttpContext.Current;
                }

                if ( propogateCallContext && ( MiGetLogicalCallContext != null ) )
                {
                    _callContext = ( LogicalCallContext ) MiGetLogicalCallContext.Invoke( Thread.CurrentThread, null );
                    _callContext = ( LogicalCallContext ) _callContext.Clone();

                    // TODO: consider serialize/deserialize call context to get a MBV snapshot
                    //       instead of leaving it up to the Clone method.
                }

                if ( propogateCasMarkers )
                {
                    // TODO: Uncomment the following when Thread.GetCompressedStack is no longer guarded
                    //       by a StrongNameIdentityPermission.
                    //
                    // compressedStack = Thread.CurrentThread.GetCompressedStack();
                }
            }

            private readonly IPrincipal _principal;
            private readonly LogicalCallContext _callContext;
#pragma warning disable 649
            private CompressedStack _compressedStack; // Always null until Get/SetCompressedStack are opened up.
#pragma warning restore 649
            private readonly HttpContext _httpContext;

            // Cached type information.
            //
            private const BindingFlags BfNonPublicInstance = BindingFlags.Instance | BindingFlags.NonPublic;
            private const BindingFlags BfNonPublicStatic = BindingFlags.Static | BindingFlags.NonPublic;

            private static readonly MethodInfo MiGetLogicalCallContext =
                typeof( Thread ).GetMethod( "GetLogicalCallContext", BfNonPublicInstance );

            private static readonly MethodInfo MiSetLogicalCallContext =
                typeof( Thread ).GetMethod( "SetLogicalCallContext", BfNonPublicInstance );

            private static readonly string HttpContextSlotName;

            static ThreadInfo()
            {
                // Lookup the value of HttpContext.CallContextSlotName (if it exists)
                // to see what the name of the call context slot is where HttpContext.Current
                // is stashed.  As a fallback, if this field isn't present anymore, just
                // try for the original "HttpContext" slot name.
                //
                var fi = typeof( HttpContext ).GetField( "CallContextSlotName", BfNonPublicStatic );

                if ( fi != null )
                {
                    HttpContextSlotName = ( string ) fi.GetValue( null );
                }
                else
                {
                    HttpContextSlotName = "HttpContext";
                }
            }
        }

        #endregion

        #region ThreadPool.WorkRequest

        private class WorkRequest : IWorkRequest
        {
            internal const int Pending = 0;
            internal const int Processed = 1;
            internal const int Cancelled = 2;

            internal readonly object ProcArg; // State to pass to function.
            internal readonly object[] ProcArgs; // Used with Delegate.DynamicInvoke.
            internal readonly Delegate TargetProc; // Function to call.
            internal ThreadInfo InternalThreadInfo; // Everything we know about a thread.
            internal int State = Pending; // The state of this particular request.
            internal DateTime TimeStampStarted; // Time work request was originally enqueued (held constant).
            internal DateTime WorkingTime; // Current timestamp used for triggering new threads (moving target).

            public WorkRequest( WorkRequestDelegate cb, object arg,
                                bool propogateThreadPrincipal, bool propogateCallContext,
                                bool propogateHttpContext, bool propogateCASMarkers )
            {
                TargetProc = cb;
                ProcArg = arg;
                ProcArgs = null;

                Initialize( propogateThreadPrincipal, propogateCallContext,
                            propogateHttpContext, propogateCASMarkers );
            }

            public WorkRequest( Delegate cb, object[] args,
                                bool propogateThreadPrincipal, bool propogateCallContext,
                                bool propogateHttpContext, bool propogateCASMarkers )
            {
                TargetProc = cb;
                ProcArg = null;
                ProcArgs = args;

                Initialize( propogateThreadPrincipal, propogateCallContext,
                            propogateHttpContext, propogateCASMarkers );
            }

            #region IWorkRequest Members

            public bool Cancel()
            {
                // If the work request was pending, mark it cancelled.  Otherwise,
                // this method was called too late.  Note that this call can
                // cancel an operation without any race conditions.  But if the
                // result of this test-and-set indicates the request is in the
                // "processed" state, it might actually be about to be processed.
                //
                return ( Interlocked.CompareExchange( ref State, Cancelled, Pending ) == Pending );
            }

            #endregion

            private void Initialize( bool propogateThreadPrincipal, bool propogateCallContext,
                                     bool propogateHttpContext, bool propogateCASMarkers )
            {
                WorkingTime = TimeStampStarted = DateTime.Now;
                InternalThreadInfo = ThreadInfo.Capture( propogateThreadPrincipal, propogateCallContext,
                                                         propogateHttpContext, propogateCASMarkers );
            }
        }

        #endregion

        #region ThreadPool.ThreadWrapper

        private class ThreadWrapper
        {
            private readonly bool _isPermanent;
            private readonly string _name;
            private readonly ThreadPool _pool;
            private readonly ThreadPriority _priority;

            public ThreadWrapper( ThreadPool pool, bool isPermanent,
                                  ThreadPriority priority, string name )
            {
                _pool = pool;
                _isPermanent = isPermanent;
                _priority = priority;
                _name = name;

                lock ( pool )
                {
                    // Update the total # of threads in the pool.
                    //
                    pool._currentThreadCount++;
                }
            }

            public void Start()
            {
                var t = new Thread( ThreadProc )
                            {
                                Name = _name,
                                Priority = _priority,
                                IsBackground = _pool._useBackgroundThreads
                            };
                t.SetApartmentState( ApartmentState.MTA );
                t.Start();
            }

            private void ThreadProc()
            {
                Debug.WriteLine( string.Format( "[{0}, {1}] Worker thread started",
                                                Thread.CurrentThread.ManagedThreadId, Thread.CurrentThread.Name ) );

                var done = false;

                while ( !done )
                {
                    WorkRequest wr = null;
                    ThreadWrapper newThread = null;

                    lock ( _pool )
                    {
                        // As long as the request queue is empty and a shutdown hasn't
                        // been initiated, wait for a new work request to arrive.
                        //
                        var timedOut = false;

                        while ( !_pool._stopInProgress && !timedOut && ( _pool._requestQueue.Count == 0 ) )
                        {
                            if ( !Monitor.Wait( _pool, ( _isPermanent ? Timeout.Infinite : _pool._decayTime ) ) )
                            {
                                // Timed out waiting for something to do.  Only dynamically created
                                // threads will get here, so bail out.
                                //
                                timedOut = true;
                            }
                        }

                        // We exited the loop above because one of the following conditions
                        // was met:
                        //   - ThreadPool.Stop was called to initiate a shutdown.
                        //   - A dynamic thread timed out waiting for a work request to arrive.
                        //   - There are items in the work queue to process.

                        // If we exited the loop because there's work to be done,
                        // a shutdown hasn't been initiated, and we aren't a dynamic thread
                        // that timed out, pull the request off the queue and prepare to
                        // process it.
                        //
                        if ( !_pool._stopInProgress && !timedOut && ( _pool._requestQueue.Count > 0 ) )
                        {
                            wr = ( WorkRequest ) _pool._requestQueue.Dequeue();
                            Debug.Assert( wr != null );

                            // Check to see if this work request languished in the queue
                            // very long.  If it was in the queue >= the new thread trigger
                            // time, and if we haven't reached the max thread count cap,
                            // add a new thread to the pool.
                            //
                            // If the decision is made, create the new thread object (updating
                            // the current # of threads in the pool), but defer starting the new
                            // thread until the lock is released.
                            //
                            if ( wr != null )
                            {
                                var requestTimeInQ = DateTime.Now.Subtract( wr.WorkingTime );

                                if ( ( requestTimeInQ >= _pool._newThreadTrigger ) &&
                                     ( _pool._currentThreadCount < _pool._maxThreadCount ) )
                                {
                                    // Note - the constructor for ThreadWrapper will update
                                    // pool.currentThreadCount.
                                    //
                                    newThread =
                                        new ThreadWrapper( _pool, false, _priority,
                                                           string.Format( "{0} (dynamic)", _pool._threadPoolName ) );

                                    // Since the current request we just dequeued is stale,
                                    // everything else behind it in the queue is also stale.
                                    // So reset the timestamps of the remaining pending work
                                    // requests so that we don't start creating threads
                                    // for every subsequent request.
                                    //
                                    _pool.ResetWorkRequestTimes();
                                }
                            }
                        }
                        else
                        {
                            // Should only get here if this is a dynamic thread that
                            // timed out waiting for a work request, or if the pool
                            // is shutting down.
                            //
                            Debug.Assert( ( timedOut && !_isPermanent ) || _pool._stopInProgress );
                            _pool._currentThreadCount--;

                            if ( _pool._currentThreadCount == 0 )
                            {
                                // Last one out turns off the lights.
                                //
                                Debug.Assert( _pool._stopInProgress );

                                if ( _pool.Stopped != null )
                                {
                                    _pool.Stopped();
                                }

                                _pool._stopCompleteEvent.Set();
                            }

                            done = true;
                        }
                    } // lock

                    // No longer holding pool lock here...

                    if ( !done && ( wr != null ) )
                    {
                        // Check to see if this request has been cancelled while
                        // stuck in the work queue.
                        //
                        // If the work request was pending, mark it processed and proceed
                        // to handle.  Otherwise, the request must have been cancelled
                        // before we plucked it off the request queue.
                        //
                        if ( Interlocked.CompareExchange( ref wr.State, WorkRequest.Processed, WorkRequest.Pending ) !=
                             WorkRequest.Pending )
                        {
                            // Request was cancelled before we could get here.
                            // Bail out.
                            continue;
                        }

                        if ( newThread != null )
                        {
                            Debug.WriteLine( string.Format( "[{0}, {1}] Adding dynamic thread to pool",
                                                            Thread.CurrentThread.ManagedThreadId, Thread.CurrentThread.Name ) );
                            newThread.Start();
                        }

                        // Dispatch the work request.
                        //
                        ThreadInfo originalThreadInfo = null;

                        try
                        {
                            // Impersonate (as much as possible) what we know about
                            // the thread that issued the work request.
                            //
                            originalThreadInfo = ThreadInfo.Impersonate( wr.InternalThreadInfo );

                            var targetProc = wr.TargetProc as WorkRequestDelegate;

                            if ( targetProc != null )
                            {
                                targetProc( wr.ProcArg, wr.TimeStampStarted );
                            }
                            else
                            {
                                wr.TargetProc.DynamicInvoke( wr.ProcArgs );
                            }
                        }
                        catch ( Exception e )
                        {
                            Debug.WriteLine( string.Format( "Exception thrown performing callback:\n{0}\n{1}", e.Message,
                                                            e.StackTrace ) );
                        }
                        finally
                        {
                            // Restore our worker thread's identity.
                            //
                            ThreadInfo.Restore( originalThreadInfo );
                        }
                    }
                }

                Debug.WriteLine( string.Format( "[{0}, {1}] Worker thread exiting pool",
                                                Thread.CurrentThread.ManagedThreadId, Thread.CurrentThread.Name ) );
            }
        }

        #endregion

        #region ThreadPool.Stop and InternalStop

        public void Stop()
        {
            InternalStop( false, Timeout.Infinite );
        }

        public void StopAndWait()
        {
            InternalStop( true, Timeout.Infinite );
        }

        public bool StopAndWait( int timeout )
        {
            return InternalStop( true, timeout );
        }

        private bool InternalStop( bool wait, int timeout )
        {
            if ( !_hasBeenStarted )
            {
                throw new InvalidOperationException( "Cannot stop a thread pool that has not been started yet." );
            }

            lock ( this )
            {
                Debug.WriteLine( string.Format( "[{0}, {1}] Stopping pool (# threads = {2})", Thread.CurrentThread.ManagedThreadId, Thread.CurrentThread.Name, _currentThreadCount ) );
                _stopInProgress = true;
                Monitor.PulseAll( this );
            }

            if ( wait )
            {
                var stopComplete = WaitOne( timeout, true );

                if ( stopComplete )
                {
                    // If the stop was successful, we can support being
                    // to be restarted.  If the stop was requested, but not
                    // waited on, then we don't support restarting.
                    //
                    _hasBeenStarted = false;
                    _stopInProgress = false;
                    _requestQueue.Clear();
                    _stopCompleteEvent.Reset();
                }

                return ( stopComplete );
            }

            return ( true );
        }

        #endregion

        #region ThreadPool.PostRequest(early bound)

        // Overloads for the early bound WorkRequestDelegate-based targets.
        //
        public bool PostRequest( WorkRequestDelegate cb )
        {
            return PostRequest( cb, ( object ) null );
        }

        public bool PostRequest( WorkRequestDelegate cb, object state )
        {
            IWorkRequest notUsed;
            return PostRequest( cb, state, out notUsed );
        }

        public bool PostRequest( WorkRequestDelegate cb, object state, out IWorkRequest reqStatus )
        {
            var request = new WorkRequest( cb, state,
                                           _propogateThreadPrincipal, _propogateCallContext,
                                           _propogateHttpContext, PropogateCasMarkers );
            reqStatus = request;
            return PostRequest( request );
        }

        #endregion

        #region ThreadPool.PostRequest(late bound)

        // Overloads for the late bound Delegate.DynamicInvoke-based targets.
        //
        public bool PostRequest( Delegate cb, object[] args )
        {
            IWorkRequest notUsed;
            return PostRequest( cb, args, out notUsed );
        }

        public bool PostRequest( Delegate cb, object[] args, out IWorkRequest reqStatus )
        {
            var request = new WorkRequest( cb, args,
                                           _propogateThreadPrincipal, _propogateCallContext,
                                           _propogateHttpContext, PropogateCasMarkers );
            reqStatus = request;
            return PostRequest( request );
        }

        #endregion
    }

    #endregion
}