#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.SessionState;
using System.Xml;
using System.Xml.Xsl;
using NetJsWire.JsonRpcHandler.Util;
using Newtonsoft.Json;

namespace NetJsWire.JsonRpcHandler
{
    /// <summary>
    ///   The class responsible of holding object references and render the JavaScript to the clients.
    /// </summary>
    public class WebApplication
    {
        #region Delegates

        /// <summary>
        ///   For status requests.
        /// </summary>
        /// <param name = "sender">Allways the instance of WebApplication</param>
        /// <param name = "args">If the event was handled by the user and the HTML result in it.</param>
        public delegate void StatusRequestEventHandler( object sender, StatusRequestEventArgs args );

        #endregion

        private const string BaseXml = "<?xml version=\"1.0\"?><class><precode/><controls/><code/><propertys/></class>";

        private static WebApplication _instance;
        public static string SessionEndedErrorTitle = "The session has ended";
        public static string SessionEndedErrorText = "Please reload the page.";
        public static string ObjectNotFoundErrorText = "The object is not in the database.";

        private readonly List<string> _directChanges;
        private readonly Dictionary<string, object> _objects;

        private readonly XmlDocument _xmlDocument;
        private readonly XslCompiledTransform _xt = new XslCompiledTransform( true );
        private string _clientHostName = string.Empty;
        private string _clientIp = string.Empty;
        private string _clientUserAgent = string.Empty;
        private bool _formPostSuccess = true;
        private bool _isNull = true;

        private DateTime _lastAccess = DateTime.Now;

        private SettingsHandler _settingsHandler =
            ( SettingsHandler ) ConfigurationManager.GetSection( "NetJsWireSettings/NetJsWire" );

        /// <summary>
        ///   Initializes a new instance of WebApplication.
        /// </summary>
        /// <param name = "sessId">The Session ID to pass for the monitoring collection.</param>
        public WebApplication( string sessId )
        {
#if DEBUG
            //newLine = Environment.NewLine;
#endif

            _objects = new Dictionary<string, object>();
            _directChanges = new List<string>();
            _xmlDocument = new XmlDocument();
            _xmlDocument.LoadXml( BaseXml );
            var doc = new XmlDocument();
            doc.Load( GetType().Assembly.GetManifestResourceStream( "NetJsWire.Xsl.Class2.xslt" ) );
            _xt.Load( doc );

            if ( sessId != null )
            {
                //WebApps.AddObject(sessId, this);
            }
        }

        ///<summary>
        ///  If true we only going to render changes or the requesting grid.
        ///</summary>
        public bool OnlyGridData { get; set; }

        ///<summary>
        ///  The ID of the grid requesting data.
        ///</summary>
        public string IdGridRequesting { get; set; }

        /// <summary>
        ///   The SettingsHandler for the application.
        /// </summary>
        public SettingsHandler SettingsHandler
        {
            get { return _settingsHandler; }
            set { _settingsHandler = value; }
        }

        /// <summary>
        ///   True if the client script is already rendered.
        /// </summary>
        public bool ClientScriptRendered { get; set; }

        /// <summary>
        ///   The IP of the connected client.
        /// </summary>
        public string ClientIP
        {
            get { return _clientIp; }
            set { _clientIp = value; }
        }

        /// <summary>
        ///   The host name of the connected client.
        /// </summary>
        public string ClientHostName
        {
            get { return _clientHostName; }
            set { _clientHostName = value; }
        }

        /// <summary>
        ///   The browser capabilities of the connected client.
        /// </summary>
        public HttpBrowserCapabilities ClientBrowser { get; set; }

        /// <summary>
        ///   The user agent of the connected client.
        /// </summary>
        public string ClientUserAgent
        {
            get { return _clientUserAgent; }
            set { _clientUserAgent = value; }
        }

        internal bool IsNull
        {
            get { return _isNull; }
        }

        ///<summary>
        ///  Gets the current HttpApplication instance.
        ///</summary>
        public static HttpApplication Application
        {
            get { return HttpContext.Current.ApplicationInstance; }
        }

        ///<summary>
        ///  Gets the current Session.
        ///</summary>
        public HttpSessionState Session
        {
            get { return HttpContext.Current != null ? HttpContext.Current.Session : null; }
        }

        ///<summary>
        ///  Gets the current Context.
        ///</summary>
        public HttpContext Context
        {
            get { return HttpContext.Current; }
        }

        ///<summary>
        ///  Gets or sets the running instance.
        ///</summary>
        public static WebApplication Instance
        {
            get
            {
                if ( HttpContext.Current != null && HttpContext.Current.ApplicationInstance != null &&
                     HttpContext.Current.ApplicationInstance.Session[ "WebApplication" ] != null )
                    return HttpContext.Current.ApplicationInstance.Session[ "WebApplication" ] as WebApplication;

                // this is to allow designer to work
                return _instance ?? ( _instance = new WebApplication( null ) { _isNull = true } );
            }
            set
            {
                if ( HttpContext.Current != null && HttpContext.Current.ApplicationInstance != null )
                {
                    HttpContext.Current.ApplicationInstance.Session[ "WebApplication" ] = value;

                    if ( value != null )
                        ( ( WebApplication ) HttpContext.Current.ApplicationInstance.Session[ "WebApplication" ] ).
                            _isNull = false;
                }
            }
        }

        ///<summary>
        ///  The current XmlDocument that holds objects definitions.
        ///</summary>
        public XmlDocument XmlDocument
        {
            get { return _xmlDocument; }
        }

        ///<summary>
        ///  Gets or sets the <see cref = "IClientDocument" /> of the application.
        ///</summary>
        public IClientDocument ClientDocument { get; set; }

        /// <summary>
        ///   The objects on the application, those are your Widgets.
        /// </summary>
        public Dictionary<string, object> Objects
        {
            get { return _objects; }
        }

        /// <summary>
        ///   True if there is any pending change to be rendered.
        /// </summary>
        public bool HasChanges { get; set; }

        internal List<string> DirectChanges
        {
            get { return _directChanges; }
        }

        public object UserStorage { get; set; }

        public DateTime LastAccess
        {
            get { return _lastAccess; }
            set { _lastAccess = value; }
        }

        public bool FormPostSuccess
        {
            get { return _formPostSuccess; }
            set { _formPostSuccess = value; }
        }

        public event EventHandler Unload;
        public event UnhandledExceptionEventHandler UnhandledException;
        public event EventHandler SessionEnded;

        /// <summary>
        ///   Suscribe to this event to provide your own status page instead of the default one.
        /// </summary>
        public event StatusRequestEventHandler StatusRequest;

        public void RaiseSessionEnded()
        {
            var sessEnd = SessionEnded;
            if ( null != sessEnd )
            {
                sessEnd( this, EventArgs.Empty );
            }
        }

        public virtual void OnUnhandledException( object sender, UnhandledExceptionEventArgs e )
        {
            try
            {
                var evtLog = new EventLog( "Application" ) { MachineName = ".", Source = "NetJsWire" }; // Event Log type

                var ex = ( Exception ) e.ExceptionObject;

                evtLog.WriteEntry( "Unhandled exception: " + ex.Message + Environment.NewLine + "StackTrace: " +
                                   ex.StackTrace + Environment.NewLine +
                                   ( ex.InnerException != null
                                         ? "InnerException: " + ex.InnerException.Message + Environment.NewLine +
                                           "StackTrace: " + ex.InnerException.StackTrace
                                         : "" ) );
                evtLog.Close();
            }
            catch
            {
            }

            object unHe = UnhandledException;
            if ( unHe != null )
            {
                UnhandledException( sender, e );
            }
        }

        /// <summary>
        ///   Event that raises when the application throws an uncaught Exception.
        /// </summary>
        public event EventHandler WebApplicationError;

        /// <summary>
        ///   Swaps a style sheet.
        /// </summary>
        /// <param name = "cssId">The id of an existing link tag to remove.</param>
        /// <param name = "cssUrl">The href of the new stylesheet to include</param>
        public void SwapStyleSheet( string cssId, string cssUrl )
        {
            AddChange( string.Format( "Ext.util.CSS.swapStyleSheet('{0}', '{1}');", cssId, cssUrl ) );
        }

        /// <summary>
        ///   Removes a style or link tag by id.
        /// </summary>
        /// <param name = "cssId">The id of the tag.</param>
        public void RemoveStyleSheet( string cssId )
        {
            //refreshCache()
            AddChange( string.Format( "Ext.util.CSS.removeStyleSheet('{0}');", cssId ) );
            AddChange( "Ext.util.CSS.refreshCache();" );
        }

        ///<summary>
        ///  Sets the current context between asyn calls.
        ///</summary>
        ///<param name = "context">The real context coming from the async call.</param>
        public static void SetInstanceContext( HttpContext context )
        {
            HttpContext.Current = context;
        }

        ///<summary>
        ///  Adds and object the the WebApplication objects collection so it becomes ready for rendering and events from clients.
        ///</summary>
        ///<param name = "key">The internal name of the object (InternalID).</param>
        ///<param name = "obj">The object itself.</param>
        public void AddObject( string key, object obj )
        {
            lock ( _objects )
            {
                if ( !_objects.ContainsKey( key ) )
                {
                    _objects.Add( key, obj );
                }
            }
        }

        ///<summary>
        ///  Removes an object from the WebApplication objects collection.
        ///</summary>
        ///<param name = "key">The internal name of the object (InternalID).</param>
        public void RemoveObject( string key )
        {
            lock ( _objects )
            {
                if ( key == null || !_objects.ContainsKey( key ) )
                {
                    return;
                }

                var ctrlNode =
                    XmlDocument.DocumentElement.SelectSingleNode( "//class//controls//control[@id='" + key + "']" );
                if ( ctrlNode != null )
                {
                    XmlDocument[ "class" ][ "controls" ].RemoveChild( ctrlNode );
                }
                _objects.Remove( key );
            }
        }

        /// <summary>
        ///   Opens a native (Browser) window.
        /// </summary>
        /// <param name = "url">The url to load.</param>
        public void OpenNativeWindow( string url )
        {
            var change = string.Format( "NetJsWire.OpenWindow('{0}');", url );
            XmlDocument[ "class" ][ "code" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).
                InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   Opens a native (Browser) window.
        /// </summary>
        /// <param name = "url">The url to load.</param>
        /// <param name = "width">The width of the window.</param>
        /// <param name = "height">The Heigth of the window.</param>
        public void OpenNativeWindow( string url, int width, int height )
        {
            var change = string.Format( "NetJsWire.OpenWindow('{0}', {1}, {2});", url, width, height );
            XmlDocument[ "class" ][ "code" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).
                InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   Adds a change to the subtle XMLDocument.
        /// </summary>
        /// <param name = "change">The change to add.</param>
        public void AddChange( string change )
        {
            XmlDocument[ "class" ][ "code" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).
                InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   Adds a precode change to the subtle XMLDocument.
        ///   <remarks>
        ///     A precode change is going to get rendered before the normal changes.
        ///   </remarks>
        /// </summary>
        /// <param name = "change">The change to add.</param>
        public void AddPreCodeChange( string change )
        {
            XmlDocument[ "class" ][ "precode" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).
                InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   A change that will gets replaced if you put it twice.
        /// </summary>
        /// <param name = "change"></param>
        public void AddSingleChange( string change )
        {
            if ( XmlDocument[ "class" ][ "code" ].HasChildNodes )
            {
                if ( XmlDocument[ "class" ][ "code" ].ChildNodes.Cast< XmlNode >().Any( node => node.InnerText == change ) )
                {
                    return;
                }
            }
            XmlDocument[ "class" ][ "code" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   Adds a single precode change to the subtle XMLDocument.
        /// </summary>
        /// <remarks>
        ///   A precode change is going to get rendered before the normal changes.
        ///   This change will get replaced if you put it twice.
        /// </remarks>
        /// <param name = "change">The change to add.</param>
        public void AddSinglePreCodeChange( string change )
        {
            if ( XmlDocument[ "class" ][ "precode" ].HasChildNodes )
            {
                if ( XmlDocument[ "class" ][ "precode" ].ChildNodes.Cast< XmlNode >().Any( node => node.InnerText == change ) )
                {
                    return;
                }
            }
            XmlDocument[ "class" ][ "precode" ].AppendChild( XmlDocument.CreateNode( "element", "codeline", "" ) ).
                InnerText = change;
            HasChanges = true;
        }

        /// <summary>
        ///   Render changes and transform it to JavaScript.
        /// </summary>
        /// <returns>The JavaScript code to render on the client.</returns>
        public string RenderChanges()
        {
            _lastAccess = DateTime.Now;

            if ( !HasChanges && !OnlyGridData )
            {
                return string.Empty;
            }

            var sw = new StringWriter();

            if ( !OnlyGridData )
            {
                lock ( Objects )
                {
                    foreach ( dynamic o in Objects.Values )
                    {
                        if ( o is IWidget )
                        {
                            o.Renderer.Render();
                        }
                    }
                }

                using ( var sr = new StringReader( _xmlDocument.InnerXml ) )
                {
                    var tr = new XmlTextReader( sr );
                    _xt.Transform( tr, null, sw );
                }

                _xmlDocument.LoadXml( BaseXml );
                HasChanges = false;
            }
            else //TODO: Some cleaning here...
            {
                var gridRequesting = Objects[ IdGridRequesting ] as IWidget;

                if ( gridRequesting != null )
                {
                    using ( var sr = new StringReader( gridRequesting.Renderer.ControlNode.InnerXml ) )
                    {
                        var nt = new NameTable();
                        var nsmgr = new XmlNamespaceManager( nt );
                        var context = new XmlParserContext( null, nsmgr, null, XmlSpace.None );
                        var settings = new XmlReaderSettings
                                           {
                                               ConformanceLevel = ConformanceLevel.Fragment
                                           };
                        var reader = XmlReader.Create( sr, settings, context );
                        _xt.Transform( reader, null, sw );
                    }
                    if ( gridRequesting.Renderer.ControlNode[ "griddata" ] != null ) gridRequesting.Renderer.ControlNode[ "griddata" ].RemoveAll();
                }
                OnlyGridData = false;
                IdGridRequesting = null;

                var res = sw.ToString();

                if ( string.IsNullOrWhiteSpace( res ) )
                {
                    sw.Write( "{totalCount:0,results:[]}" );
                }
            }

            return sw.ToString();
        }

        /// <summary>
        ///   Method that gets called in case of a HTTPApplication Error.
        ///   You can capture this errors by subscribing to <see cref = "WebApplicationError" />.
        /// </summary>
        /// <param name = "sender">The sender.</param>
        /// <param name = "e">The EventArgs.</param>
        public void ApplicationError( object sender, EventArgs e )
        {
            object appError = WebApplicationError;

            if ( appError != null )
            {
                WebApplicationError( this, EventArgs.Empty );
            }
            else
            {
                var ex = Application.Server.GetLastError();
                if ( ex != null )
                {
                    AddChange( string.Format( "ShowException('{0}');", JsonConvert.ToString( ex.Message ) ) );
                    HasChanges = true;
                }
            }
        }

        internal void OnUnload()
        {
            object unl = Unload;
            if ( unl != null )
            {
                Unload( this, EventArgs.Empty );
            }
            UserStorage = null;
        }

        internal string OnStatusRequest()
        {
            object sReq = StatusRequest;
            if ( sReq != null )
            {
                var args = new StatusRequestEventArgs( null, false );
                StatusRequest( this, args );
                if ( args.Handled )
                {
                    return args.Result;
                }
            }

            return null;
        }
    }
}