﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.Util;
using NetJsWire.Widgets.Ui.Layout;
using ToolBar = NetJsWire.Widgets.Ui.ToolBar.ToolBar;

namespace NetJsWire.Widgets.Config
{
    [TypeConverter( typeof( ExpandableObjectConverter ) )]
    [PropertyTabAttribute( typeof( ConfigOptionsPropertyTab ), PropertyTabScope.Document )]
    public partial interface IConfigOptions
    {
        /// <summary>
        ///   <b>Note:</b> this config is only used when this Component is rendered by a Container which has been configured to use an AnchorLayout based layout manager, for example:
        ///   Ext.form.FormPanel
        ///   specifying layout: 'anchor' // or 'form', or 'absolute'
        ///   See Ext.layout.AnchorLayout.anchor also.
        /// </summary>
        [Description( "This config is only used when this Component is rendered by a Container which has been configured to use an AnchorLayout based layout manager" )]
        [DefaultValue( ( string ) null )]
        string Anchor { get; set; }

        /// <summary>
        ///   Specify the id of the element, a DOM element or an existing Element corresponding to a DIV that is already present in the document that specifies some structural markup for this component.
        ///   Description :
        ///   When applyTo is used, constituent parts of the component can also be specified by id or CSS class name within the main element, and the component being created may attempt to create its subcomponents from that markup if applicable.
        ///   Notes :
        ///   When using this config, a call to render() is not required.
        ///   If applyTo is specified, any value passed for renderTo will be ignored and the target element's parent node will automatically be used as the component's container.
        /// </summary>
        [Description( "Specify the id of the element, a DOM element or an existing Element corresponding to a DIV that is already present in the document that specifies some structural markup for this component." )]
        [DefaultValue( ( string ) null )]
        string ApplyTo { get; set; }

        /// <summary>
        ///   A tag name or DomHelper spec used to create the Element which will encapsulate this Component.
        ///   You do not normally need to specify this. For the base classes Ext.Component, Ext.BoxComponent, and Ext.Container, this defaults to 'div'. The more complex Ext classes use a more complex DOM structure created by their own onRender methods.
        ///   This is intended to allow the developer to create application-specific utility Components encapsulated by different DOM elements. Example usage:
        ///   {
        ///   xtype: 'box',
        ///   autoEl: {
        ///   tag: 'img',
        ///   src: 'http://www.example.com/example.jpg'
        ///   }
        ///   }, {
        ///   xtype: 'box',
        ///   autoEl: {
        ///   tag: 'blockquote',
        ///   html: 'autoEl is cool!'
        ///   }
        ///   }, {
        ///   xtype: 'container',
        ///   autoEl: 'ul',
        ///   cls: 'ux-unordered-list',
        ///   items: {
        ///   xtype: 'box',
        ///   autoEl: 'li',
        ///   html: 'First list item'
        ///   }
        ///   }
        /// </summary>
        [Description( "A tag name or DomHelper spec used to create the Element which will encapsulate this Component." )]
        [DefaultValue( ( string ) null )]
        string AutoEl { get; set; }

        /// <summary>
        ///   True if the component should check for hidden classes (e.g. 'x-hidden' or 'x-hide-display') and remove them on render (defaults to false).
        /// </summary>
        [Description( "True if the component should check for hidden classes (e.g. 'x-hidden' or 'x-hide-display') and remove them on render (defaults to false)." )]
        [DefaultValue( false )]
        bool AutoShow { get; set; }

        /// <summary>
        ///   CSS class added to the component when it is disabled (defaults to 'x-item-disabled').
        /// </summary>
        [Description( "CSS class added to the component when it is disabled (defaults to 'x-item-disabled')." )]
        [DefaultValue( ( string ) null )]
        string DisabledClass { get; set; }

        /// <summary>
        ///   <b>Note: this config is only used when this Component is rendered by a Container which has been configured to use a BoxLayout.</b>
        ///   Each child Component with a flex property will be flexed either vertically (by a VBoxLayout) or horizontally (by an HBoxLayout) according to the item's relative flex value compared to the sum of all Components with flex value specified. Any child items that have either a flex = 0 or flex = undefined will not be 'flexed' (the initial size will not be changed).
        /// </summary>
        [Description( "Each child Component with a flex property will be flexed either vertically (by a VBoxLayout) or horizontally (by an HBoxLayout) according to the item's relative flex value compared to the sum of all Components with flex value specified." )]
        [DefaultValue( 0D )]
        double Flex { get; set; }

        /// <summary>
        ///   The label text to display next to this Component (defaults to '').
        ///   <remarks>
        ///     Note: this config is only used when this Component is rendered by a Container which has been configured to use the FormLayout layout manager (e.g. Ext.form.FormPanel or specifying layout:'form').
        ///     Also see hideLabel and Ext.layout.FormLayout.fieldTpl.
        ///   </remarks>
        ///   <example>
        ///     Example use:
        ///     <code>
        ///       new Ext.FormPanel({
        ///       height: 100,
        ///       renderTo: Ext.getBody(),
        ///       items: [{
        ///       xtype: 'textfield',
        ///       fieldLabel: 'Name'
        ///       }]
        ///       });
        ///     </code>
        ///   </example>
        /// </summary>
        [Description( "The label text to display next to this Component (defaults to '')." )]
        [DefaultValue( ( string ) null )]
        string FieldLabel { get; set; }

        /// <summary>
        ///   The CSS class used to to apply to the special clearing div rendered directly after each form field wrapper to provide field clearing (defaults to 'x-form-clear-left').
        ///   <remarks>
        ///     Note: this config is only used when this Component is rendered by a Container which has been configured to use the FormLayout layout manager 
        ///     (e.g. Ext.form.FormPanel or specifying layout:'form') and either a fieldLabel is specified or isFormField=true is specified.
        ///   </remarks>
        /// </summary>
        [Description( "The CSS class used to to apply to the special clearing div rendered directly after each form field wrapper to provide field clearing (defaults to 'x-form-clear-left')." )]
        [DefaultValue( ( string ) null )]
        string ClearCls { get; set; }

        /// <summary>
        ///   An optional extra CSS class that will be added to this component's Element (defaults to ''). 
        ///   This can be useful for adding customized styles to the component or any of its children using standard CSS rules.
        /// </summary>
        [Description( "An optional extra CSS class that will be added to this component's Element (defaults to '')." )]
        [DefaultValue( ( string ) null )]
        string Cls { get; set; }

        /// <summary>
        ///   An HTML fragment, or a DomHelper specification to use as the layout element content (defaults to '').
        ///   The HTML content is added after the component is rendered, so the document will not contain this HTML at the time the render event is fired. This content is inserted into the body before any configured contentEl is appended.
        /// </summary>
        [Description( "An HTML fragment, or a DomHelper specification to use as the layout element content (defaults to '')." )]
        [DefaultValue( ( string ) null )]
        string Html { get; set; }

        /// <summary>
        ///   An additional CSS class to apply to the div wrapping the form item element of this field. If supplied, itemCls at the field level will override the default itemCls supplied at the container level. The value specified for itemCls will be added to the default class ('x-form-item').
        ///   Since it is applied to the item wrapper (see Ext.layout.FormLayout.fieldTpl), it allows you to write standard CSS rules that can apply to the field, the label (if specified), or any other element within the markup for the field.
        ///   <remarks>
        ///     This config is only used when this Component is rendered by a Container which has been configured to use the FormLayout layout manager (e.g. Ext.form.FormPanel or specifying layout:'form').
        ///   </remarks>
        /// </summary>
        [Description( "An additional CSS class to apply to the div wrapping the form item element of this field." )]
        [DefaultValue( ( string ) null )]
        string ItemCls { get; set; }

        /// <summary>
        ///   The layout type to be used for this container.
        ///   <remarks>
        ///     *Important: In order for child items to be correctly sized and positioned, typically a layout manager must be specified through the layout configuration option.
        ///   </remarks>
        ///   The sizing and positioning of child items is the responsibility of the Container's layout manager which creates and manages the type of layout you have in mind.
        ///   For example:
        ///   <example>
        ///     <code>
        ///       new Ext.Window({
        ///       width:300, height: 300,
        ///       layout: 'fit', // explicitly set layout manager: override the default (layout:'auto')
        ///       items: [{
        ///       title: 'Panel inside a Window'
        ///       }]
        ///       }).show();
        ///     </code>
        ///   </example>
        ///   If the layout configuration is not explicitly specified for a general purpose container (e.g. Container or Panel) the default layout manager will be used 
        ///   which does nothing but render child components sequentially into the Container (no sizing or positioning will be performed in this situation).
        ///   Some container classes implicitly specify a default layout (e.g. FormPanel specifies layout:'form'). Other specific purpose classes internally 
        ///   specify/manage their internal layout (e.g. GridPanel, TabPanel, TreePanel, Toolbar, Menu, etc.).
        /// </summary>
        [Description( "The layout type to be used for this container." )]
        [DefaultValue( LayoutType.Auto )]
        LayoutType Layout { get; set; }

        /// <summary>
        ///   This is a config object containing properties specific to the chosen <see cref = "Layout" /> if <see cref = "Layout" /> has been specified as a string.
        /// </summary>
        [Description( "This is a config object containing properties specific to the chosen layout if layout has been specified as a string." )]
        [DefaultValue( ( string ) null )]
        string LayoutConfig { get; set; }

        /// <summary>
        ///   An array of events that, when fired, should trigger this component to save its state (defaults to none).
        ///   <remarks>
        ///     stateEvents may be any type of event supported by this component, including browser or custom events (e.g., ['click', 'customerchange']).
        ///   </remarks>
        ///   See <see cref = "StateFul" /> for an explanation of saving and restoring Component state.
        /// </summary>
        [Description( "An array of events that, when fired, should trigger this component to save its state (defaults to none)." )]
        [DefaultValue( ( string ) null )]
        string StateEvents { get; set; }

        /// <summary>
        ///   The unique id for this component to use for state management purposes (defaults to the component id if one was set, otherwise null if the component is using a generated id).
        /// </summary>
        [Description( "The unique id for this component to use for state management purposes (defaults to the component id if one was set, otherwise null if the component is using a generated id)." )]
        [DefaultValue( ( string ) null )]
        string StateId { get; set; }

        /// <summary>
        ///   A flag which causes the Component to attempt to restore the state of internal properties from a saved state on startup.
        ///   The component must have either a stateId or id assigned for state to be managed. Auto-generated ids are not guaranteed to be stable across page loads and cannot be relied upon to save and restore the same state for a component.
        ///   <remarks>
        ///     For state saving to work, the state manager's provider must have been set to an implementation of Ext.state.Provider which overrides the set and get methods to save and recall name/value pairs. A built-in implementation, Ext.state.CookieProvider is available.
        ///     To set the state provider for the current page:
        ///     <code>
        ///       Ext.state.Manager.setProvider(new Ext.state.CookieProvider({
        ///       expires: new Date(new Date().getTime()+(1000*60*60*24*7)), //7 days from now
        ///       }));
        ///     </code>
        ///     A stateful Component attempts to save state when one of the events listed in the stateEvents configuration fires.
        ///     To save state, a stateful Component first serializes its state by calling getState. By default, this function does nothing. The developer must provide an implementation which returns an object hash which represents the Component's restorable state.
        ///     The value yielded by getState is passed to Ext.state.Manager.set which uses the configured Ext.state.Provider to save the object keyed by the Component's stateId, or, if that is not specified, its id.
        ///     During construction, a stateful Component attempts to restore its state by calling Ext.state.Manager.get passing the stateId, or, if that is not specified, the id.
        ///     The resulting object is passed to applyState. The default implementation of applyState simply copies properties into the object, but a developer may override this to support more behaviour.
        ///     You can perform extra processing on state save and restore by attaching handlers to the beforestaterestore, staterestore, beforestatesave and statesave events.
        ///   </remarks>
        /// </summary>
        [Description( "A flag which causes the Component to attempt to restore the state of internal properties from a saved state on startup." )]
        [DefaultValue( false )]
        bool StateFul { get; set; }

        /// <summary>
        ///   A custom style specification to be applied to this component's Element. Should be a valid argument to Ext.Element.applyStyles.
        /// </summary>
        [Description( "A custom style specification to be applied to this component's Element. Should be a valid argument to Ext.Element.applyStyles." )]
        [DefaultValue( ( string ) null )]
        string Style { get; set; }

        /// <summary>
        ///   An array of tool button configs to be added to the header tool area. When rendered, each tool is stored as an Element referenced by a public property called tools.&lt;tool-type&gt;.
        /// </summary>
        [DefaultValue( null )]
        [Description( "An array of tool button configs to be added to the header tool area. When rendered, each tool is stored as an Element referenced by a public property called tools.&lt;tool-type&gt;." )]
        List<ToolConfig> Tools { get; set; }

        /// <summary>
        ///   An Ext.Template, Ext.XTemplate or an array of strings to form an Ext.XTemplate. Used in conjunction with the data and tplWriteMode configurations.
        /// </summary>
        [Description( "An Ext.Template, Ext.XTemplate or an array of strings to form an Ext.XTemplate. Used in conjunction with the data and tplWriteMode configurations." )]
        [DefaultValue( ( string ) null )]
        string Tpl { get; set; }

        /// <summary>
        ///   The Ext.(X)Template method to use when updating the content area of the Component. Defaults to 'overwrite' (see Ext.XTemplate.overwrite).
        /// </summary>
        [Description( "The Ext.(X)Template method to use when updating the content area of the Component. Defaults to 'overwrite' (see Ext.XTemplate.overwrite)." )]
        [DefaultValue( ( string ) null )]
        string TplWriteMode { get; set; }

        /// <summary>
        ///   The registered xtype to create.
        ///   This config option is not used when passing a config object into a constructor.
        ///   This config option is used only when lazy instantiation is being used, and a child item of a Container is being specified not as a fully instantiated Component, but as a Component config object.
        ///   The xtype will be looked up at render time up to determine what type of child Component to create.
        /// </summary>
        [Description( "The registered xtype to create. This config option is not used when passing a config object into a constructor." )]
        [DefaultValue( ( string ) null )]
        [TypeConverter( typeof( XtypeStringConverter ) )]
        string Xtype { get; set; }

        /// <summary>
        ///   true to animate the transition when the panel is collapsed, false to skip the animation (defaults to true if the Ext.Fx class is available, otherwise false).
        /// </summary>
        [Description( "true to animate the transition when the panel is collapsed, false to skip the animation (defaults to true if the Ext.Fx class is available, otherwise false)." )]
        [DefaultValue( true )]
        bool AnimCollapse { get; set; }

        /// <summary>
        ///   true to use height:'auto', false to use fixed height (defaults to false).
        ///   <remarks>
        ///     Although many components inherit this config option, not all will function as expected with a height of 'auto'. Setting autoHeight:true means 
        ///     that the browser will manage height based on the element's contents, and that Ext will not manage it at all.
        ///     If the browser is managing the height, be aware that resizes performed by the browser in response to changes within the structure of the 
        ///     Component cannot be detected. Therefore changes to the height might result in elements needing to be synchronized with the new height. Example:
        ///     <example>
        ///       <code>
        ///         var w = new Ext.Window({
        ///         title: 'Window',
        ///         width: 600,
        ///         autoHeight: true,
        ///         items: {
        ///         title: 'Collapse Me',
        ///         height: 400,
        ///         collapsible: true,
        ///         border: false,
        ///         listeners: {
        ///         beforecollapse: function() {
        ///         w.el.shadow.hide();
        ///         },
        ///         beforeexpand: function() {
        ///         w.el.shadow.hide();
        ///         },
        ///         collapse: function() {
        ///         w.syncShadow();
        ///         },
        ///         expand: function() {
        ///         w.syncShadow();
        ///         }
        ///         }
        ///         }
        ///         }).show();
        ///       </code>
        ///     </example>
        ///   </remarks>
        /// </summary>
        [Description( "true to use height:'auto', false to use fixed height (defaults to false)." )]
        [DefaultValue( false )]
        bool AutoHeight { get; set; }

        /// <summary>
        ///   A valid url spec according to the Updater Ext.Updater.update method.
        ///   If autoLoad is not null, the panel will attempt to load its contents immediately upon render.
        ///   The URL will become the default URL for this panel's body element, so it may be refreshed at any time.
        /// </summary>
        [Description( "A valid url spec according to the Updater Ext.Updater.update method." )]
        [DefaultValue( ( string ) null )]
        string AutoLoad { get; set; }

        /// <summary>
        ///   The base CSS class to apply to this panel's element (defaults to 'x-panel').
        ///   Another option available by default is to specify 'x-plain' which strips all styling except for required attributes for Ext layouts to function (e.g. overflow:hidden). 
        ///   See <see cref = "Unstyled" /> also.
        /// </summary>
        [Description( "The base CSS class to apply to this panel's element (defaults to 'x-panel')." )]
        [DefaultValue( ( string ) null )]
        string BaseCls { get; set; }

        /// <summary>
        ///   This can be a Ext.Toolbar object, a toolbar config, or an array of buttons/button configs to be added to the toolbar. Note that this is not available as a property after render.
        ///   To access the bottom toolbar after render, use getBottomToolbar.
        ///   <b>Note:</b> Although a Toolbar may contain Field components, these will not be updated by a load of an ancestor FormPanel. A Panel's toolbars are not part of the standard 
        ///   Container->Component hierarchy, and so are not scanned to collect form items. However, the values will be submitted because form submission parameters are collected from the DOM tree.
        /// </summary>
        [Description( "The bottom toolbar of the panel." )]
        [DefaultValue( null )]
        ToolBar BottomBar { get; set; }

        /// <summary>
        ///   A DomHelper element specification object specifying the element structure of this Panel's bbar Element. See <see cref = "BodyCfg" /> also.
        /// </summary>
        [Description( "A DomHelper element specification object specifying the element structure of this Panel's bbar Element." )]
        [DefaultValue( null )]
        string BottomBarCfg { get; set; }

        /// <summary>
        ///   True to display an interior border on the body element of the panel, false to hide it (defaults to true).
        ///   This only applies when border == true. If border == true and bodyBorder == false, the border will display as a 1px wide inset border, giving the entire body element an inset appearance.
        /// </summary>
        [Description( "True to display an interior border on the body element of the panel, false to hide it (defaults to true)." )]
        [DefaultValue( false )]
        bool BodyBorder { get; set; }

        /// <summary>
        ///   A DomHelper element specification object may be specified for any Panel Element.
        ///   By default, the Default element in the table below will be used for the html markup to create a child element with the commensurate Default class name (<see cref = "BaseCls" /> will be replaced by <see cref = "BaseCls" />):
        /// </summary>
        /// <remarks>
        ///   <table>
        ///     <theader>
        ///       <tr>
        ///         <td>Panel Element</td>
        ///         <td>Default Element</td>
        ///         <td>Default class</td>
        ///         <td>Custom Element</td>
        ///         <td>Additional class</td>
        ///         <td>Additional style</td>
        ///       </tr>
        ///     </theader>
        ///     <tbody>
        ///       <tr>
        ///         <td>header</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-header'</td>
        ///         <td>headerCfg</td>
        ///         <td>headerCssClass</td>
        ///         <td>headerStyle</td>
        ///       </tr>
        ///       <tr>
        ///         <td>header</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-bwrap'</td>
        ///         <td>bwrapCfg</td>
        ///         <td>bwrapCssClass</td>
        ///         <td>bwrapStyle</td>
        ///       </tr>
        ///       <tr>
        ///         <td>+ tbar</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-tbar'</td>
        ///         <td>tbarCfg</td>
        ///         <td>tbarCssClass</td>
        ///         <td>tbarStyle</td>
        ///       </tr>
        ///       <tr>
        ///         <td>+ body</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-body'</td>
        ///         <td>bodyCfg</td>
        ///         <td>bodyCssClass</td>
        ///         <td>bodyStyle</td>
        ///       </tr>
        ///       <tr>
        ///         <td>+ bbar</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-bbar'</td>
        ///         <td>bbarCfg</td>
        ///         <td>bbarCssClass</td>
        ///         <td>bbarStyle</td>
        ///       </tr>
        ///       <tr>
        ///         <td>+ footer</td>
        ///         <td>div</td>
        ///         <td>baseCls+'-footer'</td>
        ///         <td>footerCfg</td>
        ///         <td>footerCssClass</td>
        ///         <td>footerStyle</td>
        ///       </tr>
        ///     </tbody>
        ///   </table>
        ///   <example>
        ///     Configuring a Custom element may be used, for example, to force the body Element to use a different form of markup than is created by default.
        ///     An example of this might be to create a child Panel containing a custom content, such as a header, or forcing centering of all Panel content by having the body be a &lt;center&gt; element:
        ///     <code>
        ///       new Ext.Panel({
        ///       title: 'Message Title',
        ///       renderTo: Ext.getBody(),
        ///       width: 200, height: 130,
        ///       bodyCfg: {
        ///       tag: 'center',
        ///       cls: 'x-panel-body',  // Default class not applied if Custom element specified
        ///       html: 'Message'
        ///       },
        ///       footerCfg: {
        ///       tag: 'h2',
        ///       cls: 'x-panel-footer'        // same as the Default class
        ///       html: 'footer html'
        ///       },
        ///       footerCssClass: 'custom-footer', // additional css class, see addClass
        ///       footerStyle:    'background-color:red' // see bodyStyle
        ///       });
        ///     </code>
        ///     The example above also explicitly creates a footer with custom markup and styling applied.
        ///   </example>
        /// </remarks>
        [Description( "A DomHelper element specification object may be specified for any Panel Element." )]
        [DefaultValue( null )]
        string BodyCfg { get; set; }

        /// <summary>
        ///   Additional css class selector to be applied to the body element in the format expected by <c>Ext.Element.addClass</c> (defaults to null). See <see cref = "BodyCfg" />.
        /// </summary>
        [Description( "Additional css class selector to be applied to the body element in the format expected by Ext.Element.addClass (defaults to null)." )]
        [DefaultValue( null )]
        string BodyCssClass { get; set; }

        /// <summary>
        ///   Custom CSS styles to be applied to the body element in the format expected by <c>Ext.Element.applyStyles</c> (defaults to null). See <see cref = "BodyCfg" />.
        /// </summary>
        [Description( "Custom CSS styles to be applied to the body element in the format expected by Ext.Element.applyStyles (defaults to null)." )]
        [DefaultValue( null )]
        string BodyStyle { get; set; }

        /// <summary>
        ///   True to display the borders of the panel's body element, false to hide them (defaults to true).
        ///   By default, the border is a 2px wide inset border, but this can be further altered by setting <see cref = "BodyBorder" /> to false.
        /// </summary>
        [Description( "True to display the borders of the panel's body element, false to hide them (defaults to true)." )]
        [DefaultValue( false )]
        bool Border { get; set; }

        /// <summary>
        ///   The alignment of any buttons added to this panel.
        /// </summary>
        [Description( "The alignment of any buttons added to this panel." )]
        [DefaultValue( ButtonAlign.Right )]
        ButtonAlign ButtonAlign { get; set; }

        /// <summary>
        ///   A DomHelper element specification object specifying the element structure of this Panel's bwrap Element. See <see cref = "BodyCfg" /> also.
        /// </summary>
        [Description( "A DomHelper element specification object specifying the element structure of this Panel's bwrap Element." )]
        [DefaultValue( null )]
        string BwrapCfg { get; set; }

        /// <summary>
        ///   Panels themselves do not directly support being closed, but some Panel subclasses do (like Ext.Window) or a Panel Class within an Ext.TabPanel. Specify true to enable closing in such situations. Defaults to false.
        /// </summary>
        [Description( "Panels themselves do not directly support being closed, but some Panel subclasses do (like Ext.Window) or a Panel Class within an Ext.TabPanel. Specify true to enable closing in such situations. Defaults to false." )]
        [DefaultValue( false )]
        bool Closable { get; set; }

        /// <summary>
        ///   true to make sure the collapse/expand toggle button always renders first (to the left of) any other tools in the panel's title bar, false to render it last (defaults to true).
        /// </summary>
        [Description( "true to make sure the collapse/expand toggle button always renders first (to the left of) any other tools in the panel's title bar, false to render it last (defaults to true)." )]
        [DefaultValue( false )]
        bool CollapseFirst { get; set; }

        /// <summary>
        ///   true to render the panel collapsed, false to render it expanded (defaults to false).
        /// </summary>
        [Description( "true to render the panel collapsed, false to render it expanded (defaults to false)." )]
        [DefaultValue( false )]
        bool Collapsed { get; set; }

        /// <summary>
        ///   True to make the panel collapsible and have the expand/collapse toggle button automatically rendered into the header tool button area, false to keep the panel statically sized with no button (defaults to false).
        /// </summary>
        [Description( "True to make the panel collapsible and have the expand/collapse toggle button automatically rendered into the header tool button area, false to keep the panel statically sized with no button (defaults to false)." )]
        [DefaultValue( false )]
        bool Collapsible { get; set; }

        /// <summary>
        ///   Overrides the <see cref = "BaseCls" /> setting to <see cref = "BaseCls" /><c> = 'x-plain'</c> which renders the panel unstyled except for required attributes for Ext layouts to function (e.g. <c>overflow:hidden</c>).
        /// </summary>
        [Description( "Overrides the baseCls setting to baseCls = 'x-plain' which renders the panel unstyled except for required attributes for Ext layouts to function (e.g. overflow:hidden)." )]
        [DefaultValue( false )]
        bool Unstyled { get; set; }

        /// <summary>
        ///   true to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary, false to clip any overflowing content (defaults to false).
        /// </summary>
        [Description( "true to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary, false to clip any overflowing content (defaults to false)." )]
        [DefaultValue( false )]
        bool AutoScroll { get; set; }

        /// <summary>
        ///   True to use width:'auto', false to use fixed width (or allow it to be managed by its parent Container's layout manager. Defaults to false.
        ///   <remarks>
        ///     Although many components inherit this config option, not all will function as expected with a width of 'auto'. Setting autoWidth:true means 
        ///     that the browser will manage width based on the element's contents, and that Ext will not manage it at all.
        ///     If the browser is managing the width, be aware that resizes performed by the browser in response to changes within the structure of the Component 
        ///     cannot be detected. Therefore changes to the width might result in elements needing to be synchronized 
        ///     with the new width. For example, where the target element is:
        ///     <code>
        ///       <![CDATA[<div id='grid-container' style='margin-left:25%;width:50%'></div>]]>
        ///     </code>
        ///     A Panel rendered into that target element must listen for browser window resize in order to relay its child items when the browser changes its width:
        ///     <code>
        ///       var myPanel = new Ext.Panel({
        ///       renderTo: 'grid-container',
        ///       monitorResize: true, // relay on browser resize
        ///       title: 'Panel',
        ///       height: 400,
        ///       autoWidth: true,
        ///       layout: 'hbox',
        ///       layoutConfig: {
        ///       align: 'stretch'
        ///       },
        ///       defaults: {
        ///       flex: 1
        ///       },
        ///       items: [{
        ///       title: 'Box 1',
        ///       }, {
        ///       title: 'Box 2'
        ///       }, {
        ///       title: 'Box 3'
        ///       }],
        ///       });
        ///     </code>
        ///   </remarks>
        /// </summary>
        [Description( "True to use width:'auto', false to use fixed width (or allow it to be managed by its parent Container's layout manager. Defaults to false." )]
        [DefaultValue( false )]
        bool AutoWidth { get; set; }

        /// <summary>
        ///   The maximum value in pixels which this BoxComponent will set its height to.
        ///   <remarks>
        ///     Warning: This will override any size management applied by layout managers.
        ///   </remarks>
        /// </summary>
        [Description( "The maximum value in pixels which this BoxComponent will set its height to." )]
        [DefaultValue( null )]
        int? BoxMaxHeight { get; set; }

        /// <summary>
        ///   The maximum value in pixels which this BoxComponent will set its width to.
        ///   <remarks>
        ///     Warning: This will override any size management applied by layout managers.
        ///   </remarks>
        /// </summary>
        [Description( "The maximum value in pixels which this BoxComponent will set its width to." )]
        [DefaultValue( null )]
        int? BoxMaxWidth { get; set; }

        /// <summary>
        ///   The minimum value in pixels which this BoxComponent will set its height to.
        ///   <remarks>
        ///     Warning: This will override any size management applied by layout managers.
        ///   </remarks>
        /// </summary>
        [Description( "The minimum value in pixels which this BoxComponent will set its height to." )]
        [DefaultValue( null )]
        int? BoxMinHeight { get; set; }

        /// <summary>
        ///   The minimum value in pixels which this BoxComponent will set its width to.
        ///   <remarks>
        ///     Warning: This will override any size management applied by layout managers.
        ///   </remarks>
        /// </summary>
        [Description( "The minimum value in pixels which this BoxComponent will set its width to." )]
        [DefaultValue( null )]
        int? BoxMinWidth { get; set; }

        /// <summary>
        ///   An object containing margins to apply to this BoxComponent in the format:
        ///   <remarks>
        ///     this config is only used when this BoxComponent is rendered by a Container which has been configured to use the 
        ///     BorderLayout or one of the two BoxLayout subclasses.
        ///   </remarks>
        ///   E.g.:
        ///   <code>
        ///     {
        ///     top: (top margin),
        ///     right: (right margin),
        ///     bottom: (bottom margin),
        ///     left: (left margin)
        ///     }
        ///   </code>
        ///   May also be a string containing space-separated, numeric margin values. The order of the sides associated with each value matches the way CSS processes margin values:
        ///   <list type = "bullet">
        ///     <item>
        ///       If there is only one value, it applies to all sides.
        ///     </item>
        ///     <item>
        ///       If there are two values, the top and bottom borders are set to the first value and the right and left are set to the second.
        ///     </item>
        ///     <item>
        ///       If there are three values, the top is set to the first value, the left and right are set to the second, and the bottom is set to the third.
        ///     </item>
        ///     <item>
        ///       If there are four values, they apply to the top, right, bottom, and left, respectively.
        ///     </item>
        ///   </list>
        ///   Defaults to:
        ///   <code>
        ///     {top:0, right:0, bottom:0, left:0}
        ///   </code>
        /// </summary>
        [Description( "An object containing margins to apply to this BoxComponent." )]
        [DefaultValue( ( string ) null )]
        Padding Margins { get; set; }

        /// <summary>
        ///   An object or array of objects that will provide custom functionality for this component.
        ///   The only requirement for a valid plugin is that it contain an init method that accepts a reference of type Ext.Component.
        ///   When a component is created, if any plugins are available, the component will call the init method on each plugin, passing a reference to itself.
        ///   Each plugin can then call methods or respond to events on the component as needed to provide its functionality.
        /// </summary>
        [Description( "An object or array of objects that will provide custom functionality for this component." )]
        [DefaultValue( null )]
        List<object> Plugins { get; }

        /// <summary>
        ///   The region this Component represents in a BorderLayout.
        ///   <remarks>
        ///     This config is only used when this BoxComponent is rendered by a Container which has been configured to use the BorderLayout layout manager 
        ///     (e.g. specifying layout:'border').
        ///   </remarks>
        /// </summary>
        [Description( "The region this Component represents in a BorderLayout." )]
        [DefaultValue( ( string ) null )]
        string Region { get; set; }

        /// <summary>
        ///   A string component id or the numeric index of the component that should be initially activated within the container's layout on render. 
        ///   For example, <c>activeItem: 'item-1'</c> or <c>activeItem: 0</c> (index 0 = the first item in the container's collection).
        ///   <remarks>
        ///     activeItem only applies to layout styles that can display items one at a time (like Ext.layout.AccordionLayout, Ext.layout.CardLayout and 
        ///     Ext.layout.FitLayout). Related to Ext.layout.ContainerLayout.activeItem.
        ///   </remarks>
        /// </summary>
        [Description( "A string component id or the numeric index of the component that should be initially activated within the container's layout on render." )]
        [DefaultValue( null )]
        object ActiveItem { get; set; }

        /// <summary>
        ///   If true the container will automatically destroy any contained component that is removed from it, else destruction must be handled manually (defaults to true).
        /// </summary>
        [Description( "If true the container will automatically destroy any contained component that is removed from it, else destruction must be handled manually (defaults to true)." )]
        [DefaultValue( false )]
        bool AutoDestroy { get; set; }

        /// <summary>
        ///   An array of events that, when fired, should be bubbled to any parent container.
        /// </summary>
        [Description( "An array of events that, when fired, should be bubbled to any parent container." )]
        [DefaultValue( ( string ) null )]
        string BubbleEvents { get; set; }

        /// <summary>
        ///   When set to true (50 milliseconds) or a number of milliseconds, the layout assigned for this container will buffer the frequency it calculates and does a re-layout of components.
        ///   This is useful for heavy containers or containers with a large quantity of sub-components for which frequent layout calls would be expensive. Defaults to 50.
        /// </summary>
        [Description( "When set to true (50 milliseconds) or a number of milliseconds, the layout assigned for this container will buffer the frequency it calculates and does a re-layout of components." )]
        [DefaultValue( null )]
        int? BufferResize { get; set; }

        /// <summary>
        ///   This option is a means of applying default settings to all added items whether added through the items config or via the add or insert methods.
        ///   If an added item is a config object, and not an instantiated Component, then the default properties are unconditionally applied. If the added item
        ///   is an instantiated Component, then the default properties are applied conditionally so as not to override existing properties in the item.
        ///   If the defaults option is specified as a function, then the function will be called using this Container as the scope (this reference) and passing 
        ///   the added item as the first parameter. Any resulting object from that call is then applied to the item as default properties.
        ///   For example, to automatically apply padding to the body of each of a set of contained Ext.Panel items, you could pass: <c>defaults: {bodyStyle:'padding:15px'}.</c>
        ///   <example>
        ///     <code>
        ///       defaults: {               // defaults are applied to items, not the container
        ///       autoScroll:true
        ///       },
        ///       items: [
        ///       {
        ///       xtype: 'panel',   // defaults do not have precedence over
        ///       id: 'panel1',     // options in config objects, so the defaults
        ///       autoScroll: false // will not be applied here, panel1 will be autoScroll:false
        ///       },
        ///       new Ext.Panel({       // defaults do have precedence over options
        ///       id: 'panel2',     // options in components, so the defaults
        ///       autoScroll: false // will be applied here, panel2 will be autoScroll:true.
        ///       })
        ///       ]
        ///     </code>
        ///   </example>
        /// </summary>
        [Description( "This option is a means of applying default settings to all added items whether added through the items config or via the add or insert methods." )]
        [DefaultValue( null )]
        string Defaults { get; set; }

        /// <summary>
        ///   If true the container will force a layout initially even if hidden or collapsed.
        ///   This option is useful for forcing forms to render in collapsed or hidden containers. (defaults to false).
        /// </summary>
        [Description( "If true the container will force a layout initially even if hidden or collapsed." )]
        [DefaultValue( false )]
        bool ForceLayout { get; set; }

        /// <summary>
        ///   True to hide the borders of each contained component, false to defer to the component's existing border settings (defaults to false).
        /// </summary>
        [Description( "True to hide the borders of each contained component, false to defer to the component's existing border settings (defaults to false)." )]
        [DefaultValue( false )]
        bool HideBorders { get; set; }

        /// <summary>
        ///   True to automatically monitor window resize events to handle anything that is sensitive to the current size of the viewport.
        ///   <i>This value is typically managed by the chosen layout and should not need to be set manually.</i>
        /// </summary>
        [Description( "True to automatically monitor window resize events to handle anything that is sensitive to the current size of the viewport." )]
        [DefaultValue( null )]
        bool? MonitorResize { get; set; }

        /// <summary>
        ///   The event to listen to for resizing in layouts. Defaults to 'resize'.
        /// </summary>
        [Description( "The event to listen to for resizing in layouts. Defaults to 'resize'." )]
        [DefaultValue( ( string ) null )]
        string ResizeEvent { get; set; }

        #region Field

        [Description( "The default CSS class for the field (defaults to 'x-form-field')." )]
        [DefaultValue( ( string ) null )]
        string FieldClass { get; set; }

        [Description( "The CSS class to use when the field receives focus (defaults to 'x-form-focus')." )]
        [DefaultValue( ( string ) null )]
        string FocusClass { get; set; }

        [Description( "The field's HTML name attribute (defaults to ''). Note: this property must be set if this field is to be automatically included with form submit()." )]
        [DefaultValue( ( string ) null )]
        string Name { get; set; }

        #endregion

        #region Window

        /// <summary>
        ///   True to display a shadow around this panel.
        /// </summary>
        [Description( "true to display a shadow around this panel" )]
        [DefaultValue( false )]
        bool Shadow { get; set; }

        #endregion

        #region TabPanel

        /// <summary>
        ///   true by default to defer the rendering of child items to the browsers DOM until a tab is activated. false will render all contained items as soon as the layout is rendered.
        ///   If there is a significant amount of content or a lot of heavy controls being rendered into panels that are not displayed by default, setting this to true might improve performance.
        ///   The deferredRender property is internally passed to the layout manager for TabPanels (Ext.layout.CardLayout) as its Ext.layout.CardLayout.deferredRender configuration value.
        /// </summary>
        /// <remarks>
        ///   <b>Note:</b> leaving deferredRender as true means that the content within an unactivated tab will not be available. For example, this means that if the TabPanel is within a form, 
        ///   then until a tab is activated, any Fields within unactivated tabs will not be rendered, and will therefore not be submitted and will not be available to either getValues or setValues.
        /// </remarks>
        [Description( "true by default to defer the rendering of child items to the browsers DOM until a tab is activated. false will render all contained items as soon as the layout is rendered." )]
        [DefaultValue( false )]
        bool DeferredRender { get; set; }

        #endregion
    }
}