﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System.ComponentModel;
using NetJsWire.JsonRpcHandler;
using NetJsWire.Widgets.Ui.Layout;

namespace NetJsWire.Widgets.Config
{
    public class TabPanelConfigOptions : PanelConfigOptions
    {
        private const LayoutType DefaultLayout = LayoutType.Card;
        private bool _deferredRender = true;

        public TabPanelConfigOptions( IWidget ownerWidget )
            : base( ownerWidget )
        {
        }

        /// <summary>
        ///   true by default to defer the rendering of child items to the browsers DOM until a tab is activated. false will render all contained items as soon as the layout is rendered.
        ///   If there is a significant amount of content or a lot of heavy controls being rendered into panels that are not displayed by default, setting this to true might improve performance.
        ///   The deferredRender property is internally passed to the layout manager for TabPanels (Ext.layout.CardLayout) as its Ext.layout.CardLayout.deferredRender configuration value.
        /// </summary>
        /// <remarks>
        ///   <b>Note:</b> leaving deferredRender as true means that the content within an unactivated tab will not be available. For example, this means that if the TabPanel is within a form, 
        ///   then until a tab is activated, any Fields within unactivated tabs will not be rendered, and will therefore not be submitted and will not be available to either getValues or setValues.
        /// </remarks>
        [Description( "true by default to defer the rendering of child items to the browsers DOM until a tab is activated. false will render all contained items as soon as the layout is rendered." )]
        [DefaultValue( true )]
        [Browsable( true )]
        public override bool DeferredRender
        {
            get
            {
                return _deferredRender;
            }
            set
            {
                SetValue( "deferredRender", value, true, _deferredRender );
                _deferredRender = value;
            }
        }

        /// <summary>
        ///   The layout type to be used for this container.
        ///   <remarks>
        ///     <b>*Important:</b> In order for child items to be correctly sized and positioned, typically a layout manager must be specified through the layout configuration option.
        ///   </remarks>
        ///   The sizing and positioning of child items is the responsibility of the Container's layout manager which creates and manages the type of layout you have in mind.
        ///   For example:
        ///   <example>
        ///     <code>
        ///       new Ext.Window({
        ///       width:300, height: 300,
        ///       layout: 'fit', // explicitly set layout manager: override the default (layout:'auto')
        ///       items: [{
        ///       title: 'Panel inside a Window'
        ///       }]
        ///       }).show();
        ///     </code>
        ///   </example>
        ///   If the layout configuration is not explicitly specified for a general purpose container (e.g. Container or Panel) the default layout manager will be used 
        ///   which does nothing but render child components sequentially into the Container (no sizing or positioning will be performed in this situation).
        ///   Some container classes implicitly specify a default layout (e.g. FormPanel specifies layout:'form'). Other specific purpose classes internally 
        ///   specify/manage their internal layout (e.g. GridPanel, TabPanel, TreePanel, Toolbar, Menu, etc.).
        /// </summary>
        [Description( "The layout type to be used for this TabPanel." )]
        [DefaultValue( LayoutType.Card )]
        [Browsable( true )]
        public override LayoutType Layout
        {
            get
            {
                return DefaultLayout;
            }
            set
            {
                // Can't set, this component can only have LayoutType.Card
            }
        }
    }
}
