#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;
using Newtonsoft.Json;

namespace NetJsWire.Widgets.Data
{
    ///<summary>
    ///  Represents a client-side data container.
    ///</summary>
    [Renderer( typeof( Renderers.Data.Store ) )]
    [JsonObject( MemberSerialization = MemberSerialization.OptIn )]
    public partial class Store : BindingSource, IWidget, IDataStore
    {
        private readonly IRenderer _renderer;
        private List<string> _columns = new List<string>();
        private IConfigOptions _configOptions;

        ///<summary>
        ///  Initializes a new instance of Store.
        ///</summary>
        public Store()
        {
            _renderer = Helper.GetRenderer( this );
            DataSourceChanged += StoreDataSourceChanged;
        }

        ///<summary>
        ///  Initializes a new instance of Store and adds it to the Container.
        ///</summary>
        ///<param name = "container"></param>
        public Store( IContainer container )
        {
            if ( container != null )
                container.Add( this );
            _renderer = Helper.GetRenderer( this );
            DataSourceChanged += StoreDataSourceChanged;
        }


        ///<summary>
        ///  The list of Columns that are going to be rendered (trasmited to the client).
        ///</summary>
        [JsonIgnore]
        public List<string> Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        [Browsable( false )]
        [JsonIgnore]
        public ContainerControl ContainerControl { get; set; }

        [JsonIgnore]
        public override ISite Site
        {
            set
            {
                base.Site = value;
                if ( value == null )
                    return;

                var host = ( IDesignerHost ) value.GetService( typeof( IDesignerHost ) );

                if ( host == null )
                    return;

                var baseComp = host.RootComponent;

                if ( baseComp is ContainerControl )
                    ContainerControl = ( ContainerControl ) baseComp;
            }
        }

        #region IWidget Members

        [Browsable( false )]
        public virtual IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            throw new NotSupportedException( "This component does not support this method." );
        }

        public void RemoveClass( string cls )
        {
            throw new NotSupportedException( "This component does not support this method." );
        }

        [DefaultValue( false )]
        [Description( "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer." )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All )]
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        [Category( "Config" )]
        [JsonProperty( "config" )]
        public virtual IConfigOptions ConfigOptions
        {
            get { return _configOptions ?? ( _configOptions = new StoreConfigOptions( this ) ); }
        }

        #endregion

        private void StoreDataSourceChanged( object sender, EventArgs e )
        {
            if ( !Renderer.Rendered ) return;
            if ( DataSource != null ) //TODO: Cange this so we accept a null DataSource to empty the store...
                Renderer.Render();
        }
    }
}