#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;

namespace NetJsWire.Widgets.Ui.Form
{
    [Renderer( typeof( Renderers.Form.CheckBox ) )]
    public partial class CheckBox : System.Windows.Forms.CheckBox, IWidget, IFormField
    {
        private readonly IRenderer _renderer;
        private bool _checked;
        private IConfigOptions _configOptions;
        private bool _sendUpdates = true;

        public CheckBox()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
            Check += CheckBox_CheckedChanged;
        }

        public CheckBox( IContainer container )
        {
            container.Add( this );
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
            Check += CheckBox_CheckedChanged;
        }

        [DefaultValue( true )]
        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                if ( Renderer.Rendered )
                {
                    Renderer.CallCommand( "setDisabled", !value );
                }
                else
                {
                    if ( value == false )
                        Renderer.SetConstructionProperty( "disabled", !value );
                }
            }
        }

        public override string Text
        {
            get { return base.Text; }
            set
            {
                base.Text = value;
                if ( !Renderer.Rendered )
                {
                    Renderer.SetConstructionProperty( "boxLabel", value );
                }
                else
                {
                    Renderer.AddCommand( "setLabel", value );
                }
            }
        }

        public new bool Checked
        {
            get { return _checked; }
            set
            {
                if ( Renderer.Rendered && !InternalChange )
                {
                    Renderer.AddCommand( "setValue", value );
                }
                _checked = value;
                if ( DesignMode )
                    base.Checked = value;
            }
        }

        public bool InternalChange { get; set; }

        [DefaultValue( true )]
        public bool SendUpdates
        {
            get { return _sendUpdates; }
            set { _sendUpdates = value; }
        }

        #region IFormField Members

        public string FieldLabel { get; set; }

        #endregion

        #region IWidget Members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description(
           "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
           )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new FieldConfigOptions( this ) { Xtype = "ncheckbox" };
                }
                return _configOptions;
            }
        }

        #endregion

        public event EventHandler Check;

        private void CheckBox_CheckedChanged( object sender, EventArgs e )
        {
            _checked = bool.Parse( sender.ToString() );
            base.OnCheckedChanged( EventArgs.Empty );
        }

        #region Hidden properties from Control base class

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleDescription { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleName { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleRole { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public override bool AllowDrop { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override ContextMenu ContextMenu { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( AnchorStyles.None )]
        public override AnchorStyles Anchor { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Color BackColor
        {
            get { return base.BackColor; }
            set { base.BackColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override Image BackgroundImage { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( ImageLayout.None )]
        public override ImageLayout BackgroundImageLayout { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override ContextMenuStrip ContextMenuStrip { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool CausesValidation { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override Cursor Cursor { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( DockStyle.None )]
        public override DockStyle Dock { get { return base.Dock; } set { base.Dock = value; } }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Font Font
        {
            get { return base.Font; }
            set { base.Font = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Color ForeColor
        {
            get { return base.ForeColor; }
            set { base.ForeColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( ImeMode.NoControl )]
        public new ImeMode ImeMode { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public new Padding Margin
        {
            get { return base.Margin; }
            set { base.Margin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Size MaximumSize
        {
            get { return base.MaximumSize; }
            set { base.MaximumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Size MinimumSize
        {
            get { return base.MinimumSize; }
            set { base.MinimumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public new Padding Padding
        {
            get { return base.Padding; }
            set { base.Padding = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( RightToLeft.No )]
        public override RightToLeft RightToLeft
        {
            get { return base.RightToLeft; }
            set { base.RightToLeft = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool TabStop { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool UseWaitCursor { get; set; }

        protected bool ShouldSerializeBackColor()
        {
            return false;
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( FlatStyle.Standard )]
        public new FlatStyle FlatStyle { get { return base.FlatStyle; } set { base.FlatStyle = value; } }

        #endregion Hidden properties from Control base class

    }
}