#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Ui.ToolBar;

namespace NetJsWire.Widgets.Ui.Form
{
    [Renderer( typeof( Renderers.Form.ComboBox ) )]
    public class ComboBox : System.Windows.Forms.ComboBox, IComboBox, IWidget, IFormField
    {
        private readonly IRenderer _renderer;
        //internal List<object> _items = new List<object>();
        private bool _allowBlank = true;
        private string _blankText = string.Empty;
        //public new event EventHandler SelectedIndexChanged;
        private string _colTemplate = string.Empty;
        private string _columnHeaders = string.Empty;
        private IConfigOptions _configOptions;

        private bool _editable = true;
        private string _emptyText = string.Empty;
        private bool _enabled = true;
        internal bool _headersChanged;
        private bool _internalChange;
        private string _itemSelector;
        private bool _selectOnFOcus = true;

        private int _selectedIndex = -1;
        private bool _sendUpdates = true;
        private string _title = string.Empty;
        private TriggerActionType _triggerAction = TriggerActionType.query;


        public ComboBox()
        {
            _renderer = Helper.GetRenderer( this );
            changeSelected += ComboBox_changeSelected;
            BindingContextChanged += ComboBox_BindingContextChanged;
            DataSourceChanged += ComboBoxEx_DataSourceChanged;
            //SelectedIndexChanged += ComboBox_SelectedIndexChanged;
            Columns = new ComboBoxColumnCollection( this );
        }

        [Editor( typeof( ComboBoxColumnEditor ), typeof( UITypeEditor ) )]
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        public ComboBoxColumnCollection Columns { get; set; }

        protected bool ShouldSerializeColumns()
        {
            return Columns.Count > 0;
        }

        protected void ResetColumns()
        {
            Columns.Clear();
        }

        [Browsable( false ), DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden ), Description( "The selected item index." )]
        public override int SelectedIndex
        {
            get
            {
                return base.SelectedIndex;
            }

            set
            {
                var count = 0;

                if ( !_internalChange && Renderer.Rendered )
                {
                    Renderer.CallCommand( "setValue", ( value >= 0 ) ? value : ( int? ) null );
                }
                else if ( !_internalChange && !Renderer.Rendered )
                {
                    if ( value == -1 )
                        Renderer.SetConstructionProperty( "value", null );
                    else
                    {
                        Renderer.SetConstructionProperty( "value", value );
                    }
                }

                if ( Items != null )
                {
                    count = Items.Count;
                }
                try
                {
                    if ( value < count )
                        base.SelectedIndex = value;
                }
                catch
                {
                }
                finally
                {
                    FromFormPost = false;
                }
            }
        }

        [DefaultValue( true )]
        public new bool Enabled
        {
            get { return _enabled; }
            set
            {
                _enabled = value;
                if ( Renderer.Rendered )
                    Renderer.CallCommand( "setDisabled", !value );
                else
                {
                    if ( !value )
                    {
                        Renderer.SetConstructionProperty( "disabled", !value );
                    }
                    else
                    {
                        Renderer.RemoveConstructionProperty( "disabled" );
                    }
                }
            }
        }

        [DefaultValue( false )]
        public bool ReadOnly { get; set; }

        [DefaultValue( false ), Browsable( false )]
        public bool FromFormPost
        {
            get;
            set;
        }

        [DefaultValue( ( string ) null )]
        public string ItemSelector
        {
            get { return _itemSelector; }
            set
            {
                _itemSelector = value;
                if ( Renderer.Rendered )
                {
                    Renderer.CallCommand( "setItemSelector", _itemSelector );
                }
            }
        }

        [DefaultValue( "" ),
         Editor(
             "System.Windows.Forms.Design.DataMemberFieldEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
             , typeof( UITypeEditor ) ), Category( "Data" ),
         TypeConverter(
             "System.Windows.Forms.Design.DataMemberFieldConverter, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
             ), Description( "Sets the display member for the combobox." )]
        public new string DisplayMember
        {
            get { return base.DisplayMember; }
            set
            {
                base.DisplayMember = value;
                if ( Renderer.Rendered )
                {
                    if ( Columns.Count > 0 )
                    {
                        SetTemplate( "<tpl for=\".\"><div class=\"x-combo-list-item\">{" + value + "}</div></tpl>" );
                        Renderer.SetEqualProperty( "displayField", value );
                    }
                    else
                    {
                        SetTemplate( "<tpl for=\".\"><div class=\"x-combo-list-item\">{display}</div></tpl>" );
                        Renderer.SetEqualProperty( "displayField", "display" );
                    }
                }
            }
        }


        [Category( "Data" ), DefaultValue( ( string ) null ), RefreshProperties( RefreshProperties.Repaint ),
         AttributeProvider( typeof( IListSource ) )]
        public new object DataSource
        {
            get { return base.DataSource; }
            set
            {
                var oldDataSource = base.DataSource;
                base.DataSource = value;
                if ( Renderer.Rendered )
                {
                    if ( oldDataSource != value )
                        ( ( Renderers.Form.ComboBox ) Renderer ).ChangeStore();
                    ( ( Renderers.Form.ComboBox ) Renderer ).RenderData();
                }
            }
        }

        [DefaultValue( ( string ) null )]
        public string ColumnHeaders
        {
            get { return _columnHeaders; }
            set
            {
                if ( Renderer.Rendered && ( _columnHeaders != value ) )
                    _headersChanged = true;
                else
                    _headersChanged = false;

                _columnHeaders = value;
            }
        }

        [Editor( "System.ComponentModel.Design.MultilineStringEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof( UITypeEditor ) )]
        [DefaultValue( ( string ) null )]
        public string ColTemplate
        {
            get { return _colTemplate; }
            set
            {
                _colTemplate = value;
                if ( Renderer.Rendered )
                {
                    if ( !string.IsNullOrEmpty( value ) )
                    {
                        SetTemplate( value );
                        //Renderer.CallCommand( "setTemplate", value );
                    }
                    else
                    {
                        SetTemplate( "<tpl for\".\"><div class=\"x-combo-list-item\">{" +
                                             ( !string.IsNullOrEmpty( DisplayMember ) ? DisplayMember : "display" ) +
                                             "}</div></tpl>" );
                    }
                }
            }
        }

        [DefaultValue( ( string ) null )]
        public string EmptyText
        {
            get { return _emptyText; }
            set
            {
                _emptyText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "emptyText", value );
                }
                else
                {
                    Renderer.SetConstructionProperty( "emptyText", value );
                }
            }
        }

        [DefaultValue( ( string ) null )]
        public string BlankText
        {
            get { return _blankText; }
            set
            {
                _blankText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "blankText", value );
                }
                else
                {
                    Renderer.SetConstructionProperty( "blankText", value );
                }
            }
        }

        [DefaultValue( true )]
        public bool AllowBlank
        {
            get { return _allowBlank; }
            set
            {
                _allowBlank = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "allowBlank", value );
                    Renderer.CallCommand( "validate", null );
                }
                else
                {
                    Renderer.SetConstructionProperty( "allowBlank", value );
                }
            }
        }


        public new int DropDownWidth
        {
            get { return base.DropDownWidth; }
            set
            {
                base.DropDownWidth = value;
                if ( Renderer.Rendered )
                {
                    //Renderer.AddCommand("list.setWidth", value);
                    //Renderer.AddCommand("innerList.setWidth", value-2);
                    Renderer.SetEqualProperty( "listWidth", value );
                }
            }
        }

        [DefaultValue( ( string ) null )]
        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                if ( Renderer.Rendered )
                {
                    //Renderer.CallCommand("setTitle", value);
                }
            }
        }

        [DefaultValue( true )]
        public bool Editable
        {
            get { return _editable; }
            set { _editable = value; }
        }

        [DefaultValue( false )]
        public bool ForceSelection { get; set; }

        [DefaultValue( true )]
        public bool SelectOnFocus
        {
            get { return _selectOnFOcus; }
            set { _selectOnFOcus = value; }
        }

        [DefaultValue( 1 )]
        public TriggerActionType TriggerAction
        {
            get { return _triggerAction; }
            set { _triggerAction = value; }
        }

        [DefaultValue( false )]
        public bool TypeAhead { get; set; }

        /// <summary>
        ///   If false the element wont be shown.
        /// </summary>
        public new bool Visible
        {
            get { return base.Visible; }
            set
            {
                if ( Renderer.Rendered )
                {
                    Renderer.AddCommand( "setVisible", value );
                }
                else
                {
                    Renderer.SetConstructionProperty( "hidden", !value );
                }

                base.Visible = value;
            }
        }

        internal bool InternalChange
        {
            get { return _internalChange; }
            set { _internalChange = value; }
        }

        #region IComboBox Members

        [DefaultValue( true )]
        public bool SendUpdates
        {
            get { return _sendUpdates; }
            set { _sendUpdates = value; }
        }

        #endregion

        #region IFormField Members

        /// <summary>
        ///   The label to apply to the field, only valid if contained in a FormPanel or a FieldSet
        /// </summary>
        [DefaultValue( ( string ) null )]
        public string FieldLabel { get; set; }

        #endregion

        #region IWidget Members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description( "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer." )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new FieldConfigOptions( this );
                }
                return _configOptions;
            }
        }

        #endregion

        public event EventHandler changeSelected;

        ~ComboBox()
        {
            Dispose( true );
        }

        public void SetTemplate( string template )
        {
            Renderer.AddPrecodeCommand( "setTemplate", template );
        }

        protected override void Dispose( bool disposing )
        {
            WebApplication.Instance.RemoveObject( ( ( Renderers.Form.ComboBox ) Renderer ).InternalID );
            base.Dispose( disposing );
        }

        private void ComboBox_changeSelected( object sender, EventArgs e )
        {
            try
            {
                _internalChange = true;
                var index = int.Parse( sender.ToString() );
                if ( index >= 0 && Items.Count > 0 )
                    SelectedItem = index < Items.Count ? Items[ index ] : Items[ Items.Count - 1 ];

                SelectedIndex = index;
                _selectedIndex = index;
            }
            finally
            {
                _internalChange = false;
            }
        }

        private void UpdateText()
        {
            string _text = null;
            if ( SelectedIndex != -1 )
            {
                var item = Items[ SelectedIndex ];
                if ( item != null )
                {
                    _text = base.GetItemText( item );
                }
            }
            Text = _text;
        }

        private void ComboBoxEx_DataSourceChanged( object sender, EventArgs e )
        {
            if ( Renderer.Rendered )
                ( ( Renderers.Form.ComboBox ) Renderer ).RenderData();
        }

        private void ComboBox_BindingContextChanged( object sender, EventArgs e )
        {
            if ( DataSource != null )
            {
                if ( BindingContext != null )
                    ( ( CurrencyManager ) BindingContext[ DataSource ] ).ListChanged += cm_ListChanged;
            }
            else
            {
                //TODO: Remove the listener
            }
        }

        private void cm_ListChanged( object sender, ListChangedEventArgs e )
        {
            if ( Renderer.Rendered )
            {
                if ( e.ListChangedType == ListChangedType.ItemChanged )
                {
                    var val = string.Empty;

                    if ( Items[ e.NewIndex ] is DataRowView )
                    {
                        val = ( ( DataRowView ) Items[ e.NewIndex ] )[ DisplayMember ].ToString();
                    }
                    else if ( Items[ e.NewIndex ] is string )
                    {
                        val = Items[ e.NewIndex ].ToString();
                    }

                    //Renderer.AddCommand("getList().getChildren()[" + e.NewIndex + "].setLabel", val);
                }
                else if ( e.ListChangedType == ListChangedType.ItemAdded )
                {
                    //((qx.renderers.ui.form.ComboBoxEx)Renderer).RenderData(true);
                    //Renderer.AddCommand("add", "new qx.ui.form.ListItem(" + e.PropertyDescriptor.GetValue(this).ToString() + ")");
                }
                else if ( e.ListChangedType == ListChangedType.Reset )
                {
                    //((Renderers.Form.ComboBox)Renderer).CallCommand("renderData", true);
                    if ( Renderer.Rendered )
                        ( ( Renderers.Form.ComboBox ) Renderer ).RenderData();
                }
            }
        }

        /// <summary>
        ///   Updates the rendered data, you have to call this method if you are adding not DataSourced items.
        /// </summary>
        public void UpdateData()
        {
            ( ( Renderers.Form.ComboBox ) Renderer ).RenderData();
        }

        private void ComboBox_SelectedIndexChanged( object sender, EventArgs e )
        {
            if ( !_internalChange && Renderer.Rendered )
                Renderer.CallCommand( "setValue", ( SelectedIndex >= 0 ) ? SelectedIndex : ( int? ) null );
            else if ( !_internalChange && !Renderer.Rendered )
            {
                Renderer.SetConstructionProperty( "value", ( SelectedIndex >= 0 ) ? SelectedIndex : ( int? ) null );
            }
        }

        public new void Focus()
        {
            Renderer.AddCommand( "focus" );
        }

        #region Nested type: SetIndexCallback

        private delegate void SetIndexCallback( int index );

        #endregion

        #region Hidden properties from Control base class

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleDescription { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleName { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new string AccessibleRole { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public override bool AllowDrop { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override ContextMenu ContextMenu { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( AnchorStyles.None )]
        public override AnchorStyles Anchor { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Color BackColor
        {
            get { return base.BackColor; }
            set { base.BackColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override Image BackgroundImage { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( ImageLayout.None )]
        public override ImageLayout BackgroundImageLayout { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override ContextMenuStrip ContextMenuStrip { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool CausesValidation { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public override Cursor Cursor { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( DockStyle.None )]
        public override DockStyle Dock { get { return base.Dock; } set { base.Dock = value; } }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Font Font
        {
            get { return base.Font; }
            set { base.Font = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Color ForeColor
        {
            get { return base.ForeColor; }
            set { base.ForeColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( ImeMode.NoControl )]
        public new ImeMode ImeMode { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public new Padding Margin
        {
            get { return base.Margin; }
            set { base.Margin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Size MaximumSize
        {
            get { return base.MaximumSize; }
            set { base.MaximumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public override Size MinimumSize
        {
            get { return base.MinimumSize; }
            set { base.MinimumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public new Padding Padding
        {
            get { return base.Padding; }
            set { base.Padding = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( RightToLeft.No )]
        public override RightToLeft RightToLeft
        {
            get { return base.RightToLeft; }
            set { base.RightToLeft = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool TabStop { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool UseWaitCursor { get; set; }

        protected bool ShouldSerializeBackColor()
        {
            return false;
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( FlatStyle.Standard )]
        public new FlatStyle FlatStyle { get { return base.FlatStyle; } set { base.FlatStyle = value; } }

        #endregion Hidden properties from Control base class

        #region Hidden properties from Combo base class

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( null )]
        public new AutoCompleteStringCollection AutoCompleteCustomSource { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( AutoCompleteSource.None )]
        public new AutoCompleteSource AutoCompleteSource { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( AutoCompleteMode.None )]
        public new AutoCompleteMode AutoCompleteMode { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( DrawMode.Normal )]
        public new DrawMode DrawMode { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( ( string ) null )]
        public new string FormatString { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool FormattingEnabled { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool IntegralHeight { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( 0 )]
        public new int ItemHeight { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( 0 )]
        public new int MaxDropDownItems { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( 0 )]
        public new int DropDownHeight { get; set; }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never ), DefaultValue( false )]
        public new bool Sorted { get; set; }

        #endregion Hidden properties from Combo base class
    }
}