#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Renderers.Core;
using Newtonsoft.Json.Linq;

namespace NetJsWire.Widgets.Ui.Form
{
    /// <summary>
    ///   Basic TextField class, it sets the input type as 'text' by default
    ///   if you want to use password type then set the PasswordChar property to
    ///   some char (any char would do it) i know its ugly gonna change it :)
    /// </summary>
    [Renderer( typeof( Renderers.Form.TextField ) )]
    public class TextField : TextBox, IWidget, IFormField
    {
        private readonly IRenderer _renderer;
        private bool _allowBlank = true;
        private string _blankText;
        private IConfigOptions _configOptions;
        private string _emptyText;
        private bool _internalChange;
        private bool _isValid = true;
        private string _maxLengthText;
        private int _minLength;
        private string _minLengthText;
        private bool _sendServerUpdates = true;
        private bool _sendTextUpdates = true;
        private string _text;

        public TextField()
        {
            _renderer = Helper.GetRenderer( this );
            ( ( Widget ) Renderer ).Events.Add( "Input" );
            Input += TextFieldInput;
            SendTextUpdates = true;
            Renderer.AfterRender += RendererAfterRender;
        }

        [DefaultValue( true )]
        public bool IsValid
        {
            get { return _isValid; }
            set { _isValid = value; }
        }

        [DefaultValue( ( string ) null )]
        public override string Text
        {
            get { return _text; }
            set
            {
                _text = value;
                if ( DesignMode )
                {
                    base.Text = value;
                }

                if ( Renderer.Rendered && !_internalChange && SendServerUpdates )
                {
                    Renderer.CallCommand( "setValue", value );
                }
            }
        }

        /// <summary>
        ///   If the component is enabled, defaults to true.
        /// </summary>
        [DefaultValue( true )]
        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                SetEnabled( value );
                if ( Renderer.Rendered )
                {
                    Renderer.CallCommand( "setDisabled", !value );
                }
                else
                {
                    if ( value == false )
                        Renderer.SetConstructionProperty( "disabled", true );
                }
            }
        }

        /// <summary>
        ///   Error text to display if the allow blank validation fails (defaults to "This field is required")
        /// </summary>
        [DefaultValue( ( string ) null )]
        public string BlankText
        {
            get { return _blankText; }
            set
            {
                _blankText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "blankText", value );
                }
                else
                {
                    if ( !string.IsNullOrEmpty( value ) )
                        Renderer.SetConstructionProperty( "blankText", value );
                }
            }
        }

        [DefaultValue( 32767 )]
        public override int MaxLength
        {
            get { return base.MaxLength; }
            set
            {
                base.MaxLength = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "maxLength", value );
                }
                else
                {
                    if ( value != 32767 )
                        Renderer.SetConstructionProperty( "maxLength", value );
                }
            }
        }

        ///<summary>
        ///  Sets the text to display when <see cref = "MaxLength" /> is invalid.
        ///</summary>
        [DefaultValue( ( string ) null )]
        public string MaxLengthText
        {
            get { return _maxLengthText; }
            set
            {
                _maxLengthText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "maxLengthText", value );
                }
                else
                {
                    if ( !string.IsNullOrEmpty( value ) )
                        Renderer.SetConstructionProperty( "maxLengthText", value );
                }
            }
        }

        ///<summary>
        ///  Sets the minimun length that can be entered into the field.
        ///</summary>
        [DefaultValue( 0 )]
        public int MinLength
        {
            get { return _minLength; }
            set
            {
                _minLength = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "minLength", value );
                }
                else
                {
                    if ( value != 0 )
                        Renderer.SetConstructionProperty( "minLength", value );
                }
            }
        }

        ///<summary>
        ///  Sets the text to display when <see cref = "MinLength" /> is invalid.
        ///</summary>
        [DefaultValue( ( string ) null )]
        public string MinLengthText
        {
            get { return _minLengthText; }
            set
            {
                _minLengthText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "minLengthText", value );
                }
                else
                {
                    if ( !string.IsNullOrEmpty( value ) )
                        Renderer.SetConstructionProperty( "minLengthText", value );
                }
            }
        }

        /// <summary>
        ///   False to validate that the value length > 0 (defaults to true)
        /// </summary>
        [DefaultValue( true )]
        public bool AllowBlank
        {
            get { return _allowBlank; }
            set
            {
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "allowBlank", value );
                    Renderer.AddCommand( "validate", null );
                }
                else
                {
                    Renderer.SetConstructionProperty( "allowBlank", value );
                }
                _allowBlank = value;
            }
        }

        /// <summary>
        ///   The default text to display in an empty field (defaults to null).
        /// </summary>
        [DefaultValue( ( string ) null )]
        public string EmptyText
        {
            get { return _emptyText; }
            set
            {
                _emptyText = value;
                if ( Renderer.Rendered )
                {
                    Renderer.SetEqualProperty( "emptyText", value );
                }
                else
                {
                    if ( !string.IsNullOrEmpty( value ) )
                        Renderer.SetConstructionProperty( "emptyText", value );
                }
            }
        }

        /// <summary>
        ///   If false the element wont be shown.
        /// </summary>
        public new bool Visible
        {
            get { return base.Visible; }
            set
            {
                if ( Renderer.Rendered )
                {
                    Renderer.AddCommand( "setVisible", value );
                }
                else
                {
                    Renderer.SetConstructionProperty( "hidden", !value );
                }

                base.Visible = value;
            }
        }

        /// <summary>
        ///   If the field should send updates to the server, defaults to true.
        /// </summary>
        [DefaultValue( true )]
        public bool SendTextUpdates
        {
            get { return _sendTextUpdates; }
            set
            {
                _sendTextUpdates = value;
                if ( !Renderer.Rendered )
                    Renderer.SetConstructionProperty( "sendUpdates", value );
            }
        }

        /// <summary>
        ///   Mask Individual Character Usage:
        ///   <list type = "bullet">
        ///     <item><description>
        ///             9 - designates only numeric values
        ///           </description></item>
        ///     <item><description>
        ///             L - designates only uppercase letter values
        ///           </description></item>
        ///     <item><description>
        ///             l - designates only lowercase letter values
        ///           </description></item>
        ///     <item><description>
        ///             A - designates only alphanumeric values
        ///           </description></item>
        ///     <item><description>
        ///             X - denotes that a custom client script regular expression is specified
        ///           </description></item>
        ///   </list>
        ///   All other characters are assumed to be "special" characters used to mask the input component.
        ///   <example>
        ///     Example 1:
        ///     (999)999-9999 only numeric values can be entered where the the character
        ///     position value is 9. Parenthesis and dash are non-editable/mask characters.
        ///     Example 2:
        ///     99L-ll-X[^A-C]X only numeric values for the first two characters,
        ///     uppercase values for the third character, lowercase letters for the
        ///     fifth/sixth characters, and the last character X[^A-C]X together counts
        ///     as the eighth character regular expression that would allow all characters
        ///     but "A", "B", and "C". Dashes outside the regular expression are non-editable/mask characters.
        ///   </example>
        /// </summary>
        [DefaultValue( ( string ) null )]
        public string Mask { get; set; }

        internal bool SendServerUpdates
        {
            get { return _sendServerUpdates; }
            set { _sendServerUpdates = value; }
        }

        internal bool InternalChange
        {
            get { return _internalChange; }
            set { _internalChange = value; }
        }

        #region IFormField Members

        /// <summary>
        ///   The label to apply to the field, only valid if contained in a FormPanel or a FieldSet
        /// </summary>
        [DefaultValue( ( string ) null )]
        public string FieldLabel
        {
            get { return ConfigOptions.FieldLabel; }
            set
            {
                ConfigOptions.FieldLabel = value;
                if ( Parent == null )
                    return;
                Parent.Invalidate( ClientRectangle );
                Parent.Update();
            }
        }

        #endregion

        #region IWidget Members

        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description(
           "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer."
           )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public virtual IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new FieldConfigOptions( this );
                    if ( this is NumberField )
                        _configOptions.Xtype = "nnumberfield";
                    else
                        _configOptions.Xtype = "ntextfield";
                }
                return _configOptions;
            }
        }

        #endregion

        public event EventHandler Input;
        public new event EventHandler TextChanged;

        private void RendererAfterRender( object sender, EventArgs e )
        {
            if ( SendTextUpdates )
                Input += TextFieldInput;
        }

        ~TextField()
        {
            Dispose( true );
        }

        protected override void Dispose( bool disposing )
        {
            try
            {
                WebApplication.Instance.RemoveObject( ( ( Renderers.Form.TextField ) Renderer ).InternalID );
                base.Dispose( disposing );
            }
            catch
            {
                if ( !IsDisposed && !Disposing )
                    base.Dispose( disposing );
            }
        }

        protected void TextFieldInput( object sender, EventArgs e )
        {
            _internalChange = true;
            var vals = ( JArray ) sender;
            _text = vals[ 0 ].ToString();
            //SetText(vals[0].ToString());
            object oTch = TextChanged;
            if ( oTch != null )
                TextChanged( this, null );
            _isValid = ( bool ) vals[ 1 ];
            _internalChange = false;
        }

        private void SetEnabled( bool enabled )
        {
            if ( InvokeRequired )
            {
                SetEnabledCallback d = SetEnabled;
                BeginInvoke( d, new object[] { enabled } );
            }
            else
            {
                base.Enabled = enabled;
            }
        }

        public new void Focus()
        {
            Renderer.AddCommand( "focus", null );
        }

        public void AddKeyNav( string keys, string function )
        {
            Renderer.AddKeyNav( keys, function );
        }

        #region Nested type: SetEnabledCallback

        private delegate void SetEnabledCallback( bool enabled );

        #endregion
    }
}