﻿#region Header
// Copyright 2007-2011, Jerónimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.Util;

namespace NetJsWire.Widgets.Ui.Layout.Config
{
    [ Category( "LayoutConfig" ) ]
    [ TypeConverter( typeof (ExpandableObjectConverter) ) ]
    [ Serializable ]
    public class LayoutConfig : ILayoutConfig
    {
        private BoxAlign _align = BoxAlign.Top;
        private string _extraCls;
        private bool _renderHidden;
        protected IRenderer _renderer;

        public LayoutConfig( IRenderer renderer )
        {
            _renderer = renderer;
        }

        [ DefaultValue( null ) ]
        public Padding DefaultMargins
        {
            get { return Padding.Empty; }
            set { ; }
        }

        [ DefaultValue( null ) ]
        public Padding Padding
        {
            get { return Padding.Empty; }
            set { ; }
        }

        [ DefaultValue( BoxAlign.Top ) ]
        public BoxAlign Align
        {
            get { return _align; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( BoxAlign.Top != value )
                    {
                        _renderer.SetLayoutConfigProperty( "align", value.ToString().ToLower() );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "align" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "align" );
                }

                _align = value;
            }
        }

        [ DefaultValue( -1 ) ]
        public BoxPack Pack { get; set; }

        #region ILayoutConfig Members

        /// <summary>
        ///   An optional extra CSS class that will be added to the container. This can be useful for adding customized styles to the container or any of its children using standard CSS rules. See Ext.Component.ctCls also.
        ///   Note: extraCls defaults to '' except for the following classes which assign a value by default:
        ///   Absolute Layout : <code>'x-abs-layout-item'</code>
        ///   Box Layout : <code>'x-box-item'</code>
        ///   Column Layout : <code>'x-column'</code>
        ///   To configure the above Classes with an extra CSS class append to the default. For example, for ColumnLayout:
        ///   <example>
        ///     extraCls: 'x-column custom-class'
        ///   </example>
        /// </summary>
        [ DefaultValue( ( string ) null ) ]
        [ Description(
            "An optional extra CSS class that will be added to the container. This can be useful for adding customized styles to the container or any of its children using standard CSS rules."
            ) ]
        public string ExtraCls
        {
            get { return _extraCls; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( !string.IsNullOrEmpty( value ) )
                    {
                        _renderer.SetLayoutConfigProperty( "extraCls", value );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "extraCls" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "extraCls" );
                }

                _extraCls = value;
            }
        }

        /// <summary>
        ///   True to hide each contained item on render (defaults to false).
        /// </summary>
        [ DefaultValue( false ) ]
        [ Description( "True to hide each contained item on render." ) ]
        public bool RenderHidden
        {
            get { return _renderHidden; }
            set
            {
                if ( !_renderer.Rendered )
                {
                    if ( value )
                    {
                        _renderer.SetLayoutConfigProperty( "renderHidden", value );
                    }
                    else
                    {
                        _renderer.RemoveLayoutConfigProperty( "renderHidden" );
                    }
                }
                else
                {
                    _renderer.RemoveLayoutConfigProperty( "renderHidden" );
                }

                _renderHidden = value;
            }
        }

        #endregion

        public static Padding SetNegativePaddingToZero( Padding padding )
        {
            if ( padding.All < 0 )
            {
                padding.Left = Math.Max( 0, padding.Left );
                padding.Top = Math.Max( 0, padding.Top );
                padding.Right = Math.Max( 0, padding.Right );
                padding.Bottom = Math.Max( 0, padding.Bottom );
            }
            return padding;
        }
    }
}