#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;

namespace NetJsWire.Widgets.Ui.Layout
{
    [Renderer( typeof( Renderers.Layout.Panel ) )]
    public class Panel : System.Windows.Forms.Panel, IWidget, IContentPanel, IWidgetContainer
    {
        private readonly IRenderer _renderer;
        private bool _border;
        private PanelConfigOptions _configOptions;
        private bool _float;
        private LayoutType _layout = LayoutType.Auto;
        private string _region = string.Empty;
        private bool _shadow;
        private string _title = string.Empty;
        private bool closable;
        private string collapsedTitle = string.Empty;
        private bool collapsible;
        private IContainer components;
        private bool fitToFrame;
        private string iframeUrl = string.Empty;
        private bool preventBodyReset;
        private Label titleLabel;
        private bool titlebar;
        private bool useIFrame;


        public Panel()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        public Panel( IContainer container )
        {
            container.Add( this );
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
        }

        /// <summary>
        ///   Setting the Ext.Panel configuration-option preventBodyReset: true will ensure your Panel body's structural html is isolated from the Ext theme.
        /// </summary>
        [DefaultValue( false ),
         Description(
             "Setting the Ext.Panel configuration-option preventBodyReset: true will ensure your Panel body's structural html is isolated from the Ext theme."
             )]
        public bool PreventBodyReset
        {
            get { return preventBodyReset; }
            set
            {
                preventBodyReset = value;

                if ( !Renderer.Rendered && value )
                {
                    Renderer.SetConstructionProperty( "preventBodyReset", true );
                }
            }
        }

        [DefaultValue( false )]
        public bool Split { get; set; }

        [DefaultValue( ( string ) null )]
        public string Defaults { get; set; }

        [DefaultValue( ( string ) null )]
        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                titleLabel.Text = value;
                if ( !string.IsNullOrEmpty( value ) )
                    titleLabel.Visible = true;
                if ( !Renderer.Rendered && !string.IsNullOrEmpty( value ) )
                {
                    Renderer.SetConstructionProperty( "titlebar", true );
                    Renderer.SetConstructionProperty( "title", value );
                }
            }
        }

        [DefaultValue( false )]
        public bool TitleBar
        {
            get { return titlebar; }
            set
            {
                titlebar = value;

                titleLabel.Visible = value;

                if ( !Renderer.Rendered && value )
                    Renderer.SetConstructionProperty( "titlebar", value );
            }
        }

        [DefaultValue( ( string ) null )]
        public string CollapsedTitle
        {
            get { return collapsedTitle; }
            set
            {
                collapsedTitle = value;
                if ( !Renderer.Rendered && !string.IsNullOrEmpty( value ) )
                    Renderer.SetConstructionProperty( "collapsedTitle", value );
            }
        }

        [DefaultValue( false )]
        public bool Collapsible
        {
            get { return ConfigOptions.Collapsible; }
            set { ConfigOptions.Collapsible = value; }
        }

        [DefaultValue( false )]
        public bool Closable
        {
            get { return ConfigOptions.Closable; }
            set { ConfigOptions.Closable = value; }
        }

        [DefaultValue( false )]
        public bool FitToFrame
        {
            get { return fitToFrame; }
            set
            {
                fitToFrame = value;
                if ( value && !Renderer.Rendered )
                    Renderer.SetConstructionProperty( "fitToFrame", value );
            }
        }

        [DefaultValue( false )]
        public bool UseIframe
        {
            get { return useIFrame; }
            set
            {
                useIFrame = value;
                if ( value && !Renderer.Rendered )
                    Renderer.SetConstructionProperty( "useIFrame", value );
            }
        }

        [DefaultValue( ( string ) null )]
        public string IframeUrl
        {
            get { return iframeUrl; }
            set
            {
                iframeUrl = value;
                if ( !Renderer.Rendered && !string.IsNullOrWhiteSpace( value ) )
                    Renderer.SetConstructionProperty( "iframeUrl", value );
                else
                {
                    if ( useIFrame )
                        Renderer.AddCommand( "setIframeUrl", iframeUrl );
                }
            }
        }

        public bool Floating
        {
            get { return _float; }
            set
            {
                _float = value;
                if ( !Renderer.Rendered && value )
                    Renderer.SetConstructionProperty( "floating", value );
            }
        }

        public bool Draggable { get; set; }

        /// <summary>
        ///   True to display the borders of the panel's body element, false to hide them (defaults to true).
        ///   By default, the border is a 2px wide inset border, but this can be further altered by setting <see cref = "NetJsWire.Widgets.Config.PanelConfigOptions.BodyBorder" /> to false.
        /// </summary>
        [Description( "True to display the borders of the panel's body element, false to hide them (defaults to true)." )]
        [DefaultValue( true )]
        public bool Border
        {
            get { return ConfigOptions.Border; }
            set { ConfigOptions.Border = value; }
        }

        public bool Shadow
        {
            get { return _shadow; }
            set
            {
                _shadow = value;
                if ( !Renderer.Rendered && value )
                    Renderer.SetConstructionProperty( "shadow", value );
            }
        }

        #region IContentPanel Members

        [DefaultValue( ( string ) null )]
        public new string Region
        {
            get { return _region; }
            set { _region = value; }
        }

        [DefaultValue( LayoutType.Auto )]
        [RefreshProperties( RefreshProperties.All )]
        public new LayoutType Layout
        {
            get { return _layout; }
            set
            {
                _layout = value;

                Invalidate();
            }
        }

        #endregion

        #region IWidget Members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description( "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer." )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new PanelConfigOptions( this );
                    _configOptions.Xtype = "npanel";
                }
                return _configOptions;
            }
        }

        #endregion

        #region Hidden members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new string AccessibleDescription
        {
            get { return base.AccessibleDescription; }
            set { base.AccessibleDescription = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new string AccessibleName
        {
            get { return base.AccessibleName; }
            set { base.AccessibleName = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AccessibleRole AccessibleRole
        {
            get { return base.AccessibleRole; }
            set { base.AccessibleRole = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool AllowDrop
        {
            get { return base.AllowDrop; }
            set { base.AllowDrop = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool AutoScroll
        {
            get { return base.AutoScroll; }
            set { base.AutoScroll = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size AutoScrollMargin
        {
            get { return base.AutoScrollMargin; }
            set { base.AutoScrollMargin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size AutoScrollMinSize
        {
            get { return base.AutoScrollMinSize; }
            set { base.AutoScrollMinSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AutoSizeMode AutoSizeMode
        {
            get { return base.AutoSizeMode; }
            set { base.AutoSizeMode = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Color BackColor
        {
            get { return base.BackColor; }
            set { base.BackColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Image BackgroundImage
        {
            get { return base.BackgroundImage; }
            set { base.BackgroundImage = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ImageLayout BackgroundImageLayout
        {
            get { return base.BackgroundImageLayout; }
            set { base.BackgroundImageLayout = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new BorderStyle BorderStyle
        {
            get { return base.BorderStyle; }
            set { base.BorderStyle = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool CausesValidation
        {
            get { return base.CausesValidation; }
            set { base.CausesValidation = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ContextMenuStrip ContextMenuStrip
        {
            get { return base.ContextMenuStrip; }
            set { base.ContextMenuStrip = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Cursor Cursor
        {
            get { return base.Cursor; }
            set { base.Cursor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Font Font
        {
            get { return base.Font; }
            set { base.Font = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Color ForeColor
        {
            get { return base.ForeColor; }
            set { base.ForeColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ImeMode ImeMode
        {
            get { return base.ImeMode; }
            set { base.ImeMode = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Padding Margin
        {
            get { return base.Margin; }
            set { base.Margin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size MinimumSize
        {
            get { return base.MinimumSize; }
            set { base.MinimumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size MaximumSize
        {
            get { return base.MaximumSize; }
            set { base.MaximumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Padding Padding
        {
            get { return base.Padding; }
            set { base.Padding = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new RightToLeft RightToLeft
        {
            get { return base.RightToLeft; }
            set { base.RightToLeft = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool TabStop
        {
            get { return base.TabStop; }
            set { base.TabStop = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool UseWaitCursor
        {
            get { return base.UseWaitCursor; }
            set { base.UseWaitCursor = value; }
        }

        #endregion

        #region IWidgetContainer Members

        /// <summary>
        ///   This is a config object containing properties specific to the chosen <see cref = "Layout" /> if <see cref = "Layout" /> has been specified as a string.
        /// </summary>
        [Description( "This is a config object containing properties specific to the chosen layout if layout has been specified as a string." )]
        [Category( "Layout" )]
        [DefaultValue( ( string ) null )]
        public string LayoutConfig
        {
            get { return ConfigOptions.LayoutConfig; }
            set { ConfigOptions.LayoutConfig = value; }
        }

        #endregion

        public event EventHandler Closed;
        public event EventHandler Custom;

        ~Panel()
        {
            Dispose( false );
        }

        protected override void Dispose( bool disposing )
        {
            WebApplication.Instance.RemoveObject( ( ( Renderers.Layout.Panel ) Renderer ).InternalID );
            base.Dispose( disposing );
        }

        public void SetUrl( string url )
        {
            Renderer.AddCommand( "setUrl", url );
        }

        public void SetIframeUrl( string iframeUrl )
        {
            if ( useIFrame )
            {
                this.iframeUrl = iframeUrl;
                Renderer.AddCommand( "setIframeUrl", iframeUrl );
            }
            else
            {
                throw new Exception( "You cannot set the Iframe URL if UseIframe is false, set it to true first." );
            }
        }

        public new void Refresh()
        {
            Renderer.AddCommand( "refresh", null );
        }

        private void InitializeComponent()
        {
            components = new Container();
            titleLabel = new Label();
            SuspendLayout();
            // 
            // titleLabel
            // 
            titleLabel.BackColor = Color.PaleTurquoise;
            titleLabel.Dock = DockStyle.Top;
            titleLabel.Location = new Point( 0, 0 );
            titleLabel.Name = "titleLabel";
            titleLabel.Size = new Size( 100, 27 );
            titleLabel.TabIndex = 0;
            titleLabel.Text = "label1";
            titleLabel.Visible = false;
            titleLabel.AutoSize = false;
            titleLabel.Height = 27;
            titleLabel.BackColor = Color.Aquamarine;
            titleLabel.TextAlign = ContentAlignment.MiddleLeft;
            titleLabel.Text = Title;
            titleLabel.Visible = false;

            //
            // Panel
            //
            Controls.Add( titleLabel );
            ResumeLayout( false );
            PerformLayout();
        }
    }
}