#region Header
// Copyright 2007-2011, Jernimo Milea
// 
// This file is part of NetJsWire.
// 
//     NetJsWire is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     NetJsWire is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with NetJsWire.  If not, see <http://www.gnu.org/licenses/>.
#endregion

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Layout;
using NetJsWire.JsonRpcHandler;
using NetJsWire.JsonRpcHandler.Attributes;
using NetJsWire.JsonRpcHandler.Util;
using NetJsWire.Util;
using NetJsWire.Widgets.Config;
using NetJsWire.Widgets.Core;
using NetJsWire.Widgets.Ui.Layout;

namespace NetJsWire.Widgets.Ui.Window
{
    [Renderer( typeof( Renderers.Window.Window ) )]
    [DefaultProperty( "ConfigOptions" )]
    public partial class Window : System.Windows.Forms.Form, IWidget, IWidgetContainer, ITopLevelContainer
    {
        private readonly IRenderer _renderer;
        private WindowConfigOptions _configOptions;
        private bool centered = true;
        private bool collapsible;
        private string iconClass = string.Empty;
        private WindowLayoutEngine layoutEngine;
        //private Layout.LayoutType _layout = Ui.Layout.LayoutType.Absolute;

        //public new void BringToFront()
        //{
        //    Renderer.AddApplicationTailCommand( "show", null );
        //}

        private Window parentWindow;
        private bool resizable = true;

        /// <summary>
        ///   Default empty constructor
        /// </summary>
        public Window()
        {
            InitializeComponent();
            _renderer = Helper.GetRenderer( this );
            Closed += Window_Closed;
        }

        public Window ParentWindow
        {
            get { return parentWindow; }
            set { parentWindow = value; }
        }

        /// <summary>
        ///   Determines if the Window should be rendered centered on its container (defaults to true)
        /// </summary>
        [DefaultValue( true )]
        public bool Centered
        {
            get { return centered; }
            set
            {
                centered = value;
                if ( !Renderer.Rendered && !value )
                {
                    Renderer.SetConstructionProperty( "centered", value );
                }
            }
        }

        [DefaultValue( false )]
        public new bool Modal { get; set; }

        [DefaultValue( true )]
        public bool Resizable
        {
            get { return resizable; }
            set { resizable = value; }
        }

        [DefaultValue( false )]
        public bool Collapsible
        {
            get { return ConfigOptions.Collapsible; }
            set { ConfigOptions.Collapsible = value; }
        }

        [DefaultValue( ( string ) null )]
        public string IconClass
        {
            get { return iconClass; }
            set { iconClass = value; }
        }

        //private ButtonAlign _buttonAlign = ButtonAlign.Right;

        [DefaultValue( ButtonAlign.Right ), RefreshProperties( RefreshProperties.Repaint )]
        public ButtonAlign ButtonAlign
        {
            get { return ConfigOptions.ButtonAlign; }
            set
            {
                ConfigOptions.ButtonAlign = value;
                //_buttonAlign = value;

                //if ( !Renderer.Rendered && value != NetJsWire.ButtonAlign.Right )
                //{
                //    Renderer.SetConstructionProperty( "buttonAlign", value.ToString().ToLower() );
                //}
                if ( DesignMode )
                {
                    PerformLayout();
                }
            }
        }

        public override LayoutEngine LayoutEngine
        {
            get
            {
                if ( layoutEngine == null )
                {
                    layoutEngine = new WindowLayoutEngine();
                }

                return layoutEngine;
            }
        }

        #region Hidden members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new string AccessibleDescription
        {
            get { return base.AccessibleDescription; }
            set { base.AccessibleDescription = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new string AccessibleName
        {
            get { return base.AccessibleName; }
            set { base.AccessibleName = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AccessibleRole AccessibleRole
        {
            get { return base.AccessibleRole; }
            set { base.AccessibleRole = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool AllowDrop
        {
            get { return base.AllowDrop; }
            set { base.AllowDrop = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool AutoScroll
        {
            get { return base.AutoScroll; }
            set { base.AutoScroll = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size AutoScrollMargin
        {
            get { return base.AutoScrollMargin; }
            set { base.AutoScrollMargin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size AutoScrollMinSize
        {
            get { return base.AutoScrollMinSize; }
            set { base.AutoScrollMinSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AutoSizeMode AutoSizeMode
        {
            get { return base.AutoSizeMode; }
            set { base.AutoSizeMode = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Color BackColor
        {
            get { return base.BackColor; }
            set { base.BackColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Image BackgroundImage
        {
            get { return base.BackgroundImage; }
            set { base.BackgroundImage = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ImageLayout BackgroundImageLayout
        {
            get { return base.BackgroundImageLayout; }
            set { base.BackgroundImageLayout = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new FormBorderStyle FormBorderStyle
        {
            get { return base.FormBorderStyle; }
            set { base.FormBorderStyle = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool CausesValidation
        {
            get { return base.CausesValidation; }
            set { base.CausesValidation = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ContextMenuStrip ContextMenuStrip
        {
            get { return base.ContextMenuStrip; }
            set { base.ContextMenuStrip = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Cursor Cursor
        {
            get { return base.Cursor; }
            set { base.Cursor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Font Font
        {
            get { return base.Font; }
            set { base.Font = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Color ForeColor
        {
            get { return base.ForeColor; }
            set { base.ForeColor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new ImeMode ImeMode
        {
            get { return base.ImeMode; }
            set { base.ImeMode = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Padding Margin
        {
            get { return base.Margin; }
            set { base.Margin = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size MinimumSize
        {
            get { return base.MinimumSize; }
            set { base.MinimumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Size MaximumSize
        {
            get { return base.MaximumSize; }
            set { base.MaximumSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Padding Padding
        {
            get { return base.Padding; }
            set { base.Padding = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new RightToLeft RightToLeft
        {
            get { return base.RightToLeft; }
            set { base.RightToLeft = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool TabStop
        {
            get { return base.TabStop; }
            set { base.TabStop = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool UseWaitCursor
        {
            get { return base.UseWaitCursor; }
            set { base.UseWaitCursor = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new IButtonControl AcceptButton
        {
            get { return base.AcceptButton; }
            set { base.AcceptButton = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AutoScaleMode AutoScaleMode
        {
            get { return base.AutoScaleMode; }
            set { base.AutoScaleMode = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool AutoSize
        {
            get { return base.AutoSize; }
            set { base.AutoSize = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new AutoValidate AutoValidate
        {
            get { return base.AutoValidate; }
            set { base.AutoValidate = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new IButtonControl CancelButton
        {
            get { return base.CancelButton; }
            set { base.CancelButton = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool DoubleBuffered
        {
            get { return base.DoubleBuffered; }
            set { base.DoubleBuffered = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool HelpButton
        {
            get { return base.HelpButton; }
            set { base.HelpButton = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool IsMdiContainer
        {
            get { return base.IsMdiContainer; }
            set { base.IsMdiContainer = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool KeyPreview
        {
            get { return base.KeyPreview; }
            set { base.KeyPreview = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new MenuStrip MainMenuStrip
        {
            get { return base.MainMenuStrip; }
            set { base.MainMenuStrip = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new double Opacity
        {
            get { return base.Opacity; }
            set { base.Opacity = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool RightToLeftLayout
        {
            get { return base.RightToLeftLayout; }
            set { base.RightToLeftLayout = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool ShowInTaskbar
        {
            get { return base.ShowInTaskbar; }
            set { base.ShowInTaskbar = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new SizeGripStyle SizeGripStyle
        {
            get { return base.SizeGripStyle; }
            set { base.SizeGripStyle = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new FormStartPosition StartPosition
        {
            get { return base.StartPosition; }
            set { base.StartPosition = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new bool TopMost
        {
            get { return base.TopMost; }
            set { base.TopMost = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new Color TransparencyKey
        {
            get { return base.TransparencyKey; }
            set { base.TransparencyKey = value; }
        }

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Never )]
        public new FormWindowState WindowState
        {
            get { return base.WindowState; }
            set { base.WindowState = value; }
        }
        #endregion

        #region IWidget Members

        [Browsable( false ), EditorBrowsable( EditorBrowsableState.Advanced )]
        public IRenderer Renderer
        {
            get { return _renderer; }
        }

        public void On( string name, string function )
        {
            Renderer.AddClientEvent( name, function );
        }

        public void AddClass( string cls )
        {
            Renderer.AddCommand( "addClass", cls );
        }

        public void RemoveClass( string cls )
        {
            Renderer.AddCommand( "removeClass", cls );
        }

        [DefaultValue( false )]
        [Description( "If the framework should generate a global Javascript variable that points to the Widget, the name of the variable will be the same as ClientID property of the Renderer." )]
        public bool GenerateClientMember { get; set; }

        [RefreshProperties( RefreshProperties.All ),
         DesignerSerializationVisibility( DesignerSerializationVisibility.Content ), Category( "Config" )]
        public IConfigOptions ConfigOptions
        {
            get
            {
                if ( _configOptions == null )
                {
                    _configOptions = new WindowConfigOptions( this ) { Layout = LayoutType.Absolute };
                }
                return _configOptions;
            }
        }

        #endregion

        #region IWidgetContainer Members

        [DefaultValue( LayoutType.Absolute ),
         Description( "The layout used for Widgets inside this Window." ),
         RefreshProperties( RefreshProperties.Repaint )]
        public new LayoutType Layout
        {
            get { return ConfigOptions.Layout; }
            set { ConfigOptions.Layout = value; }
        }

        [Category( "LayoutConfig" )]
        public string LayoutConfig
        {
            get { return ConfigOptions.LayoutConfig; }
            set { ConfigOptions.LayoutConfig = value; }
        }

        #endregion

        public new event EventHandler Closed;
        public new event EventHandler Shown;

        public new void Show()
        {
            if ( !( this is MessageBox.MessageBox ) )
                Renderer.AddApplicationTailCommand( "show", null );
        }

        public void Show( Window parent )
        {
            if ( !( this is MessageBox.MessageBox ) )
            {
                parentWindow = parent;
                Renderer.SetConstructionProperty( "renderTo", parent.Renderer.ClientID + ".body" );
                Renderer.AddApplicationTailCommand( "show", null );
            }
        }

        public void Show( IWidget parent )
        {
            if ( !( this is MessageBox.MessageBox ) )
            {
                Renderer.AddApplicationTailCommand( "show", parent.Renderer.GetCmp + ".el" );
            }
        }

        public void ShowDialog( IWidget parent )
        {
            if ( !( this is MessageBox.MessageBox ) )
            {
                Renderer.AddApplicationTailCommand( "show", parent.Renderer.GetCmp + ".el" );
            }
        }

        public new void ShowDialog()
        {
            if ( !( this is MessageBox.MessageBox ) )
            {
                Modal = true;
                Renderer.AddApplicationTailCommand( "show", null );
            }
        }

        public void ShowDialog( Window parent )
        {
            if ( !( this is MessageBox.MessageBox ) )
            {
                Modal = true;
                parentWindow = parent;
                Renderer.SetConstructionProperty( "renderTo", parent.Renderer.ClientID + ".body" );
                Renderer.AddApplicationTailCommand( "show", null );
            }
        }

        private void Window_Closed( object sender, EventArgs e )
        {
            Renderer.AddApplicationTailDirectEqualCommand( "var tmp", "delete " + Renderer.ClientID );
            base.Close();
            Dispose( true );
        }

        public new void Close()
        {
            Renderer.AddCommand( "close", null );
        }
    }
}